@@ -4,11 +4,13 @@ import numbers
 from typing import (
     TYPE_CHECKING,
     Any,
-    Callable,
+    Self,
 )
 
 import numpy as np
 
+from pandas._config import is_nan_na
+
 from pandas._libs import (
     lib,
     missing as libmissing,
@@ -28,17 +30,20 @@ from pandas.core.arrays.masked import (
 )
 
 if TYPE_CHECKING:
-    from collections.abc import Mapping
+    from collections.abc import (
+        Callable,
+        Mapping,
+    )
 
     import pyarrow
 
     from pandas._typing import (
-        Dtype,
         DtypeObj,
-        Self,
         npt,
     )
 
+    from pandas.core.dtypes.dtypes import ExtensionDtype
+
 
 class NumericDtype(BaseMaskedDtype):
     _default_np_dtype: np.dtype
@@ -90,14 +95,11 @@ class NumericDtype(BaseMaskedDtype):
             array = array.cast(pyarrow_type)
 
         if isinstance(array, pyarrow.ChunkedArray):
-            # TODO this "if" can be removed when requiring pyarrow >= 10.0, which fixed
-            # combine_chunks for empty arrays https://github.com/apache/arrow/pull/13757
-            if array.num_chunks == 0:
-                array = pyarrow.array([], type=array.type)
-            else:
-                array = array.combine_chunks()
+            array = array.combine_chunks()
 
         data, mask = pyarrow_array_to_numpy_and_mask(array, dtype=self.numpy_dtype)
+        if data.dtype.kind == "f" and is_nan_na():
+            mask[np.isnan(data)] = False
         return array_class(data.copy(), ~mask, copy=False)
 
     @classmethod
@@ -132,9 +134,11 @@ class NumericDtype(BaseMaskedDtype):
         raise AbstractMethodError(cls)
 
 
-def _coerce_to_data_and_mask(values, mask, dtype, copy, dtype_cls, default_dtype):
+def _coerce_to_data_and_mask(values, dtype, copy: bool, dtype_cls: type[NumericDtype]):
     checker = dtype_cls._checker
+    default_dtype = dtype_cls._default_np_dtype
 
+    mask = None
     inferred_type = None
 
     if dtype is None and hasattr(values, "dtype"):
@@ -144,7 +148,7 @@ def _coerce_to_data_and_mask(values, mask, dtype, copy, dtype_cls, default_dtype
     if dtype is not None:
         dtype = dtype_cls._standardize_dtype(dtype)
 
-    cls = dtype_cls.construct_array_type()
+    cls = dtype_cls().construct_array_type()
     if isinstance(values, cls):
         values, mask = values._data, values._mask
         if dtype is not None:
@@ -153,19 +157,28 @@ def _coerce_to_data_and_mask(values, mask, dtype, copy, dtype_cls, default_dtype
         if copy:
             values = values.copy()
             mask = mask.copy()
-        return values, mask, dtype, inferred_type
+        return values, mask
 
     original = values
-    values = np.array(values, copy=copy)
+    if not copy:
+        values = np.asarray(values)
+    else:
+        values = np.array(values, copy=copy)
     inferred_type = None
     if values.dtype == object or is_string_dtype(values.dtype):
         inferred_type = lib.infer_dtype(values, skipna=True)
         if inferred_type == "boolean" and dtype is None:
+            # object dtype array of bools
             name = dtype_cls.__name__.strip("_")
             raise TypeError(f"{values.dtype} cannot be converted to {name}")
 
     elif values.dtype.kind == "b" and checker(dtype):
-        values = np.array(values, dtype=default_dtype, copy=copy)
+        # fastpath
+        mask = np.zeros(len(values), dtype=np.bool_)
+        if not copy:
+            values = np.asarray(values, dtype=default_dtype)
+        else:
+            values = np.array(values, dtype=default_dtype, copy=copy)
 
     elif values.dtype.kind not in "iuf":
         name = dtype_cls.__name__.strip("_")
@@ -178,8 +191,23 @@ def _coerce_to_data_and_mask(values, mask, dtype, copy, dtype_cls, default_dtype
         if values.dtype.kind in "iu":
             # fastpath
             mask = np.zeros(len(values), dtype=np.bool_)
-        else:
+        elif values.dtype.kind == "f":
+            # np.isnan is faster than is_numeric_na() for floats
+            # github issue: #60066
+            if is_nan_na():
+                mask = np.isnan(values)
+            else:
+                mask = np.zeros(len(values), dtype=np.bool_)
+                if dtype_cls.__name__.strip("_").startswith(("I", "U")):
+                    wrong = np.isnan(values)
+                    if wrong.any():
+                        raise ValueError("Cannot cast NaN value to Integer dtype.")
+        elif is_nan_na():
             mask = libmissing.is_numeric_na(values)
+        else:
+            # is_numeric_na will raise on non-numeric NAs
+            libmissing.is_numeric_na(values)
+            mask = libmissing.is_pdna_or_none(values)
     else:
         assert len(mask) == len(values)
 
@@ -190,7 +218,7 @@ def _coerce_to_data_and_mask(values, mask, dtype, copy, dtype_cls, default_dtype
     if dtype is None:
         dtype = default_dtype
     else:
-        dtype = dtype.type
+        dtype = dtype.numpy_dtype
 
     if is_integer_dtype(dtype) and values.dtype.kind == "f" and len(values) > 0:
         if mask.all():
@@ -204,22 +232,21 @@ def _coerce_to_data_and_mask(values, mask, dtype, copy, dtype_cls, default_dtype
                     inferred_type not in ["floating", "mixed-integer-float"]
                     and not mask.any()
                 ):
-                    values = np.array(original, dtype=dtype, copy=False)
+                    values = np.asarray(original, dtype=dtype)
                 else:
-                    values = np.array(original, dtype="object", copy=False)
+                    values = np.asarray(original, dtype="object")
 
     # we copy as need to coerce here
     if mask.any():
         values = values.copy()
-        values[mask] = cls._internal_fill_value
+        values[mask] = dtype_cls._internal_fill_value
     if inferred_type in ("string", "unicode"):
         # casts from str are always safe since they raise
         # a ValueError if the str cannot be parsed into a float
         values = values.astype(dtype, copy=copy)
     else:
         values = dtype_cls._safe_cast(values, dtype, copy=False)
-
-    return values, mask, dtype, inferred_type
+    return values, mask
 
 
 class NumericArray(BaseMaskedArray):
@@ -247,6 +274,10 @@ class NumericArray(BaseMaskedArray):
             # If we don't raise here, then accessing self.dtype would raise
             raise TypeError("FloatingArray does not support np.float16 dtype.")
 
+        # NB: if is_nan_na() is True
+        #  then caller is responsible for ensuring
+        #  assert mask[np.isnan(values)].all()
+
         super().__init__(values, mask, copy=copy)
 
     @cache_readonly
@@ -259,16 +290,12 @@ class NumericArray(BaseMaskedArray):
         cls, value, *, dtype: DtypeObj, copy: bool = False
     ) -> tuple[np.ndarray, np.ndarray]:
         dtype_cls = cls._dtype_cls
-        default_dtype = dtype_cls._default_np_dtype
-        mask = None
-        values, mask, _, _ = _coerce_to_data_and_mask(
-            value, mask, dtype, copy, dtype_cls, default_dtype
-        )
+        values, mask = _coerce_to_data_and_mask(value, dtype, copy, dtype_cls)
         return values, mask
 
     @classmethod
     def _from_sequence_of_strings(
-        cls, strings, *, dtype: Dtype | None = None, copy: bool = False
+        cls, strings, *, dtype: ExtensionDtype, copy: bool = False
     ) -> Self:
         from pandas.core.tools.numeric import to_numeric
 
