@@ -2,36 +2,35 @@
 Provide user facing operators for doing the split part of the
 split-apply-combine paradigm.
 """
+
 from __future__ import annotations
 
-import inspect
 from typing import (
     TYPE_CHECKING,
-    Any,
-    Hashable,
     final,
 )
-import warnings
 
 import numpy as np
 
-from pandas._typing import (
-    ArrayLike,
-    NDFrameT,
-    npt,
+from pandas._libs import (
+    algos as libalgos,
 )
+from pandas._libs.tslibs import OutOfBoundsDatetime
 from pandas.errors import InvalidIndexError
-from pandas.util._decorators import cache_readonly
-from pandas.util._exceptions import find_stack_level
+from pandas.util._decorators import (
+    cache_readonly,
+    set_module,
+)
 
-from pandas.core.dtypes.cast import sanitize_to_nanoseconds
 from pandas.core.dtypes.common import (
-    is_categorical_dtype,
+    ensure_int64,
+    ensure_platform_int,
     is_list_like,
     is_scalar,
 )
+from pandas.core.dtypes.dtypes import CategoricalDtype
 
-import pandas.core.algorithms as algorithms
+from pandas.core import algorithms
 from pandas.core.arrays import (
     Categorical,
     ExtensionArray,
@@ -39,36 +38,55 @@ from pandas.core.arrays import (
 import pandas.core.common as com
 from pandas.core.frame import DataFrame
 from pandas.core.groupby import ops
-from pandas.core.groupby.categorical import (
-    recode_for_groupby,
-    recode_from_groupby,
-)
+from pandas.core.groupby.categorical import recode_for_groupby
 from pandas.core.indexes.api import (
-    CategoricalIndex,
     Index,
     MultiIndex,
+    default_index,
 )
 from pandas.core.series import Series
 
-from pandas.io.formats.printing import pprint_thing
+from pandas.io.formats.printing import (
+    PrettyDict,
+    pprint_thing,
+)
 
 if TYPE_CHECKING:
+    from collections.abc import (
+        Hashable,
+        Iterator,
+    )
+
+    from pandas._typing import (
+        ArrayLike,
+        NDFrameT,
+        npt,
+    )
+
     from pandas.core.generic import NDFrame
 
 
+@set_module("pandas")
 class Grouper:
     """
     A Grouper allows the user to specify a groupby instruction for an object.
 
     This specification will select a column via the key parameter, or if the
-    level and/or axis parameters are given, a level of the index of the target
+    level parameter is given, a level of the index of the target
     object.
 
-    If `axis` and/or `level` are passed as keywords to both `Grouper` and
+    If ``level`` is passed as a keyword to both `Grouper` and
     `groupby`, the values passed to `Grouper` take precedence.
 
     Parameters
     ----------
+    *args
+        Currently unused, reserved for future use.
+    **kwargs
+        Dictionary of the keyword arguments to pass to Grouper.
+
+    Attributes
+    ----------
     key : str, defaults to None
         Groupby key, which selects the grouping column of the target.
     level : name/number, defaults to None
@@ -76,10 +94,7 @@ class Grouper:
     freq : str / frequency object, defaults to None
         This will groupby the specified frequency if the target selection
         (via key or level) is a datetime-like object. For full specification
-        of available frequencies, please see `here
-        <https://pandas.pydata.org/pandas-docs/stable/user_guide/timeseries.html#offset-aliases>`_.
-    axis : str, int, defaults to 0
-        Number/name of the axis.
+        of available frequencies, please see :ref:`here<timeseries.offset_aliases>`.
     sort : bool, default to False
         Whether to sort the resulting labels.
     closed : {'left' or 'right'}
@@ -89,23 +104,6 @@ class Grouper:
         Only when `freq` parameter is passed.
     convention : {'start', 'end', 'e', 's'}
         If grouper is PeriodIndex and `freq` parameter is passed.
-    base : int, default 0
-        Only when `freq` parameter is passed.
-        For frequencies that evenly subdivide 1 day, the "origin" of the
-        aggregated intervals. For example, for '5min' frequency, base could
-        range from 0 through 4. Defaults to 0.
-
-        .. deprecated:: 1.1.0
-            The new arguments that you should use are 'offset' or 'origin'.
-
-    loffset : str, DateOffset, timedelta object
-        Only when `freq` parameter is passed.
-
-        .. deprecated:: 1.1.0
-            loffset is only working for ``.resample(...)`` and not for
-            Grouper (:issue:`28302`).
-            However, loffset is also deprecated for ``.resample(...)``
-            See: :class:`DataFrame.resample`
 
     origin : Timestamp or str, default 'start_day'
         The timestamp on which to adjust the grouping. The timezone of origin must
@@ -116,32 +114,31 @@ class Grouper:
         - 'start': `origin` is the first value of the timeseries
         - 'start_day': `origin` is the first day at midnight of the timeseries
 
-        .. versionadded:: 1.1.0
-
         - 'end': `origin` is the last value of the timeseries
         - 'end_day': `origin` is the ceiling midnight of the last day
 
-        .. versionadded:: 1.3.0
-
     offset : Timedelta or str, default is None
         An offset timedelta added to the origin.
 
-        .. versionadded:: 1.1.0
-
     dropna : bool, default True
         If True, and if group keys contain NA values, NA values together with
         row/column will be dropped. If False, NA values will also be treated as
         the key in groups.
 
-        .. versionadded:: 1.2.0
-
     Returns
     -------
-    A specification for a groupby instruction
+    Grouper or pandas.api.typing.TimeGrouper
+        A TimeGrouper is returned if ``freq`` is not ``None``. Otherwise, a Grouper
+        is returned.
+
+    See Also
+    --------
+    Series.groupby : Apply a function groupby to a Series.
+    DataFrame.groupby : Apply a function groupby.
 
     Examples
     --------
-    Syntactic sugar for ``df.groupby('A')``
+    ``df.groupby(pd.Grouper(key="Animal"))`` is equivalent to ``df.groupby('Animal')``
 
     >>> df = pd.DataFrame(
     ...     {
@@ -165,15 +162,15 @@ class Grouper:
     Specify a resample operation on the column 'Publish date'
 
     >>> df = pd.DataFrame(
-    ...    {
-    ...        "Publish date": [
+    ...     {
+    ...         "Publish date": [
     ...             pd.Timestamp("2000-01-02"),
     ...             pd.Timestamp("2000-01-02"),
     ...             pd.Timestamp("2000-01-09"),
-    ...             pd.Timestamp("2000-01-16")
+    ...             pd.Timestamp("2000-01-16"),
     ...         ],
     ...         "ID": [0, 1, 2, 3],
-    ...         "Price": [10, 20, 30, 40]
+    ...         "Price": [10, 20, 30, 40],
     ...     }
     ... )
     >>> df
@@ -191,8 +188,8 @@ class Grouper:
 
     If you want to adjust the start of the bins based on a fixed timestamp:
 
-    >>> start, end = '2000-10-01 23:30:00', '2000-10-02 00:30:00'
-    >>> rng = pd.date_range(start, end, freq='7min')
+    >>> start, end = "2000-10-01 23:30:00", "2000-10-02 00:30:00"
+    >>> rng = pd.date_range(start, end, freq="7min")
     >>> ts = pd.Series(np.arange(len(rng)) * 3, index=rng)
     >>> ts
     2000-10-01 23:30:00     0
@@ -204,73 +201,70 @@ class Grouper:
     2000-10-02 00:12:00    18
     2000-10-02 00:19:00    21
     2000-10-02 00:26:00    24
-    Freq: 7T, dtype: int64
+    Freq: 7min, dtype: int64
 
-    >>> ts.groupby(pd.Grouper(freq='17min')).sum()
+    >>> ts.groupby(pd.Grouper(freq="17min")).sum()
     2000-10-01 23:14:00     0
     2000-10-01 23:31:00     9
     2000-10-01 23:48:00    21
     2000-10-02 00:05:00    54
     2000-10-02 00:22:00    24
-    Freq: 17T, dtype: int64
+    Freq: 17min, dtype: int64
 
-    >>> ts.groupby(pd.Grouper(freq='17min', origin='epoch')).sum()
+    >>> ts.groupby(pd.Grouper(freq="17min", origin="epoch")).sum()
     2000-10-01 23:18:00     0
     2000-10-01 23:35:00    18
     2000-10-01 23:52:00    27
     2000-10-02 00:09:00    39
     2000-10-02 00:26:00    24
-    Freq: 17T, dtype: int64
+    Freq: 17min, dtype: int64
 
-    >>> ts.groupby(pd.Grouper(freq='17min', origin='2000-01-01')).sum()
+    >>> ts.groupby(pd.Grouper(freq="17min", origin="2000-01-01")).sum()
     2000-10-01 23:24:00     3
     2000-10-01 23:41:00    15
     2000-10-01 23:58:00    45
     2000-10-02 00:15:00    45
-    Freq: 17T, dtype: int64
+    Freq: 17min, dtype: int64
 
     If you want to adjust the start of the bins with an `offset` Timedelta, the two
     following lines are equivalent:
 
-    >>> ts.groupby(pd.Grouper(freq='17min', origin='start')).sum()
+    >>> ts.groupby(pd.Grouper(freq="17min", origin="start")).sum()
     2000-10-01 23:30:00     9
     2000-10-01 23:47:00    21
     2000-10-02 00:04:00    54
     2000-10-02 00:21:00    24
-    Freq: 17T, dtype: int64
+    Freq: 17min, dtype: int64
 
-    >>> ts.groupby(pd.Grouper(freq='17min', offset='23h30min')).sum()
+    >>> ts.groupby(pd.Grouper(freq="17min", offset="23h30min")).sum()
     2000-10-01 23:30:00     9
     2000-10-01 23:47:00    21
     2000-10-02 00:04:00    54
     2000-10-02 00:21:00    24
-    Freq: 17T, dtype: int64
+    Freq: 17min, dtype: int64
 
     To replace the use of the deprecated `base` argument, you can now use `offset`,
     in this example it is equivalent to have `base=2`:
 
-    >>> ts.groupby(pd.Grouper(freq='17min', offset='2min')).sum()
+    >>> ts.groupby(pd.Grouper(freq="17min", offset="2min")).sum()
     2000-10-01 23:16:00     0
     2000-10-01 23:33:00     9
     2000-10-01 23:50:00    36
     2000-10-02 00:07:00    39
     2000-10-02 00:24:00    24
-    Freq: 17T, dtype: int64
+    Freq: 17min, dtype: int64
     """
 
-    axis: int
     sort: bool
     dropna: bool
-    _gpr_index: Index | None
     _grouper: Index | None
 
-    _attributes: tuple[str, ...] = ("key", "level", "freq", "axis", "sort", "dropna")
+    _attributes: tuple[str, ...] = ("key", "level", "freq", "sort", "dropna")
 
     def __new__(cls, *args, **kwargs):
         if kwargs.get("freq") is not None:
             from pandas.core.resample import TimeGrouper
 
-            _check_deprecated_resample_kwargs(kwargs, origin=cls)
             cls = TimeGrouper
         return super().__new__(cls)
 
@@ -279,68 +273,54 @@ class Grouper:
         key=None,
         level=None,
         freq=None,
-        axis: int = 0,
         sort: bool = False,
         dropna: bool = True,
     ) -> None:
         self.key = key
         self.level = level
         self.freq = freq
-        self.axis = axis
         self.sort = sort
         self.dropna = dropna
 
-        self.grouper = None
-        self._gpr_index = None
-        self.obj = None
-        self.indexer = None
+        self._indexer_deprecated: npt.NDArray[np.intp] | None = None
         self.binner = None
         self._grouper = None
-        self._indexer = None
-
-    @final
-    @property
-    def ax(self) -> Index:
-        index = self._gpr_index
-        if index is None:
-            raise ValueError("_set_grouper must be called before ax is accessed")
-        return index
+        self._indexer: npt.NDArray[np.intp] | None = None
 
     def _get_grouper(
-        self, obj: NDFrameT, validate: bool = True
-    ) -> tuple[Any, ops.BaseGrouper, NDFrameT]:
+        self, obj: NDFrameT, validate: bool = True, observed: bool = True
+    ) -> tuple[ops.BaseGrouper, NDFrameT]:
         """
         Parameters
         ----------
         obj : Series or DataFrame
+            Object being grouped.
         validate : bool, default True
-            if True, validate the grouper
+            If True, validate the grouper.
+        observed : bool, default True
+            Whether only observed groups should be in the result. Only
+            has an impact when grouping on categorical data.
 
         Returns
         -------
-        a tuple of binner, grouper, obj (possibly sorted)
+        A tuple of grouper, obj (possibly sorted)
         """
-        self._set_grouper(obj)
-        # error: Value of type variable "NDFrameT" of "get_grouper" cannot be
-        # "Optional[Any]"
-        # error: Incompatible types in assignment (expression has type "BaseGrouper",
-        # variable has type "None")
-        self.grouper, _, self.obj = get_grouper(  # type: ignore[type-var,assignment]
-            self.obj,
+        obj, _, _ = self._set_grouper(obj)
+        grouper, _, obj = get_grouper(
+            obj,
             [self.key],
-            axis=self.axis,
             level=self.level,
             sort=self.sort,
             validate=validate,
             dropna=self.dropna,
+            observed=observed,
         )
 
-        # error: Incompatible return value type (got "Tuple[None, None, None]",
-        # expected "Tuple[Any, BaseGrouper, NDFrameT]")
-        return self.binner, self.grouper, self.obj  # type: ignore[return-value]
+        return grouper, obj
 
-    @final
-    def _set_grouper(self, obj: NDFrame, sort: bool = False) -> None:
+    def _set_grouper(
+        self, obj: NDFrameT, sort: bool = False, *, gpr_index: Index | None = None
+    ) -> tuple[NDFrameT, Index, npt.NDArray[np.intp] | None]:
         """
         given an object and the specifications, setup the internal grouper
         for this particular specification
@@ -350,25 +330,30 @@ class Grouper:
         obj : Series or DataFrame
         sort : bool, default False
             whether the resulting grouper should be sorted
+        gpr_index : Index or None, default None
+
+        Returns
+        -------
+        NDFrame
+        Index
+        np.ndarray[np.intp] | None
         """
         assert obj is not None
 
         if self.key is not None and self.level is not None:
             raise ValueError("The Grouper cannot specify both a key and a level!")
 
-        # Keep self.grouper value before overriding
+        # Keep self._grouper value before overriding
         if self._grouper is None:
             # TODO: What are we assuming about subsequent calls?
-            self._grouper = self._gpr_index
-            self._indexer = self.indexer
+            self._grouper = gpr_index
+            self._indexer = self._indexer_deprecated
 
         # the key must be a valid info item
         if self.key is not None:
             key = self.key
             # The 'on' is already defined
-            if getattr(self._gpr_index, "name", None) == key and isinstance(
-                obj, Series
-            ):
+            if getattr(gpr_index, "name", None) == key and isinstance(obj, Series):
                 # Sometimes self._grouper will have been resorted while
                 # obj has not. In this case there is a mismatch when we
                 # call self._grouper.take(obj.index) so we need to undo the sorting
@@ -386,7 +371,7 @@ class Grouper:
                 ax = Index(obj[key], name=key)
 
         else:
-            ax = obj._get_axis(self.axis)
+            ax = obj.index
             if self.level is not None:
                 level = self.level
 
@@ -396,35 +381,26 @@ class Grouper:
                     level = ax._get_level_number(level)
                     ax = Index(ax._get_level_values(level), name=ax.names[level])
 
-                else:
-                    if level not in (0, ax.name):
-                        raise ValueError(f"The level {level} is not valid")
+                elif level not in (0, ax.name):
+                    raise ValueError(f"The level {level} is not valid")
 
         # possibly sort
+        indexer: npt.NDArray[np.intp] | None = None
         if (self.sort or sort) and not ax.is_monotonic_increasing:
             # use stable sort to support first, last, nth
             # TODO: why does putting na_position="first" fix datetimelike cases?
-            indexer = self.indexer = ax.array.argsort(
+            indexer = self._indexer_deprecated = ax.array.argsort(
                 kind="mergesort", na_position="first"
             )
             ax = ax.take(indexer)
-            obj = obj.take(indexer, axis=self.axis)
+            obj = obj.take(indexer, axis=0)
 
-        # error: Incompatible types in assignment (expression has type
-        # "NDFrameT", variable has type "None")
-        self.obj = obj  # type: ignore[assignment]
-        self._gpr_index = ax
-
-    @final
-    @property
-    def groups(self):
-        # error: "None" has no attribute "groups"
-        return self.grouper.groups  # type: ignore[attr-defined]
+        return obj, ax, indexer
 
     @final
     def __repr__(self) -> str:
         attrs_list = (
-            f"{attr_name}={repr(getattr(self, attr_name))}"
+            f"{attr_name}={getattr(self, attr_name)!r}"
             for attr_name in self._attributes
             if getattr(self, attr_name) is not None
         )
@@ -449,20 +425,26 @@ class Grouping:
         If we are a Categorical, use the observed values
     in_axis : if the Grouping is a column in self.obj and hence among
         Groupby.exclusions list
+    dropna : bool, default True
+        Whether to drop NA groups.
+    uniques : Array-like, optional
+        When specified, will be used for unique values. Enables including empty groups
+        in the result for a BinGrouper. Must not contain duplicates.
 
-    Returns
+    Attributes
     -------
-    **Attributes**:
-      * indices : dict of {group -> index_list}
-      * codes : ndarray, group codes
-      * group_index : unique groups
-      * groups : dict of {group -> label_list}
+    indices : dict
+        Mapping of {group -> index_list}
+    codes : ndarray
+        Group codes
+    group_index : Index or None
+        unique groups
+    groups : dict
+        Mapping of {group -> label_list}
     """
 
     _codes: npt.NDArray[np.signedinteger] | None = None
-    _group_index: Index | None = None
-    _passed_categorical: bool
-    _all_grouper: Categorical | None
+    _orig_cats: Index | None
     _index: Index
 
     def __init__(
@@ -475,98 +457,112 @@ class Grouping:
         observed: bool = False,
         in_axis: bool = False,
         dropna: bool = True,
+        uniques: ArrayLike | None = None,
     ) -> None:
+        if isinstance(grouper, Series):
+            grouper = grouper.copy(deep=False)
         self.level = level
         self._orig_grouper = grouper
-        self.grouping_vector = _convert_grouper(index, grouper)
-        self._all_grouper = None
+        grouping_vector = _convert_grouper(index, grouper)
+        self._orig_cats = None
         self._index = index
         self._sort = sort
         self.obj = obj
         self._observed = observed
         self.in_axis = in_axis
         self._dropna = dropna
-
-        self._passed_categorical = False
+        self._uniques = uniques
 
         # we have a single grouper which may be a myriad of things,
         # some of which are dependent on the passing in level
 
         ilevel = self._ilevel
         if ilevel is not None:
-            mapper = self.grouping_vector
             # In extant tests, the new self.grouping_vector matches
             #  `index.get_level_values(ilevel)` whenever
             #  mapper is None and isinstance(index, MultiIndex)
-            (
-                self.grouping_vector,  # Index
-                self._codes,
-                self._group_index,
-            ) = index._get_grouper_for_level(mapper, level=ilevel, dropna=dropna)
+            if isinstance(index, MultiIndex):
+                index_level = index.get_level_values(ilevel)
+            else:
+                index_level = index
+
+            if grouping_vector is None:
+                grouping_vector = index_level
+            else:
+                mapper = grouping_vector
+                grouping_vector = index_level.map(mapper)
 
         # a passed Grouper like, directly get the grouper in the same way
         # as single grouper groupby, use the group_info to get codes
-        elif isinstance(self.grouping_vector, Grouper):
+        elif isinstance(grouping_vector, Grouper):
             # get the new grouper; we already have disambiguated
             # what key/level refer to exactly, don't need to
             # check again as we have by this point converted these
             # to an actual value (rather than a pd.Grouper)
             assert self.obj is not None  # for mypy
-            _, newgrouper, newobj = self.grouping_vector._get_grouper(
-                self.obj, validate=False
-            )
+            newgrouper, newobj = grouping_vector._get_grouper(self.obj, validate=False)
             self.obj = newobj
 
-            ng = newgrouper._get_grouper()
             if isinstance(newgrouper, ops.BinGrouper):
-                # in this case we have `ng is newgrouper`
-                self.grouping_vector = ng
+                # TODO: can we unwrap this and get a tighter typing
+                #  for self.grouping_vector?
+                grouping_vector = newgrouper
             else:
                 # ops.BaseGrouper
+                # TODO: 2023-02-03 no test cases with len(newgrouper.groupings) > 1.
+                #  If that were to occur, would we be throwing out information?
+                # error: Cannot determine type of "grouping_vector"  [has-type]
+                ng = newgrouper.groupings[0].grouping_vector  # type: ignore[has-type]
                 # use Index instead of ndarray so we can recover the name
-                self.grouping_vector = Index(ng, name=newgrouper.result_index.name)
-
-        elif is_categorical_dtype(self.grouping_vector):
-            # a passed Categorical
-            self._passed_categorical = True
-
-            self.grouping_vector, self._all_grouper = recode_for_groupby(
-                self.grouping_vector, sort, observed
-            )
+                grouping_vector = Index(
+                    ng, name=newgrouper.result_index.name, copy=False
+                )
 
         elif not isinstance(
-            self.grouping_vector, (Series, Index, ExtensionArray, np.ndarray)
+            grouping_vector, (Series, Index, ExtensionArray, np.ndarray)
         ):
             # no level passed
-            if getattr(self.grouping_vector, "ndim", 1) != 1:
-                t = self.name or str(type(self.grouping_vector))
+            if getattr(grouping_vector, "ndim", 1) != 1:
+                t = str(type(grouping_vector))
                 raise ValueError(f"Grouper for '{t}' not 1-dimensional")
 
-            self.grouping_vector = index.map(self.grouping_vector)
+            grouping_vector = index.map(grouping_vector)
 
             if not (
-                hasattr(self.grouping_vector, "__len__")
-                and len(self.grouping_vector) == len(index)
+                hasattr(grouping_vector, "__len__")
+                and len(grouping_vector) == len(index)
             ):
-                grper = pprint_thing(self.grouping_vector)
+                grper = pprint_thing(grouping_vector)
                 errmsg = (
-                    "Grouper result violates len(labels) == "
-                    f"len(data)\nresult: {grper}"
+                    f"Grouper result violates len(labels) == len(data)\nresult: {grper}"
                 )
-                self.grouping_vector = None  # Try for sanity
                 raise AssertionError(errmsg)
 
-        if isinstance(self.grouping_vector, np.ndarray):
-            # if we have a date/time-like grouper, make sure that we have
-            # Timestamps like
-            self.grouping_vector = sanitize_to_nanoseconds(self.grouping_vector)
+        if isinstance(grouping_vector, np.ndarray):
+            if grouping_vector.dtype.kind in "mM":
+                # if we have a date/time-like grouper, make sure that we have
+                # Timestamps like
+                # TODO 2022-10-08 we only have one test that gets here and
+                #  values are already in nanoseconds in that case.
+                grouping_vector = Series(grouping_vector).to_numpy()
+        elif isinstance(getattr(grouping_vector, "dtype", None), CategoricalDtype):
+            # a passed Categorical
+            self._orig_cats = grouping_vector.categories
+            grouping_vector = recode_for_groupby(grouping_vector, sort, observed)
+
+        self.grouping_vector = grouping_vector
 
     def __repr__(self) -> str:
         return f"Grouping({self.name})"
 
-    def __iter__(self):
+    def __iter__(self) -> Iterator:
         return iter(self.indices)
 
+    @cache_readonly
+    def _passed_categorical(self) -> bool:
+        dtype = getattr(self.grouping_vector, "dtype", None)
+        return isinstance(dtype, CategoricalDtype)
+
     @cache_readonly
     def name(self) -> Hashable:
         ilevel = self._ilevel
@@ -602,7 +598,7 @@ class Grouping:
 
     @property
     def ngroups(self) -> int:
-        return len(self.group_index)
+        return len(self.uniques)
 
     @cache_readonly
     def indices(self) -> dict[Hashable, npt.NDArray[np.intp]]:
@@ -615,50 +611,16 @@ class Grouping:
 
     @property
     def codes(self) -> npt.NDArray[np.signedinteger]:
-        if self._codes is not None:
-            # _codes is set in __init__ for MultiIndex cases
-            return self._codes
-
         return self._codes_and_uniques[0]
 
-    @cache_readonly
-    def group_arraylike(self) -> ArrayLike:
-        """
-        Analogous to result_index, but holding an ArrayLike to ensure
-        we can retain ExtensionDtypes.
-        """
-        if self._group_index is not None:
-            # _group_index is set in __init__ for MultiIndex cases
-            return self._group_index._values
-
-        elif self._all_grouper is not None:
-            # retain dtype for categories, including unobserved ones
-            return self.result_index._values
-
+    @property
+    def uniques(self) -> ArrayLike:
         return self._codes_and_uniques[1]
 
-    @cache_readonly
-    def result_index(self) -> Index:
-        # result_index retains dtype for categories, including unobserved ones,
-        #  which group_index does not
-        if self._all_grouper is not None:
-            group_idx = self.group_index
-            assert isinstance(group_idx, CategoricalIndex)
-            return recode_from_groupby(self._all_grouper, self._sort, group_idx)
-        return self.group_index
-
-    @cache_readonly
-    def group_index(self) -> Index:
-        if self._group_index is not None:
-            # _group_index is set in __init__ for MultiIndex cases
-            return self._group_index
-
-        uniques = self._codes_and_uniques[1]
-        return Index._with_infer(uniques, name=self.name)
-
     @cache_readonly
     def _codes_and_uniques(self) -> tuple[npt.NDArray[np.signedinteger], ArrayLike]:
-        if self._dropna and self._passed_categorical:
+        uniques: ArrayLike
+        if self._passed_categorical:
             # we make a CategoricalIndex out of the cat grouper
             # preserving the categories / ordered attributes;
             # doesn't (yet - GH#46909) handle dropna=False
@@ -668,24 +630,49 @@ class Grouping:
             if self._observed:
                 ucodes = algorithms.unique1d(cat.codes)
                 ucodes = ucodes[ucodes != -1]
-                if self._sort or cat.ordered:
+                if self._sort:
                     ucodes = np.sort(ucodes)
             else:
                 ucodes = np.arange(len(categories))
 
+            has_dropped_na = False
+            if not self._dropna:
+                na_mask = cat.isna()
+                if np.any(na_mask):
+                    has_dropped_na = True
+                    if self._sort:
+                        # NA goes at the end, gets `largest non-NA code + 1`
+                        na_code = len(categories)
+                    else:
+                        # Insert NA in result based on first appearance, need
+                        # the number of unique codes prior
+                        na_idx = na_mask.argmax()
+                        na_code = algorithms.nunique_ints(cat.codes[:na_idx])
+                    ucodes = np.insert(ucodes, na_code, -1)
+
             uniques = Categorical.from_codes(
-                codes=ucodes, categories=categories, ordered=cat.ordered
+                codes=ucodes, categories=categories, ordered=cat.ordered, validate=False
             )
-            return cat.codes, uniques
+            codes = cat.codes
+
+            if has_dropped_na:
+                if not self._sort:
+                    # NA code is based on first appearance, increment higher codes
+                    codes = np.where(codes >= na_code, codes + 1, codes)
+                codes = np.where(na_mask, na_code, codes)
+
+            return codes, uniques
 
         elif isinstance(self.grouping_vector, ops.BaseGrouper):
             # we have a list of groupers
             codes = self.grouping_vector.codes_info
-            # error: Incompatible types in assignment (expression has type "Union
-            # [ExtensionArray, ndarray[Any, Any]]", variable has type "Categorical")
-            uniques = (
-                self.grouping_vector.result_index._values  # type: ignore[assignment]
-            )
+            uniques = self.grouping_vector.result_index._values
+        elif self._uniques is not None:
+            # GH#50486 Code grouping_vector using _uniques; allows
+            # including uniques that are not present in grouping_vector.
+            cat = Categorical(self.grouping_vector, categories=self._uniques)
+            codes = cat.codes
+            uniques = self._uniques
         else:
             # GH35667, replace dropna=False with use_na_sentinel=False
             # error: Incompatible types in assignment (expression has type "Union[
@@ -696,18 +683,47 @@ class Grouping:
         return codes, uniques
 
     @cache_readonly
-    def groups(self) -> dict[Hashable, np.ndarray]:
-        return self._index.groupby(Categorical.from_codes(self.codes, self.group_index))
+    def groups(self) -> dict[Hashable, Index]:
+        codes, uniques = self._codes_and_uniques
+        uniques = Index._with_infer(uniques, name=self.name, copy=False)
+
+        r, counts = libalgos.groupsort_indexer(ensure_platform_int(codes), len(uniques))
+        counts = ensure_int64(counts).cumsum()
+        _result = (r[start:end] for start, end in zip(counts, counts[1:], strict=False))
+        # map to the label
+        result = {k: self._index.take(v) for k, v in zip(uniques, _result, strict=True)}
+
+        return PrettyDict(result)
+
+    @property
+    def observed_grouping(self) -> Grouping:
+        if self._observed:
+            return self
+
+        return self._observed_grouping
+
+    @cache_readonly
+    def _observed_grouping(self) -> Grouping:
+        grouping = Grouping(
+            self._index,
+            self._orig_grouper,
+            obj=self.obj,
+            level=self.level,
+            sort=self._sort,
+            observed=True,
+            in_axis=self.in_axis,
+            dropna=self._dropna,
+            uniques=self._uniques,
+        )
+        return grouping
 
 
 def get_grouper(
     obj: NDFrameT,
     key=None,
-    axis: int = 0,
     level=None,
     sort: bool = True,
     observed: bool = False,
-    mutated: bool = False,
     validate: bool = True,
     dropna: bool = True,
 ) -> tuple[ops.BaseGrouper, frozenset[Hashable], NDFrameT]:
@@ -720,8 +736,8 @@ def get_grouper(
     Groupers are ultimately index mappings. They can originate as:
     index mappings, keys to columns, functions, or Groupers
 
-    Groupers enable local references to axis,level,sort, while
-    the passed in axis, level, and sort are 'global'.
+    Groupers enable local references to level,sort, while
+    the passed in level, and sort are 'global'.
 
     This routine tries to figure out what the passing in references
     are and then creates a Grouping for each one, combined into
@@ -733,10 +749,10 @@ def get_grouper(
     If validate, then check for key/level overlaps.
 
     """
-    group_axis = obj._get_axis(axis)
+    group_axis = obj.index
 
     # validate that the passed single level is compatible with the passed
-    # axis of the object
+    # index of the object
     if level is not None:
         # TODO: These if-block and else-block are almost same.
         # MultiIndex instance check is removable, but it seems that there are
@@ -769,11 +785,8 @@ def get_grouper(
                     raise ValueError("multiple levels only valid with MultiIndex")
 
             if isinstance(level, str):
-                if obj._get_axis(axis).name != level:
-                    raise ValueError(
-                        f"level name {level} is not the name "
-                        f"of the {obj._get_axis_name(axis)}"
-                    )
+                if obj.index.name != level:
+                    raise ValueError(f"level name {level} is not the name of the index")
             elif level > 0 or level < -1:
                 raise ValueError("level > 0 or level < -1 only valid with MultiIndex")
 
@@ -784,7 +797,7 @@ def get_grouper(
 
     # a passed-in Grouper, directly convert
     if isinstance(key, Grouper):
-        binner, grouper, obj = key._get_grouper(obj, validate=False)
+        grouper, obj = key._get_grouper(obj, validate=False, observed=observed)
         if key.key is None:
             return grouper, frozenset(), obj
         else:
@@ -839,7 +852,6 @@ def get_grouper(
 
     # if the actual grouper should be obj[key]
     def is_in_axis(key) -> bool:
-
         if not _is_label_like(key):
             if obj.ndim == 1:
                 return False
@@ -849,7 +861,7 @@ def get_grouper(
             try:
                 items.get_loc(key)
             except (KeyError, TypeError, InvalidIndexError):
-                # TypeError shows up here if we pass e.g. Int64Index
+                # TypeError shows up here if we pass e.g. an Index
                 return False
 
         return True
@@ -858,39 +870,39 @@ def get_grouper(
     def is_in_obj(gpr) -> bool:
         if not hasattr(gpr, "name"):
             return False
+        # We check the references to determine if the
+        # series is part of the object
         try:
-            return gpr is obj[gpr.name]
-        except (KeyError, IndexError, InvalidIndexError):
-            # IndexError reached in e.g. test_skip_group_keys when we pass
-            #  lambda here
-            # InvalidIndexError raised on key-types inappropriate for index,
-            #  e.g. DatetimeIndex.get_loc(tuple())
+            obj_gpr_column = obj[gpr.name]
+        except (KeyError, IndexError, InvalidIndexError, OutOfBoundsDatetime):
             return False
+        if isinstance(gpr, Series) and isinstance(obj_gpr_column, Series):
+            return gpr._mgr.references_same_values(obj_gpr_column._mgr, 0)
+        return False
 
-    for gpr, level in zip(keys, levels):
-
+    for gpr, level in zip(keys, levels, strict=True):
         if is_in_obj(gpr):  # df.groupby(df['name'])
             in_axis = True
             exclusions.add(gpr.name)
 
         elif is_in_axis(gpr):  # df.groupby('name')
-            if gpr in obj:
+            if obj.ndim != 1 and gpr in obj:
                 if validate:
-                    obj._check_label_or_level_ambiguity(gpr, axis=axis)
+                    obj._check_label_or_level_ambiguity(gpr, axis=0)
                 in_axis, name, gpr = True, gpr, obj[gpr]
                 if gpr.ndim != 1:
                     # non-unique columns; raise here to get the name in the
                     # exception message
                     raise ValueError(f"Grouper for '{name}' not 1-dimensional")
                 exclusions.add(name)
-            elif obj._is_level_reference(gpr, axis=axis):
+            elif obj._is_level_reference(gpr, axis=0):
                 in_axis, level, gpr = False, gpr, None
             else:
                 raise KeyError(gpr)
         elif isinstance(gpr, Grouper) and gpr.key is not None:
             # Add key to exclusions
             exclusions.add(gpr.key)
-            in_axis = False
+            in_axis = True
         else:
             in_axis = False
 
@@ -915,13 +927,11 @@ def get_grouper(
 
     if len(groupings) == 0 and len(obj):
         raise ValueError("No group keys passed!")
-    elif len(groupings) == 0:
-        groupings.append(Grouping(Index([], dtype="int"), np.array([], dtype=np.intp)))
+    if len(groupings) == 0:
+        groupings.append(Grouping(default_index(0), np.array([], dtype=np.intp)))
 
     # create the internals grouper
-    grouper = ops.BaseGrouper(
-        group_axis, groupings, sort=sort, mutated=mutated, dropna=dropna
-    )
+    grouper = ops.BaseGrouper(group_axis, groupings, sort=sort, dropna=dropna)
     return grouper, frozenset(exclusions), obj
 
 
@@ -948,51 +958,3 @@ def _convert_grouper(axis: Index, grouper):
         return grouper
     else:
         return grouper
-
-
-def _check_deprecated_resample_kwargs(kwargs, origin):
-    """
-    Check for use of deprecated parameters in ``resample`` and related functions.
-
-    Raises the appropriate warnings if these parameters are detected.
-    Only sets an approximate ``stacklevel`` for the warnings (see #37603, #36629).
-
-    Parameters
-    ----------
-    kwargs : dict
-        Dictionary of keyword arguments to check for deprecated parameters.
-    origin : object
-        From where this function is being called; either Grouper or TimeGrouper. Used
-        to determine an approximate stacklevel.
-    """
-    # Deprecation warning of `base` and `loffset` since v1.1.0:
-    # we are raising the warning here to be able to set the `stacklevel`
-    # properly since we need to raise the `base` and `loffset` deprecation
-    # warning from three different cases:
-    #   core/generic.py::NDFrame.resample
-    #   core/groupby/groupby.py::GroupBy.resample
-    #   core/groupby/grouper.py::Grouper
-    # raising these warnings from TimeGrouper directly would fail the test:
-    #   tests/resample/test_deprecated.py::test_deprecating_on_loffset_and_base
-
-    if kwargs.get("base", None) is not None:
-        warnings.warn(
-            "'base' in .resample() and in Grouper() is deprecated.\n"
-            "The new arguments that you should use are 'offset' or 'origin'.\n"
-            '\n>>> df.resample(freq="3s", base=2)\n'
-            "\nbecomes:\n"
-            '\n>>> df.resample(freq="3s", offset="2s")\n',
-            FutureWarning,
-            stacklevel=find_stack_level(inspect.currentframe()),
-        )
-    if kwargs.get("loffset", None) is not None:
-        warnings.warn(
-            "'loffset' in .resample() and in Grouper() is deprecated.\n"
-            '\n>>> df.resample(freq="3s", loffset="8H")\n'
-            "\nbecomes:\n"
-            "\n>>> from pandas.tseries.frequencies import to_offset"
-            '\n>>> df = df.resample(freq="3s").mean()'
-            '\n>>> df.index = df.index.to_timestamp() + to_offset("8H")\n',
-            FutureWarning,
-            stacklevel=find_stack_level(inspect.currentframe()),
-        )
