@@ -1,10 +1,12 @@
 """
 datetimelike delegation
 """
+
 from __future__ import annotations
 
 from typing import (
     TYPE_CHECKING,
+    NoReturn,
     cast,
 )
 import warnings
@@ -12,6 +14,7 @@ import warnings
 import numpy as np
 
 from pandas._libs import lib
+from pandas.errors import Pandas4Warning
 from pandas.util._exceptions import find_stack_level
 
 from pandas.core.dtypes.common import (
@@ -85,9 +88,7 @@ class Properties(PandasDelegate, PandasObject, NoNewAttributesMixin):
             f"cannot convert an object of type {type(data)} to a datetimelike index"
         )
 
-    # error: Signature of "_delegate_property_get" incompatible with supertype
-    # "PandasDelegate"
-    def _delegate_property_get(self, name: str):  # type: ignore[override]
+    def _delegate_property_get(self, name: str):
         from pandas import Series
 
         values = self._get_values()
@@ -108,18 +109,9 @@ class Properties(PandasDelegate, PandasObject, NoNewAttributesMixin):
         else:
             index = self._parent.index
         # return the result as a Series
-        result = Series(result, index=index, name=self.name).__finalize__(self._parent)
+        return Series(result, index=index, name=self.name).__finalize__(self._parent)
 
-        # setting this object will show a SettingWithCopyWarning/Error
-        result._is_copy = (
-            "modifications to a property of a datetimelike "
-            "object are not supported and are discarded. "
-            "Change values on the original."
-        )
-
-        return result
-
-    def _delegate_property_set(self, name: str, value, *args, **kwargs):
+    def _delegate_property_set(self, name: str, value, *args, **kwargs) -> NoReturn:
         raise ValueError(
             "modifications to a property of a datetimelike object are not supported. "
             "Change values on the original."
@@ -136,20 +128,25 @@ class Properties(PandasDelegate, PandasObject, NoNewAttributesMixin):
         if not is_list_like(result):
             return result
 
-        result = Series(result, index=self._parent.index, name=self.name).__finalize__(
+        return Series(result, index=self._parent.index, name=self.name).__finalize__(
             self._parent
         )
 
-        # setting this object will show a SettingWithCopyWarning/Error
-        result._is_copy = (
-            "modifications to a method of a datetimelike "
-            "object are not supported and are discarded. "
-            "Change values on the original."
-        )
-
-        return result
-
 
+@delegate_names(
+    delegate=ArrowExtensionArray,
+    accessors=TimedeltaArray._datetimelike_ops,
+    typ="property",
+    accessor_mapping=lambda x: f"_dt_{x}",
+    raise_on_missing=False,
+)
+@delegate_names(
+    delegate=ArrowExtensionArray,
+    accessors=TimedeltaArray._datetimelike_methods,
+    typ="method",
+    accessor_mapping=lambda x: f"_dt_{x}",
+    raise_on_missing=False,
+)
 @delegate_names(
     delegate=ArrowExtensionArray,
     accessors=DatetimeArray._datetimelike_ops,
@@ -175,7 +172,7 @@ class ArrowTemporalProperties(PandasDelegate, PandasObject, NoNewAttributesMixin
         self._orig = orig
         self._freeze()
 
-    def _delegate_property_get(self, name: str):  # type: ignore[override]
+    def _delegate_property_get(self, name: str):
         if not hasattr(self._parent.array, f"_dt_{name}"):
             raise NotImplementedError(
                 f"dt.{name} is not supported for {self._parent.dtype}"
@@ -215,19 +212,23 @@ class ArrowTemporalProperties(PandasDelegate, PandasObject, NoNewAttributesMixin
 
         return result
 
-    def to_pydatetime(self):
-        # GH#20306
+    def to_pytimedelta(self):
+        # GH 57463
         warnings.warn(
-            f"The behavior of {type(self).__name__}.to_pydatetime is deprecated, "
+            f"The behavior of {type(self).__name__}.to_pytimedelta is deprecated, "
             "in a future version this will return a Series containing python "
-            "datetime objects instead of an ndarray. To retain the old behavior, "
-            "call `np.array` on the result",
-            FutureWarning,
+            "datetime.timedelta objects instead of an ndarray. To retain the "
+            "old behavior, call `np.array` on the result",
+            Pandas4Warning,
             stacklevel=find_stack_level(),
         )
+        return cast(ArrowExtensionArray, self._parent.array)._dt_to_pytimedelta()
+
+    def to_pydatetime(self) -> Series:
+        # GH#20306
         return cast(ArrowExtensionArray, self._parent.array)._dt_to_pydatetime()
 
-    def isocalendar(self):
+    def isocalendar(self) -> DataFrame:
         from pandas import DataFrame
 
         result = (
@@ -243,15 +244,35 @@ class ArrowTemporalProperties(PandasDelegate, PandasObject, NoNewAttributesMixin
         )
         return iso_calendar_df
 
+    @property
+    def components(self) -> DataFrame:
+        from pandas import DataFrame
+
+        components_df = DataFrame(
+            {
+                col: getattr(self._parent.array, f"_dt_{col}")
+                for col in [
+                    "days",
+                    "hours",
+                    "minutes",
+                    "seconds",
+                    "milliseconds",
+                    "microseconds",
+                    "nanoseconds",
+                ]
+            }
+        )
+        return components_df
+
 
 @delegate_names(
     delegate=DatetimeArray,
-    accessors=DatetimeArray._datetimelike_ops + ["unit"],
+    accessors=[*DatetimeArray._datetimelike_ops, "unit"],
     typ="property",
 )
 @delegate_names(
     delegate=DatetimeArray,
-    accessors=DatetimeArray._datetimelike_methods + ["as_unit"],
+    accessors=[*DatetimeArray._datetimelike_methods, "as_unit"],
     typ="method",
 )
 class DatetimeProperties(Properties):
@@ -265,7 +286,7 @@ class DatetimeProperties(Properties):
     0   2000-01-01 00:00:00
     1   2000-01-01 00:00:01
     2   2000-01-01 00:00:02
-    dtype: datetime64[ns]
+    dtype: datetime64[us]
     >>> seconds_series.dt.second
     0    0
     1    1
@@ -277,19 +298,19 @@ class DatetimeProperties(Properties):
     0   2000-01-01 00:00:00
     1   2000-01-01 01:00:00
     2   2000-01-01 02:00:00
-    dtype: datetime64[ns]
+    dtype: datetime64[us]
     >>> hours_series.dt.hour
     0    0
     1    1
     2    2
     dtype: int32
 
-    >>> quarters_series = pd.Series(pd.date_range("2000-01-01", periods=3, freq="q"))
+    >>> quarters_series = pd.Series(pd.date_range("2000-01-01", periods=3, freq="QE"))
     >>> quarters_series
     0   2000-03-31
     1   2000-06-30
     2   2000-09-30
-    dtype: datetime64[ns]
+    dtype: datetime64[us]
     >>> quarters_series.dt.quarter
     0    1
     1    2
@@ -300,15 +321,9 @@ class DatetimeProperties(Properties):
     Raises TypeError if the Series does not contain datetimelike values.
     """
 
-    def to_pydatetime(self) -> np.ndarray:
+    def to_pydatetime(self) -> Series:
         """
-        Return the data as an array of :class:`datetime.datetime` objects.
-
-        .. deprecated:: 2.1.0
-
-            The current behavior of dt.to_pydatetime is deprecated.
-            In a future version this will return a Series containing python
-            datetime objects instead of a ndarray.
+        Return the data as a Series of :class:`datetime.datetime` objects.
 
         Timezone information is retained if present.
 
@@ -328,41 +343,59 @@ class DatetimeProperties(Properties):
 
         Examples
         --------
-        >>> s = pd.Series(pd.date_range('20180310', periods=2))
+        >>> s = pd.Series(pd.date_range("20180310", periods=2))
         >>> s
         0   2018-03-10
         1   2018-03-11
-        dtype: datetime64[ns]
+        dtype: datetime64[us]
 
         >>> s.dt.to_pydatetime()
-        array([datetime.datetime(2018, 3, 10, 0, 0),
-               datetime.datetime(2018, 3, 11, 0, 0)], dtype=object)
+        0    2018-03-10 00:00:00
+        1    2018-03-11 00:00:00
+        dtype: object
 
         pandas' nanosecond precision is truncated to microseconds.
 
-        >>> s = pd.Series(pd.date_range('20180310', periods=2, freq='ns'))
+        >>> s = pd.Series(pd.date_range("20180310", periods=2, freq="ns"))
         >>> s
         0   2018-03-10 00:00:00.000000000
         1   2018-03-10 00:00:00.000000001
         dtype: datetime64[ns]
 
         >>> s.dt.to_pydatetime()
-        array([datetime.datetime(2018, 3, 10, 0, 0),
-               datetime.datetime(2018, 3, 10, 0, 0)], dtype=object)
+        0    2018-03-10 00:00:00
+        1    2018-03-10 00:00:00
+        dtype: object
         """
         # GH#20306
-        warnings.warn(
-            f"The behavior of {type(self).__name__}.to_pydatetime is deprecated, "
-            "in a future version this will return a Series containing python "
-            "datetime objects instead of an ndarray. To retain the old behavior, "
-            "call `np.array` on the result",
-            FutureWarning,
-            stacklevel=find_stack_level(),
-        )
-        return self._get_values().to_pydatetime()
+        from pandas import Series
+
+        return Series(self._get_values().to_pydatetime(), dtype=object)
 
     @property
     def freq(self):
+        """
+        Tries to return a string representing a frequency generated by infer_freq.
+
+        Returns None if it can't autodetect the frequency.
+
+        See Also
+        --------
+        Series.dt.to_period : Cast to PeriodArray/PeriodIndex at a particular
+            frequency.
+
+        Examples
+        --------
+        >>> ser = pd.Series(["2024-01-01", "2024-01-02", "2024-01-03", "2024-01-04"])
+        >>> ser = pd.to_datetime(ser)
+        >>> ser.dt.freq
+        'D'
+
+        >>> ser = pd.Series(["2022-01-01", "2024-01-01", "2026-01-01", "2028-01-01"])
+        >>> ser = pd.to_datetime(ser)
+        >>> ser.dt.freq
+        '2YS-JAN'
+        """
         return self._get_values().inferred_freq
 
     def isocalendar(self) -> DataFrame:
@@ -414,13 +447,13 @@ class TimedeltaProperties(Properties):
     Examples
     --------
     >>> seconds_series = pd.Series(
-    ...     pd.timedelta_range(start="1 second", periods=3, freq="S")
+    ...     pd.timedelta_range(start="1 second", periods=3, freq="s")
     ... )
     >>> seconds_series
     0   0 days 00:00:01
     1   0 days 00:00:02
     2   0 days 00:00:03
-    dtype: timedelta64[ns]
+    dtype: timedelta64[us]
     >>> seconds_series.dt.seconds
     0    1
     1    2
@@ -449,41 +482,60 @@ class TimedeltaProperties(Properties):
 
         Examples
         --------
-        >>> s = pd.Series(pd.to_timedelta(np.arange(5), unit="d"))
+        >>> s = pd.Series(pd.to_timedelta(np.arange(5), unit="D"))
         >>> s
         0   0 days
         1   1 days
         2   2 days
         3   3 days
         4   4 days
-        dtype: timedelta64[ns]
+        dtype: timedelta64[s]
 
         >>> s.dt.to_pytimedelta()
         array([datetime.timedelta(0), datetime.timedelta(days=1),
         datetime.timedelta(days=2), datetime.timedelta(days=3),
         datetime.timedelta(days=4)], dtype=object)
         """
+        # GH 57463
+        warnings.warn(
+            f"The behavior of {type(self).__name__}.to_pytimedelta is deprecated, "
+            "in a future version this will return a Series containing python "
+            "datetime.timedelta objects instead of an ndarray. To retain the "
+            "old behavior, call `np.array` on the result",
+            Pandas4Warning,
+            stacklevel=find_stack_level(),
+        )
         return self._get_values().to_pytimedelta()
 
     @property
-    def components(self):
+    def components(self) -> DataFrame:
         """
         Return a Dataframe of the components of the Timedeltas.
 
+        Each row of the DataFrame corresponds to a Timedelta in the original
+        Series and contains the individual components (days, hours, minutes,
+        seconds, milliseconds, microseconds, nanoseconds) of the Timedelta.
+
         Returns
         -------
         DataFrame
 
+        See Also
+        --------
+        TimedeltaIndex.components : Return a DataFrame of the individual resolution
+            components of the Timedeltas.
+        Series.dt.total_seconds : Return the total number of seconds in the duration.
+
         Examples
         --------
-        >>> s = pd.Series(pd.to_timedelta(np.arange(5), unit='s'))
+        >>> s = pd.Series(pd.to_timedelta(np.arange(5), unit="s"))
         >>> s
         0   0 days 00:00:00
         1   0 days 00:00:01
         2   0 days 00:00:02
         3   0 days 00:00:03
         4   0 days 00:00:04
-        dtype: timedelta64[ns]
+        dtype: timedelta64[s]
         >>> s.dt.components
            days  hours  minutes  seconds  milliseconds  microseconds  nanoseconds
         0     0      0        0        0             0             0            0
@@ -528,7 +580,7 @@ class PeriodProperties(Properties):
     1    2000-01-01 00:00:01
     2    2000-01-01 00:00:02
     3    2000-01-01 00:00:03
-    dtype: period[S]
+    dtype: period[s]
     >>> seconds_series.dt.second
     0    0
     1    1
@@ -544,7 +596,7 @@ class PeriodProperties(Properties):
     1    2000-01-01 01:00
     2    2000-01-01 02:00
     3    2000-01-01 03:00
-    dtype: period[H]
+    dtype: period[h]
     >>> hours_series.dt.hour
     0    0
     1    1
@@ -573,6 +625,44 @@ class PeriodProperties(Properties):
 class CombinedDatetimelikeProperties(
     DatetimeProperties, TimedeltaProperties, PeriodProperties
 ):
+    """
+    Accessor object for Series values' datetime-like, timedelta and period properties.
+
+    See Also
+    --------
+    DatetimeIndex : Index of datetime64 data.
+
+    Examples
+    --------
+    >>> dates = pd.Series(
+    ...     ["2024-01-01", "2024-01-15", "2024-02-5"], dtype="datetime64[ns]"
+    ... )
+    >>> dates.dt.day
+    0     1
+    1    15
+    2     5
+    dtype: int32
+    >>> dates.dt.month
+    0    1
+    1    1
+    2    2
+    dtype: int32
+
+    >>> dates = pd.Series(
+    ...     ["2024-01-01", "2024-01-15", "2024-02-5"], dtype="datetime64[ns, UTC]"
+    ... )
+    >>> dates.dt.day
+    0     1
+    1    15
+    2     5
+    dtype: int32
+    >>> dates.dt.month
+    0    1
+    1    1
+    2    2
+    dtype: int32
+    """
+
     def __new__(cls, data: Series):  # pyright: ignore[reportInconsistentConstructor]
         # CombinedDatetimelikeProperties isn't really instantiated. Instead
         # we need to choose which parent (datetime or timedelta) is
@@ -594,7 +684,7 @@ class CombinedDatetimelikeProperties(
                 index=orig.index,
             )
 
-        if isinstance(data.dtype, ArrowDtype) and data.dtype.kind == "M":
+        if isinstance(data.dtype, ArrowDtype) and data.dtype.kind in "Mm":
             return ArrowTemporalProperties(data, orig)
         if lib.is_np_dtype(data.dtype, "M"):
             return DatetimeProperties(data, orig)
