@@ -11,7 +11,6 @@ from numpy in the following ways:
        pandas convention is to return [-inf, nan, inf] for all dtype
        combinations.
 
-       Note: the numpy behavior described here is py3-specific.
 
     2) np.array([-1, 0, 1], dtype=dtype1) % np.array([0, 0, 0], dtype=dtype2)
        gives precisely the same results as the // operation.
@@ -21,20 +20,17 @@ from numpy in the following ways:
 
     3) divmod behavior consistent with 1) and 2).
 """
+
+from __future__ import annotations
+
 import operator
 
 import numpy as np
 
-from pandas.core.dtypes.common import (
-    is_float_dtype,
-    is_integer_dtype,
-    is_scalar,
-)
-
-from pandas.core.ops import roperator
+from pandas.core import roperator
 
 
-def _fill_zeros(result, x, y):
+def _fill_zeros(result: np.ndarray, x, y) -> np.ndarray:
     """
     If this is a reversed op, then flip x,y
 
@@ -44,23 +40,22 @@ def _fill_zeros(result, x, y):
 
     Mask the nan's from x.
     """
-    if is_float_dtype(result.dtype):
+    if result.dtype.kind == "f":
         return result
 
     is_variable_type = hasattr(y, "dtype")
-    is_scalar_type = is_scalar(y)
+    is_scalar_type = not isinstance(y, np.ndarray)
 
     if not is_variable_type and not is_scalar_type:
+        # e.g. test_series_ops_name_retention with mod we get here with list/tuple
         return result
 
     if is_scalar_type:
         y = np.array(y)
 
-    if is_integer_dtype(y.dtype):
-
+    if y.dtype.kind in "iu":
         ymask = y == 0
         if ymask.any():
-
             # GH#7325, mask and nans must be broadcastable
             mask = ymask & ~np.isnan(result)
 
@@ -94,9 +89,9 @@ def mask_zero_div_zero(x, y, result: np.ndarray) -> np.ndarray:
     >>> x = np.array([1, 0, -1], dtype=np.int64)
     >>> x
     array([ 1,  0, -1])
-    >>> y = 0       # int 0; numpy behavior is different with float
+    >>> y = 0  # int 0; numpy behavior is different with float
     >>> result = x // y
-    >>> result      # raw numpy result does not fill division by zero
+    >>> result  # raw numpy result does not fill division by zero
     array([0, 0, 0])
     >>> mask_zero_div_zero(x, y, result)
     array([ inf,  nan, -inf])
@@ -112,7 +107,6 @@ def mask_zero_div_zero(x, y, result: np.ndarray) -> np.ndarray:
     zmask = y == 0
 
     if zmask.any():
-
         # Flip sign if necessary for -0.0
         zneg_mask = zmask & np.signbit(y)
         zpos_mask = zmask & ~zneg_mask
@@ -120,9 +114,8 @@ def mask_zero_div_zero(x, y, result: np.ndarray) -> np.ndarray:
         x_lt0 = x < 0
         x_gt0 = x > 0
         nan_mask = zmask & (x == 0)
-        with np.errstate(invalid="ignore"):
-            neginf_mask = (zpos_mask & x_lt0) | (zneg_mask & x_gt0)
-            posinf_mask = (zpos_mask & x_gt0) | (zneg_mask & x_lt0)
+        neginf_mask = (zpos_mask & x_lt0) | (zneg_mask & x_gt0)
+        posinf_mask = (zpos_mask & x_gt0) | (zneg_mask & x_lt0)
 
         if nan_mask.any() or neginf_mask.any() or posinf_mask.any():
             # Fill negative/0 with -inf, positive/0 with +inf, 0/0 with NaN
@@ -144,7 +137,9 @@ def dispatch_fill_zeros(op, left, right, result):
     ----------
     op : function (operator.add, operator.div, ...)
     left : object (np.ndarray for non-reversed ops)
+        We have excluded ExtensionArrays here
     right : object (np.ndarray for reversed ops)
+        We have excluded ExtensionArrays here
     result : ndarray
 
     Returns
@@ -167,11 +162,11 @@ def dispatch_fill_zeros(op, left, right, result):
             _fill_zeros(result[1], right, left),
         )
     elif op is operator.floordiv:
-        # Note: no need to do this for truediv; in py3 numpy behaves the way
+        # Note: no need to do this for truediv; numpy behaves the way
         #  we want.
         result = mask_zero_div_zero(left, right, result)
     elif op is roperator.rfloordiv:
-        # Note: no need to do this for rtruediv; in py3 numpy behaves the way
+        # Note: no need to do this for rtruediv; numpy behaves the wayS
         #  we want.
         result = mask_zero_div_zero(right, left, result)
     elif op is operator.mod:
