@@ -5,22 +5,18 @@ from typing import TYPE_CHECKING
 
 import numpy as np
 
-from pandas.util._decorators import Appender
+from pandas.util._decorators import set_module
 
-from pandas.core.dtypes.common import is_list_like
+from pandas.core.dtypes.common import (
+    is_iterator,
+    is_list_like,
+)
 from pandas.core.dtypes.concat import concat_compat
 from pandas.core.dtypes.missing import notna
 
 import pandas.core.algorithms as algos
-from pandas.core.arrays import Categorical
-import pandas.core.common as com
-from pandas.core.indexes.api import (
-    Index,
-    MultiIndex,
-)
+from pandas.core.indexes.api import MultiIndex
 from pandas.core.reshape.concat import concat
-from pandas.core.reshape.util import tile_compat
-from pandas.core.shared_docs import _shared_docs
 from pandas.core.tools.numeric import to_numeric
 
 if TYPE_CHECKING:
@@ -31,7 +27,21 @@ if TYPE_CHECKING:
     from pandas import DataFrame
 
 
-@Appender(_shared_docs["melt"] % {"caller": "pd.melt(df, ", "other": "DataFrame.melt"})
+def ensure_list_vars(arg_vars, variable: str, columns) -> list:
+    if arg_vars is not None:
+        if not is_list_like(arg_vars):
+            return [arg_vars]
+        elif isinstance(columns, MultiIndex) and not isinstance(arg_vars, list):
+            raise ValueError(
+                f"{variable} must be a list of tuples when columns are a MultiIndex"
+            )
+        else:
+            return list(arg_vars)
+    else:
+        return []
+
+
+@set_module("pandas")
 def melt(
     frame: DataFrame,
     id_vars=None,
@@ -41,63 +51,166 @@ def melt(
     col_level=None,
     ignore_index: bool = True,
 ) -> DataFrame:
-    # If multiindex, gather names of columns on all level for checking presence
-    # of `id_vars` and `value_vars`
-    if isinstance(frame.columns, MultiIndex):
-        cols = [x for c in frame.columns for x in c]
-    else:
-        cols = list(frame.columns)
+    """
+    Unpivot a DataFrame from wide to long format, optionally leaving identifiers set.
 
+    This function is useful to reshape a DataFrame into a format where one
+    or more columns are identifier variables (`id_vars`), while all other
+    columns are considered measured variables (`value_vars`), and are "unpivoted" to
+    the row axis, leaving just two non-identifier columns, 'variable' and
+    'value'.
+
+    Parameters
+    ----------
+    frame : DataFrame
+        The DataFrame to unpivot.
+    id_vars : scalar, tuple, list, or ndarray, optional
+        Column(s) to use as identifier variables.
+    value_vars : scalar, tuple, list, or ndarray, optional
+        Column(s) to unpivot. If not specified, uses all columns that
+        are not set as `id_vars`.
+    var_name : scalar, tuple, list, or ndarray, optional
+        Name to use for the 'variable' column. If None it uses
+        ``frame.columns.name`` or 'variable'. Must be a scalar if columns are a
+        MultiIndex.
+    value_name : scalar, default 'value'
+        Name to use for the 'value' column, can't be an existing column label.
+    col_level : scalar, optional
+        If columns are a MultiIndex then use this level to melt.
+    ignore_index : bool, default True
+        If True, original index is ignored. If False, the original index is retained.
+        Index labels will be repeated as necessary.
+
+    Returns
+    -------
+    DataFrame
+        Unpivoted DataFrame.
+
+    See Also
+    --------
+    DataFrame.melt : Identical method.
+    pivot_table : Create a spreadsheet-style pivot table as a DataFrame.
+    DataFrame.pivot : Return reshaped DataFrame organized
+        by given index / column values.
+    DataFrame.explode : Explode a DataFrame from list-like
+            columns to long format.
+
+    Notes
+    -----
+    Reference :ref:`the user guide <reshaping.melt>` for more examples.
+
+    Examples
+    --------
+    >>> df = pd.DataFrame(
+    ...     {
+    ...         "A": {0: "a", 1: "b", 2: "c"},
+    ...         "B": {0: 1, 1: 3, 2: 5},
+    ...         "C": {0: 2, 1: 4, 2: 6},
+    ...     }
+    ... )
+    >>> df
+    A  B  C
+    0  a  1  2
+    1  b  3  4
+    2  c  5  6
+
+    >>> pd.melt(df, id_vars=["A"], value_vars=["B"])
+    A variable  value
+    0  a        B      1
+    1  b        B      3
+    2  c        B      5
+
+    >>> pd.melt(df, id_vars=["A"], value_vars=["B", "C"])
+    A variable  value
+    0  a        B      1
+    1  b        B      3
+    2  c        B      5
+    3  a        C      2
+    4  b        C      4
+    5  c        C      6
+
+    The names of 'variable' and 'value' columns can be customized:
+
+    >>> pd.melt(
+    ...     df,
+    ...     id_vars=["A"],
+    ...     value_vars=["B"],
+    ...     var_name="myVarname",
+    ...     value_name="myValname",
+    ... )
+    A myVarname  myValname
+    0  a         B          1
+    1  b         B          3
+    2  c         B          5
+
+    Original index values can be kept around:
+
+    >>> pd.melt(df, id_vars=["A"], value_vars=["B", "C"], ignore_index=False)
+    A variable  value
+    0  a        B      1
+    1  b        B      3
+    2  c        B      5
+    0  a        C      2
+    1  b        C      4
+    2  c        C      6
+
+    If you have multi-index columns:
+
+    >>> df.columns = [list("ABC"), list("DEF")]
+    >>> df
+    A  B  C
+    D  E  F
+    0  a  1  2
+    1  b  3  4
+    2  c  5  6
+
+    >>> pd.melt(df, col_level=0, id_vars=["A"], value_vars=["B"])
+    A variable  value
+    0  a        B      1
+    1  b        B      3
+    2  c        B      5
+
+    >>> pd.melt(df, id_vars=[("A", "D")], value_vars=[("B", "E")])
+    (A, D) variable_0 variable_1  value
+    0      a          B          E      1
+    1      b          B          E      3
+    2      c          B          E      5
+    """
     if value_name in frame.columns:
         raise ValueError(
             f"value_name ({value_name}) cannot match an element in "
             "the DataFrame columns."
         )
+    id_vars = ensure_list_vars(id_vars, "id_vars", frame.columns)
+    value_vars_was_not_none = value_vars is not None
+    value_vars = ensure_list_vars(value_vars, "value_vars", frame.columns)
 
-    if id_vars is not None:
-        if not is_list_like(id_vars):
-            id_vars = [id_vars]
-        elif isinstance(frame.columns, MultiIndex) and not isinstance(id_vars, list):
-            raise ValueError(
-                "id_vars must be a list of tuples when columns are a MultiIndex"
-            )
-        else:
-            # Check that `id_vars` are in frame
-            id_vars = list(id_vars)
-            missing = Index(com.flatten(id_vars)).difference(cols)
-            if not missing.empty:
-                raise KeyError(
-                    "The following 'id_vars' are not present "
-                    f"in the DataFrame: {list(missing)}"
-                )
-    else:
-        id_vars = []
+    # GH61475 - prevent AttributeError when duplicate column in id_vars
+    if len(frame.columns.get_indexer_for(id_vars)) > len(id_vars):
+        raise ValueError("id_vars cannot contain duplicate columns.")
 
-    if value_vars is not None:
-        if not is_list_like(value_vars):
-            value_vars = [value_vars]
-        elif isinstance(frame.columns, MultiIndex) and not isinstance(value_vars, list):
-            raise ValueError(
-                "value_vars must be a list of tuples when columns are a MultiIndex"
-            )
-        else:
-            value_vars = list(value_vars)
-            # Check that `value_vars` are in frame
-            missing = Index(com.flatten(value_vars)).difference(cols)
-            if not missing.empty:
-                raise KeyError(
-                    "The following 'value_vars' are not present in "
-                    f"the DataFrame: {list(missing)}"
-                )
+    if id_vars or value_vars:
         if col_level is not None:
-            idx = frame.columns.get_level_values(col_level).get_indexer(
-                id_vars + value_vars
-            )
+            level = frame.columns.get_level_values(col_level)
         else:
-            idx = algos.unique(frame.columns.get_indexer_for(id_vars + value_vars))
-        frame = frame.iloc[:, idx]
+            level = frame.columns
+        labels = id_vars + value_vars
+        idx = level.get_indexer_for(labels)
+        missing = idx == -1
+        if missing.any():
+            missing_labels = [
+                lab for lab, not_found in zip(labels, missing, strict=True) if not_found
+            ]
+            raise KeyError(
+                "The following id_vars or value_vars are not present in "
+                f"the DataFrame: {missing_labels}"
+            )
+        if value_vars_was_not_none:
+            frame = frame.iloc[:, algos.unique(idx)]
+        else:
+            frame = frame.copy(deep=False)
     else:
-        frame = frame.copy()
+        frame = frame.copy(deep=False)
 
     if col_level is not None:  # allow list or other?
         # frame is a copy
@@ -113,45 +226,60 @@ def melt(
             var_name = [
                 frame.columns.name if frame.columns.name is not None else "variable"
             ]
-    if isinstance(var_name, str):
+    elif is_list_like(var_name):
+        if isinstance(frame.columns, MultiIndex):
+            if is_iterator(var_name):
+                var_name = list(var_name)
+            if len(var_name) > len(frame.columns):
+                raise ValueError(
+                    f"{var_name=} has {len(var_name)} items, "
+                    f"but the dataframe columns only have {len(frame.columns)} levels."
+                )
+        else:
+            raise ValueError(f"{var_name=} must be a scalar.")
+    else:
         var_name = [var_name]
 
-    N, K = frame.shape
-    K -= len(id_vars)
+    num_rows, K = frame.shape
+    num_cols_adjusted = K - len(id_vars)
 
     mdata: dict[Hashable, AnyArrayLike] = {}
     for col in id_vars:
         id_data = frame.pop(col)
         if not isinstance(id_data.dtype, np.dtype):
             # i.e. ExtensionDtype
-            if K > 0:
-                mdata[col] = concat([id_data] * K, ignore_index=True)
+            if num_cols_adjusted > 0:
+                mdata[col] = concat([id_data] * num_cols_adjusted, ignore_index=True)
             else:
                 # We can't concat empty list. (GH 46044)
                 mdata[col] = type(id_data)([], name=id_data.name, dtype=id_data.dtype)
         else:
-            mdata[col] = np.tile(id_data._values, K)
+            mdata[col] = np.tile(id_data._values, num_cols_adjusted)
 
     mcolumns = id_vars + var_name + [value_name]
 
-    if frame.shape[1] > 0:
+    if frame.shape[1] > 0 and not any(
+        not isinstance(dt, np.dtype) and dt._supports_2d for dt in frame.dtypes
+    ):
         mdata[value_name] = concat(
-            [frame.iloc[:, i] for i in range(frame.shape[1])]
+            [frame.iloc[:, i] for i in range(frame.shape[1])], ignore_index=True
         ).values
     else:
         mdata[value_name] = frame._values.ravel("F")
     for i, col in enumerate(var_name):
-        mdata[col] = frame.columns._get_level_values(i).repeat(N)
+        mdata[col] = frame.columns._get_level_values(i).repeat(num_rows)
 
     result = frame._constructor(mdata, columns=mcolumns)
 
     if not ignore_index:
-        result.index = tile_compat(frame.index, K)
+        taker = np.tile(np.arange(len(frame)), num_cols_adjusted)
+        result.index = frame.index.take(taker)
 
     return result
 
 
-def lreshape(data: DataFrame, groups, dropna: bool = True) -> DataFrame:
+@set_module("pandas")
+def lreshape(data: DataFrame, groups: dict, dropna: bool = True) -> DataFrame:
     """
     Reshape wide-format data to long. Generalized inverse of DataFrame.pivot.
 
@@ -189,45 +317,41 @@ def lreshape(data: DataFrame, groups, dropna: bool = True) -> DataFrame:
 
     Examples
     --------
-    >>> data = pd.DataFrame({'hr1': [514, 573], 'hr2': [545, 526],
-    ...                      'team': ['Red Sox', 'Yankees'],
-    ...                      'year1': [2007, 2007], 'year2': [2008, 2008]})
+    >>> data = pd.DataFrame(
+    ...     {
+    ...         "hr1": [514, 573],
+    ...         "hr2": [545, 526],
+    ...         "team": ["Red Sox", "Yankees"],
+    ...         "year1": [2007, 2007],
+    ...         "year2": [2008, 2008],
+    ...     }
+    ... )
     >>> data
        hr1  hr2     team  year1  year2
     0  514  545  Red Sox   2007   2008
     1  573  526  Yankees   2007   2008
 
-    >>> pd.lreshape(data, {'year': ['year1', 'year2'], 'hr': ['hr1', 'hr2']})
+    >>> pd.lreshape(data, {"year": ["year1", "year2"], "hr": ["hr1", "hr2"]})
           team  year   hr
     0  Red Sox  2007  514
     1  Yankees  2007  573
     2  Red Sox  2008  545
     3  Yankees  2008  526
     """
-    if isinstance(groups, dict):
-        keys = list(groups.keys())
-        values = list(groups.values())
-    else:
-        keys, values = zip(*groups)
-
-    all_cols = list(set.union(*(set(x) for x in values)))
-    id_cols = list(data.columns.difference(all_cols))
-
-    K = len(values[0])
-
-    for seq in values:
-        if len(seq) != K:
-            raise ValueError("All column lists must be same length")
-
     mdata = {}
     pivot_cols = []
-
-    for target, names in zip(keys, values):
+    all_cols: set[Hashable] = set()
+    K = len(next(iter(groups.values())))
+    for target, names in groups.items():
+        if len(names) != K:
+            raise ValueError("All column lists must be same length")
         to_concat = [data[col]._values for col in names]
 
         mdata[target] = concat_compat(to_concat)
         pivot_cols.append(target)
+        all_cols = all_cols.union(names)
 
+    id_cols = list(data.columns.difference(all_cols))
     for col in id_cols:
         mdata[col] = np.tile(data[col]._values, K)
 
@@ -241,6 +365,7 @@ def lreshape(data: DataFrame, groups, dropna: bool = True) -> DataFrame:
     return data._constructor(mdata, columns=id_cols + pivot_cols)
 
 
+@set_module("pandas")
 def wide_to_long(
     df: DataFrame, stubnames, i, j, sep: str = "", suffix: str = r"\d+"
 ) -> DataFrame:
@@ -313,12 +438,15 @@ def wide_to_long(
     Examples
     --------
     >>> np.random.seed(123)
-    >>> df = pd.DataFrame({"A1970" : {0 : "a", 1 : "b", 2 : "c"},
-    ...                    "A1980" : {0 : "d", 1 : "e", 2 : "f"},
-    ...                    "B1970" : {0 : 2.5, 1 : 1.2, 2 : .7},
-    ...                    "B1980" : {0 : 3.2, 1 : 1.3, 2 : .1},
-    ...                    "X"     : dict(zip(range(3), np.random.randn(3)))
-    ...                   })
+    >>> df = pd.DataFrame(
+    ...     {
+    ...         "A1970": {0: "a", 1: "b", 2: "c"},
+    ...         "A1980": {0: "d", 1: "e", 2: "f"},
+    ...         "B1970": {0: 2.5, 1: 1.2, 2: 0.7},
+    ...         "B1980": {0: 3.2, 1: 1.3, 2: 0.1},
+    ...         "X": dict(zip(range(3), np.random.randn(3), strict=True)),
+    ...     }
+    ... )
     >>> df["id"] = df.index
     >>> df
       A1970 A1980  B1970  B1980         X  id
@@ -338,12 +466,14 @@ def wide_to_long(
 
     With multiple id columns
 
-    >>> df = pd.DataFrame({
-    ...     'famid': [1, 1, 1, 2, 2, 2, 3, 3, 3],
-    ...     'birth': [1, 2, 3, 1, 2, 3, 1, 2, 3],
-    ...     'ht1': [2.8, 2.9, 2.2, 2, 1.8, 1.9, 2.2, 2.3, 2.1],
-    ...     'ht2': [3.4, 3.8, 2.9, 3.2, 2.8, 2.4, 3.3, 3.4, 2.9]
-    ... })
+    >>> df = pd.DataFrame(
+    ...     {
+    ...         "famid": [1, 1, 1, 2, 2, 2, 3, 3, 3],
+    ...         "birth": [1, 2, 3, 1, 2, 3, 1, 2, 3],
+    ...         "ht1": [2.8, 2.9, 2.2, 2, 1.8, 1.9, 2.2, 2.3, 2.1],
+    ...         "ht2": [3.4, 3.8, 2.9, 3.2, 2.8, 2.4, 3.3, 3.4, 2.9],
+    ...     }
+    ... )
     >>> df
        famid  birth  ht1  ht2
     0      1      1  2.8  3.4
@@ -355,8 +485,8 @@ def wide_to_long(
     6      3      1  2.2  3.3
     7      3      2  2.3  3.4
     8      3      3  2.1  2.9
-    >>> l = pd.wide_to_long(df, stubnames='ht', i=['famid', 'birth'], j='age')
-    >>> l
+    >>> long_format = pd.wide_to_long(df, stubnames="ht", i=["famid", "birth"], j="age")
+    >>> long_format
     ... # doctest: +NORMALIZE_WHITESPACE
                       ht
     famid birth age
@@ -381,9 +511,9 @@ def wide_to_long(
 
     Going from long back to wide just takes some creative use of `unstack`
 
-    >>> w = l.unstack()
-    >>> w.columns = w.columns.map('{0[0]}{0[1]}'.format)
-    >>> w.reset_index()
+    >>> wide_format = long_format.unstack()
+    >>> wide_format.columns = wide_format.columns.map("{0[0]}{0[1]}".format)
+    >>> wide_format.reset_index()
        famid  birth  ht1  ht2
     0      1      1  2.8  3.4
     1      1      2  2.9  3.8
@@ -398,20 +528,23 @@ def wide_to_long(
     Less wieldy column names are also handled
 
     >>> np.random.seed(0)
-    >>> df = pd.DataFrame({'A(weekly)-2010': np.random.rand(3),
-    ...                    'A(weekly)-2011': np.random.rand(3),
-    ...                    'B(weekly)-2010': np.random.rand(3),
-    ...                    'B(weekly)-2011': np.random.rand(3),
-    ...                    'X' : np.random.randint(3, size=3)})
-    >>> df['id'] = df.index
-    >>> df # doctest: +NORMALIZE_WHITESPACE, +ELLIPSIS
+    >>> df = pd.DataFrame(
+    ...     {
+    ...         "A(weekly)-2010": np.random.rand(3),
+    ...         "A(weekly)-2011": np.random.rand(3),
+    ...         "B(weekly)-2010": np.random.rand(3),
+    ...         "B(weekly)-2011": np.random.rand(3),
+    ...         "X": np.random.randint(3, size=3),
+    ...     }
+    ... )
+    >>> df["id"] = df.index
+    >>> df  # doctest: +NORMALIZE_WHITESPACE, +ELLIPSIS
        A(weekly)-2010  A(weekly)-2011  B(weekly)-2010  B(weekly)-2011  X  id
     0        0.548814        0.544883        0.437587        0.383442  0   0
     1        0.715189        0.423655        0.891773        0.791725  1   1
     2        0.602763        0.645894        0.963663        0.528895  1   2
 
-    >>> pd.wide_to_long(df, ['A(weekly)', 'B(weekly)'], i='id',
-    ...                 j='year', sep='-')
+    >>> pd.wide_to_long(df, ["A(weekly)", "B(weekly)"], i="id", j="year", sep="-")
     ... # doctest: +NORMALIZE_WHITESPACE
              X  A(weekly)  B(weekly)
     id year
@@ -426,8 +559,13 @@ def wide_to_long(
     stubnames and pass that list on to wide_to_long
 
     >>> stubnames = sorted(
-    ...     set([match[0] for match in df.columns.str.findall(
-    ...         r'[A-B]\(.*\)').values if match != []])
+    ...     set(
+    ...         [
+    ...             match[0]
+    ...             for match in df.columns.str.findall(r"[A-B]\(.*\)").values
+    ...             if match != []
+    ...         ]
+    ...     )
     ... )
     >>> list(stubnames)
     ['A(weekly)', 'B(weekly)']
@@ -435,12 +573,14 @@ def wide_to_long(
     All of the above examples have integers as suffixes. It is possible to
     have non-integers as suffixes.
 
-    >>> df = pd.DataFrame({
-    ...     'famid': [1, 1, 1, 2, 2, 2, 3, 3, 3],
-    ...     'birth': [1, 2, 3, 1, 2, 3, 1, 2, 3],
-    ...     'ht_one': [2.8, 2.9, 2.2, 2, 1.8, 1.9, 2.2, 2.3, 2.1],
-    ...     'ht_two': [3.4, 3.8, 2.9, 3.2, 2.8, 2.4, 3.3, 3.4, 2.9]
-    ... })
+    >>> df = pd.DataFrame(
+    ...     {
+    ...         "famid": [1, 1, 1, 2, 2, 2, 3, 3, 3],
+    ...         "birth": [1, 2, 3, 1, 2, 3, 1, 2, 3],
+    ...         "ht_one": [2.8, 2.9, 2.2, 2, 1.8, 1.9, 2.2, 2.3, 2.1],
+    ...         "ht_two": [3.4, 3.8, 2.9, 3.2, 2.8, 2.4, 3.3, 3.4, 2.9],
+    ...     }
+    ... )
     >>> df
        famid  birth  ht_one  ht_two
     0      1      1     2.8     3.4
@@ -453,9 +593,10 @@ def wide_to_long(
     7      3      2     2.3     3.4
     8      3      3     2.1     2.9
 
-    >>> l = pd.wide_to_long(df, stubnames='ht', i=['famid', 'birth'], j='age',
-    ...                     sep='_', suffix=r'\w+')
-    >>> l
+    >>> long_format = pd.wide_to_long(
+    ...     df, stubnames="ht", i=["famid", "birth"], j="age", sep="_", suffix=r"\w+"
+    ... )
+    >>> long_format
     ... # doctest: +NORMALIZE_WHITESPACE
                       ht
     famid birth age
@@ -479,10 +620,9 @@ def wide_to_long(
                 two  2.9
     """
 
-    def get_var_names(df, stub: str, sep: str, suffix: str) -> list[str]:
+    def get_var_names(df, stub: str, sep: str, suffix: str):
         regex = rf"^{re.escape(stub)}{re.escape(sep)}{suffix}$"
-        pattern = re.compile(regex)
-        return [col for col in df.columns if pattern.match(col)]
+        return df.columns[df.columns.str.match(regex)]
 
     def melt_stub(df, stub: str, i, j, value_vars, sep: str):
         newdf = melt(
@@ -492,20 +632,23 @@ def wide_to_long(
             value_name=stub.rstrip(sep),
             var_name=j,
         )
-        newdf[j] = Categorical(newdf[j])
         newdf[j] = newdf[j].str.replace(re.escape(stub + sep), "", regex=True)
 
         # GH17627 Cast numerics suffixes to int/float
-        newdf[j] = to_numeric(newdf[j], errors="ignore")
+        try:
+            newdf[j] = to_numeric(newdf[j])
+        except (TypeError, ValueError, OverflowError):
+            # TODO: anything else to catch?
+            pass
 
-        return newdf.set_index(i + [j])
+        return newdf.set_index([*i, j])
 
     if not is_list_like(stubnames):
         stubnames = [stubnames]
     else:
         stubnames = list(stubnames)
 
-    if any(col in stubnames for col in df.columns):
+    if df.columns.isin(stubnames).any():
         raise ValueError("stubname can't be identical to a column name")
 
     if not is_list_like(i):
@@ -516,18 +659,18 @@ def wide_to_long(
     if df[i].duplicated().any():
         raise ValueError("the id variables need to uniquely identify each row")
 
-    value_vars = [get_var_names(df, stub, sep, suffix) for stub in stubnames]
+    _melted = []
+    value_vars_flattened = []
+    for stub in stubnames:
+        value_var = get_var_names(df, stub, sep, suffix)
+        value_vars_flattened.extend(value_var)
+        _melted.append(melt_stub(df, stub, i, j, value_var, sep))
 
-    value_vars_flattened = [e for sublist in value_vars for e in sublist]
-    id_vars = list(set(df.columns.tolist()).difference(value_vars_flattened))
-
-    _melted = [melt_stub(df, s, i, j, v, sep) for s, v in zip(stubnames, value_vars)]
-    melted = _melted[0].join(_melted[1:], how="outer")
+    melted = concat(_melted, axis=1)
+    id_vars = df.columns.difference(value_vars_flattened)
+    new = df[id_vars]
 
     if len(i) == 1:
-        new = df[id_vars].set_index(i).join(melted)
-        return new
-
-    new = df[id_vars].merge(melted.reset_index(), on=i).set_index(i + [j])
-
-    return new
+        return new.set_index(i).join(melted)
+    else:
+        return new.merge(melted.reset_index(), on=i).set_index([*i, j])
