@@ -1,34 +1,64 @@
 from __future__ import annotations
 
+from typing import (
+    TYPE_CHECKING,
+    Literal,
+)
+
 import numpy as np
 
-from pandas._libs import lib
+from pandas._libs import (
+    lib,
+    missing as libmissing,
+)
+from pandas._libs.tslibs import (
+    Timedelta,
+    Timestamp,
+)
+from pandas.util._decorators import set_module
+from pandas.util._validators import check_dtype_backend
 
 from pandas.core.dtypes.cast import maybe_downcast_numeric
 from pandas.core.dtypes.common import (
     ensure_object,
-    is_datetime_or_timedelta_dtype,
+    is_bool_dtype,
     is_decimal,
     is_integer_dtype,
     is_number,
     is_numeric_dtype,
     is_scalar,
+    is_string_dtype,
     needs_i8_conversion,
 )
+from pandas.core.dtypes.dtypes import ArrowDtype
 from pandas.core.dtypes.generic import (
     ABCIndex,
     ABCSeries,
 )
 
-import pandas as pd
-from pandas.core.arrays.numeric import NumericArray
+from pandas.core.arrays import BaseMaskedArray
+from pandas.core.arrays.string_ import StringDtype
+
+if TYPE_CHECKING:
+    from pandas._typing import (
+        DateTimeErrorChoices,
+        DtypeBackend,
+        npt,
+    )
 
 
-def to_numeric(arg, errors="raise", downcast=None):
+@set_module("pandas")
+def to_numeric(
+    arg,
+    errors: DateTimeErrorChoices = "raise",
+    downcast: Literal["integer", "signed", "unsigned", "float"] | None = None,
+    dtype_backend: DtypeBackend | lib.NoDefault = lib.no_default,
+):
     """
     Convert argument to a numeric type.
 
-    The default return dtype is `float64` or `int64`
+    If the input is already of a numeric dtype, the dtype will be preserved.
+    For non-numeric inputs, the default return dtype is `float64` or `int64`
     depending on the data supplied. Use the `downcast` parameter
     to obtain other dtypes.
 
@@ -37,17 +67,18 @@ def to_numeric(arg, errors="raise", downcast=None):
     numbers smaller than `-9223372036854775808` (np.iinfo(np.int64).min)
     or larger than `18446744073709551615` (np.iinfo(np.uint64).max) are
     passed in, it is very likely they will be converted to float so that
-    they can stored in an `ndarray`. These warnings apply similarly to
+    they can be stored in an `ndarray`. These warnings apply similarly to
     `Series` since it internally leverages `ndarray`.
 
     Parameters
     ----------
     arg : scalar, list, tuple, 1-d array, or Series
         Argument to be converted.
-    errors : {'ignore', 'raise', 'coerce'}, default 'raise'
+
+    errors : {'raise', 'coerce'}, default 'raise'
         - If 'raise', then invalid parsing will raise an exception.
         - If 'coerce', then invalid parsing will be set as NaN.
-        - If 'ignore', then invalid parsing will return the input.
+
     downcast : str, default None
         Can be 'integer', 'signed', 'unsigned', or 'float'.
         If not None, and if the data has been successfully cast to a
@@ -69,12 +100,31 @@ def to_numeric(arg, errors="raise", downcast=None):
         checked satisfy that specification, no downcasting will be
         performed on the data.
 
+    dtype_backend : {'numpy_nullable', 'pyarrow'}
+        Back-end data type applied to the resultant :class:`DataFrame`
+        (still experimental). If not specified, the default behavior
+        is to not use nullable data types. If specified, the behavior
+        is as follows:
+
+        * ``"numpy_nullable"``: returns nullable-dtype-backed object
+        * ``"pyarrow"``: returns with pyarrow-backed nullable object
+
+        .. versionadded:: 2.0
+
     Returns
     -------
     ret
         Numeric if parsing succeeded.
         Return type depends on input.  Series if Series, otherwise ndarray.
 
+    Raises
+    ------
+    ValueError
+        If the input contains non-numeric values and `errors='raise'`.
+    TypeError
+        If the input is not list-like, 1D, or scalar convertible to numeric,
+        such as nested lists or unsupported input types (e.g., dict).
+
     See Also
     --------
     DataFrame.astype : Cast argument to a specified dtype.
@@ -87,30 +137,24 @@ def to_numeric(arg, errors="raise", downcast=None):
     --------
     Take separate series and convert to numeric, coercing when told to
 
-    >>> s = pd.Series(['1.0', '2', -3])
+    >>> s = pd.Series(["1.0", "2", -3])
     >>> pd.to_numeric(s)
     0    1.0
     1    2.0
     2   -3.0
     dtype: float64
-    >>> pd.to_numeric(s, downcast='float')
+    >>> pd.to_numeric(s, downcast="float")
     0    1.0
     1    2.0
     2   -3.0
     dtype: float32
-    >>> pd.to_numeric(s, downcast='signed')
+    >>> pd.to_numeric(s, downcast="signed")
     0    1
     1    2
     2   -3
     dtype: int8
-    >>> s = pd.Series(['apple', '1.0', '2', -3])
-    >>> pd.to_numeric(s, errors='ignore')
-    0    apple
-    1      1.0
-    2        2
-    3       -3
-    dtype: object
-    >>> pd.to_numeric(s, errors='coerce')
+    >>> s = pd.Series(["apple", "1.0", "2", -3])
+    >>> pd.to_numeric(s, errors="coerce")
     0    NaN
     1    1.0
     2    2.0
@@ -135,9 +179,11 @@ def to_numeric(arg, errors="raise", downcast=None):
     if downcast not in (None, "integer", "signed", "unsigned", "float"):
         raise ValueError("invalid downcasting method provided")
 
-    if errors not in ("ignore", "raise", "coerce"):
+    if errors not in ("raise", "coerce"):
         raise ValueError("invalid error value specified")
 
+    check_dtype_backend(dtype_backend)
+
     is_series = False
     is_index = False
     is_scalars = False
@@ -148,7 +194,7 @@ def to_numeric(arg, errors="raise", downcast=None):
     elif isinstance(arg, ABCIndex):
         is_index = True
         if needs_i8_conversion(arg.dtype):
-            values = arg.asi8
+            values = arg.view("i8")
         else:
             values = arg.values
     elif isinstance(arg, (list, tuple)):
@@ -158,6 +204,8 @@ def to_numeric(arg, errors="raise", downcast=None):
             return float(arg)
         if is_number(arg):
             return arg
+        if isinstance(arg, (Timedelta, Timestamp)):
+            return arg._value
         is_scalars = True
         values = np.array([arg], dtype="O")
     elif getattr(arg, "ndim", 1) > 1:
@@ -167,26 +215,42 @@ def to_numeric(arg, errors="raise", downcast=None):
 
     # GH33013: for IntegerArray & FloatingArray extract non-null values for casting
     # save mask to reconstruct the full array after casting
-    mask: np.ndarray | None = None
-    if isinstance(values, NumericArray):
+    mask: npt.NDArray[np.bool_] | None = None
+    if isinstance(values, BaseMaskedArray):
         mask = values._mask
         values = values._data[~mask]
 
     values_dtype = getattr(values, "dtype", None)
+    if isinstance(values_dtype, ArrowDtype):
+        mask = values.isna()
+        values = values.dropna().to_numpy()
+    new_mask: np.ndarray | None = None
     if is_numeric_dtype(values_dtype):
         pass
-    elif is_datetime_or_timedelta_dtype(values_dtype):
+    elif lib.is_np_dtype(values_dtype, "mM"):
         values = values.view(np.int64)
     else:
         values = ensure_object(values)
-        coerce_numeric = errors not in ("ignore", "raise")
-        try:
-            values, _ = lib.maybe_convert_numeric(
-                values, set(), coerce_numeric=coerce_numeric
-            )
-        except (ValueError, TypeError):
-            if errors == "raise":
-                raise
+        coerce_numeric = errors != "raise"
+        values, new_mask = lib.maybe_convert_numeric(  # type: ignore[call-overload]
+            values,
+            set(),
+            coerce_numeric=coerce_numeric,
+            convert_to_masked_nullable=dtype_backend is not lib.no_default
+            or (
+                isinstance(values_dtype, StringDtype)
+                and values_dtype.na_value is libmissing.NA
+            ),
+        )
+
+    if new_mask is not None:
+        # Remove unnecessary values, is expected later anyway and enables
+        # downcasting
+        values = values[~new_mask]
+    elif (dtype_backend is not lib.no_default and new_mask is None) or (
+        isinstance(values_dtype, StringDtype) and values_dtype.na_value is libmissing.NA
+    ):
+        new_mask = np.zeros(values.shape, dtype=np.bool_)
 
     # attempt downcast only if the data has been successfully converted
     # to a numerical dtype and if a downcast method has been specified
@@ -218,25 +282,45 @@ def to_numeric(arg, errors="raise", downcast=None):
                     if values.dtype == dtype:
                         break
 
-    # GH33013: for IntegerArray & FloatingArray need to reconstruct masked array
-    if mask is not None:
+    # GH33013: for IntegerArray, BooleanArray & FloatingArray need to reconstruct
+    # masked array
+    if (mask is not None or new_mask is not None) and not is_string_dtype(values.dtype):
+        if mask is None or (new_mask is not None and new_mask.shape == mask.shape):
+            # GH 52588
+            mask = new_mask
+        else:
+            mask = mask.copy()
+        assert isinstance(mask, np.ndarray)
         data = np.zeros(mask.shape, dtype=values.dtype)
         data[~mask] = values
 
         from pandas.core.arrays import (
+            ArrowExtensionArray,
+            BooleanArray,
             FloatingArray,
             IntegerArray,
         )
 
-        klass = IntegerArray if is_integer_dtype(data.dtype) else FloatingArray
-        values = klass(data, mask.copy())
+        klass: type[IntegerArray | BooleanArray | FloatingArray]
+        if is_integer_dtype(data.dtype):
+            klass = IntegerArray
+        elif is_bool_dtype(data.dtype):
+            klass = BooleanArray
+        else:
+            klass = FloatingArray
+        values = klass(data, mask)
+
+        if dtype_backend == "pyarrow" or isinstance(values_dtype, ArrowDtype):
+            values = ArrowExtensionArray(values.__arrow_array__())
 
     if is_series:
         return arg._constructor(values, index=arg.index, name=arg.name)
     elif is_index:
         # because we want to coerce to numeric if possible,
         # do not use _shallow_copy
-        return pd.Index(values, name=arg.name)
+        from pandas import Index
+
+        return Index(values, name=arg.name)
     elif is_scalars:
         return values[0]
     else:
