@@ -2,31 +2,27 @@ from __future__ import annotations
 
 import datetime
 from functools import partial
-from textwrap import dedent
-from typing import TYPE_CHECKING
+from typing import (
+    TYPE_CHECKING,
+    cast,
+)
 
 import numpy as np
 
 from pandas._libs.tslibs import Timedelta
 import pandas._libs.window.aggregations as window_aggregations
-from pandas._typing import (
-    Axis,
-    TimedeltaConvertibleTypes,
-)
-
-if TYPE_CHECKING:
-    from pandas import DataFrame, Series
-    from pandas.core.generic import NDFrame
-
-from pandas.util._decorators import doc
+from pandas.util._decorators import set_module
 
 from pandas.core.dtypes.common import (
-    is_datetime64_ns_dtype,
+    is_datetime64_dtype,
     is_numeric_dtype,
 )
+from pandas.core.dtypes.dtypes import DatetimeTZDtype
+from pandas.core.dtypes.generic import ABCSeries
 from pandas.core.dtypes.missing import isna
 
 from pandas.core import common
+from pandas.core.arrays.datetimelike import dtype_to_unit
 from pandas.core.indexers.objects import (
     BaseIndexer,
     ExponentialMovingWindowIndexer,
@@ -37,16 +33,6 @@ from pandas.core.util.numba_ import (
     maybe_use_numba,
 )
 from pandas.core.window.common import zsqrt
-from pandas.core.window.doc import (
-    _shared_docs,
-    create_section_header,
-    kwargs_numeric_only,
-    numba_notes,
-    template_header,
-    template_returns,
-    template_see_also,
-    window_agg_numba_parameters,
-)
 from pandas.core.window.numba_ import (
     generate_numba_ewm_func,
     generate_numba_ewm_table_func,
@@ -60,6 +46,19 @@ from pandas.core.window.rolling import (
     BaseWindowGroupby,
 )
 
+if TYPE_CHECKING:
+    from pandas._typing import (
+        TimedeltaConvertibleTypes,
+        TimeUnit,
+        npt,
+    )
+
+    from pandas import (
+        DataFrame,
+        Series,
+    )
+    from pandas.core.generic import NDFrame
+
 
 def get_center_of_mass(
     comass: float | None,
@@ -97,7 +96,7 @@ def get_center_of_mass(
 def _calculate_deltas(
     times: np.ndarray | NDFrame,
     halflife: float | TimedeltaConvertibleTypes | None,
-) -> np.ndarray:
+) -> npt.NDArray[np.float64]:
     """
     Return the diff of the times divided by the half-life. These values are used in
     the calculation of the ewm mean.
@@ -115,19 +114,25 @@ def _calculate_deltas(
     np.ndarray
         Diff of the times divided by the half-life
     """
+    unit = dtype_to_unit(times.dtype)
+    unit = cast("TimeUnit", unit)
+    if isinstance(times, ABCSeries):
+        times = times._values
     _times = np.asarray(times.view(np.int64), dtype=np.float64)
-    # TODO: generalize to non-nano?
-    _halflife = float(Timedelta(halflife).as_unit("ns")._value)
+    _halflife = float(Timedelta(halflife).as_unit(unit)._value)
     return np.diff(_times) / _halflife
 
 
+@set_module("pandas.api.typing")
 class ExponentialMovingWindow(BaseWindow):
     r"""
     Provide exponentially weighted (EW) calculations.
 
     Exactly one of ``com``, ``span``, ``halflife``, or ``alpha`` must be
-    provided if ``times`` is not provided. If ``times`` is provided,
+    provided if ``times`` is not provided. If ``times`` is provided and ``adjust=True``,
     ``halflife`` and one of ``com``, ``span`` or ``alpha`` may be provided.
+    If ``times`` is provided and ``adjust=False``, ``halflife`` must be the only
+    provided decay-specification parameter.
 
     Parameters
     ----------
@@ -151,8 +156,6 @@ class ExponentialMovingWindow(BaseWindow):
         observation decays to half its value. Only applicable to ``mean()``,
         and halflife value will not apply to the other functions.
 
-        .. versionadded:: 1.1.0
-
     alpha : float, optional
         Specify smoothing factor :math:`\alpha` directly
 
@@ -197,17 +200,8 @@ class ExponentialMovingWindow(BaseWindow):
           [:math:`x_0`, None, :math:`x_2`] are :math:`1-\alpha` and :math:`1` if
           ``adjust=True``, and :math:`1-\alpha` and :math:`\alpha` if ``adjust=False``.
 
-    axis : {0, 1}, default 0
-        If ``0`` or ``'index'``, calculate across the rows.
-
-        If ``1`` or ``'columns'``, calculate across the columns.
-
-        For `Series` this parameter is unused and defaults to 0.
-
     times : np.ndarray, Series, default None
 
-        .. versionadded:: 1.1.0
-
         Only applicable to ``mean()``.
 
         Times corresponding to the observations. Must be monotonically increasing and
@@ -216,8 +210,6 @@ class ExponentialMovingWindow(BaseWindow):
         If 1-D array like, a sequence with the same shape as the observations.
 
     method : str {'single', 'table'}, default 'single'
-        .. versionadded:: 1.4.0
-
         Execute the rolling operation per single column or row (``'single'``)
         or over the entire object (``'table'``).
 
@@ -228,7 +220,9 @@ class ExponentialMovingWindow(BaseWindow):
 
     Returns
     -------
-    ``ExponentialMovingWindow`` subclass
+    pandas.api.typing.ExponentialMovingWindow
+        An instance of ExponentialMovingWindow for further exponentially weighted (EW)
+        calculations, e.g. using the ``mean`` method.
 
     See Also
     --------
@@ -323,7 +317,6 @@ class ExponentialMovingWindow(BaseWindow):
         "min_periods",
         "adjust",
         "ignore_na",
-        "axis",
         "times",
         "method",
     ]
@@ -338,7 +331,6 @@ class ExponentialMovingWindow(BaseWindow):
         min_periods: int | None = 0,
         adjust: bool = True,
         ignore_na: bool = False,
-        axis: Axis = 0,
         times: np.ndarray | NDFrame | None = None,
         method: str = "single",
         *,
@@ -351,7 +343,6 @@ class ExponentialMovingWindow(BaseWindow):
             center=False,
             closed=None,
             method=method,
-            axis=axis,
             selection=selection,
         )
         self.com = com
@@ -362,10 +353,12 @@ class ExponentialMovingWindow(BaseWindow):
         self.ignore_na = ignore_na
         self.times = times
         if self.times is not None:
-            if not self.adjust:
-                raise NotImplementedError("times is not supported with adjust=False.")
-            if not is_datetime64_ns_dtype(self.times):
-                raise ValueError("times must be datetime64[ns] dtype.")
+            times_dtype = getattr(self.times, "dtype", None)
+            if not (
+                is_datetime64_dtype(times_dtype)
+                or isinstance(times_dtype, DatetimeTZDtype)
+            ):
+                raise ValueError("times must be datetime64 dtype.")
             if len(self.times) != len(obj):
                 raise ValueError("times must be the same length as the object.")
             if not isinstance(self.halflife, (str, datetime.timedelta, np.timedelta64)):
@@ -376,6 +369,11 @@ class ExponentialMovingWindow(BaseWindow):
             # Halflife is no longer applicable when calculating COM
             # But allow COM to still be calculated if the user passes other decay args
             if common.count_not_none(self.com, self.span, self.alpha) > 0:
+                if not self.adjust:
+                    raise NotImplementedError(
+                        "None of com, span, or alpha can be specified if "
+                        "times is provided and adjust=False"
+                    )
                 self._com = get_center_of_mass(self.com, self.span, None, self.alpha)
             else:
                 self._com = 1.0
@@ -388,9 +386,7 @@ class ExponentialMovingWindow(BaseWindow):
                     "times is not None."
                 )
             # Without times, points are equally spaced
-            self._deltas = np.ones(
-                max(self.obj.shape[self.axis] - 1, 0), dtype=np.float64
-            )
+            self._deltas = np.ones(max(self.obj.shape[0] - 1, 0), dtype=np.float64)
             self._com = get_center_of_mass(
                 # error: Argument 3 to "get_center_of_mass" has incompatible type
                 # "Union[float, Any, None, timedelta64, signedinteger[_64Bit]]";
@@ -421,8 +417,6 @@ class ExponentialMovingWindow(BaseWindow):
         Return an ``OnlineExponentialMovingWindow`` object to calculate
         exponentially moving window aggregations in an online method.
 
-        .. versionadded:: 1.3.0
-
         Parameters
         ----------
         engine: str, default ``'numba'``
@@ -451,24 +445,67 @@ class ExponentialMovingWindow(BaseWindow):
             min_periods=self.min_periods,
             adjust=self.adjust,
             ignore_na=self.ignore_na,
-            axis=self.axis,
             times=self.times,
             engine=engine,
             engine_kwargs=engine_kwargs,
             selection=self._selection,
         )
 
-    @doc(
-        _shared_docs["aggregate"],
-        see_also=dedent(
-            """
+    def aggregate(self, func=None, *args, **kwargs):
+        """
+        Aggregate using one or more operations over the specified axis.
+
+        Parameters
+        ----------
+        func : function, str, list or dict
+            Function to use for aggregating the data. If a function, must either
+            work when passed a Series/Dataframe or when passed to
+            Series/Dataframe.apply.
+
+            Accepted combinations are:
+
+            - function
+            - string function name
+            - list of functions and/or function names, e.g. ``[np.sum, 'mean']``
+            - dict of axis labels -> functions, function names or list of such.
+        *args
+            Positional arguments to pass to `func`.
+        **kwargs
+            Keyword arguments to pass to `func`.
+
+        Returns
+        -------
+        scalar, Series or DataFrame
+
+            The return can be:
+
+            * scalar : when Series.agg is called with single function
+            * Series : when DataFrame.agg is called with a single function
+            * DataFrame : when DataFrame.agg is called with several functions
+
         See Also
         --------
         pandas.DataFrame.rolling.aggregate
-        """
-        ),
-        examples=dedent(
-            """
+
+        Notes
+        -----
+        The aggregation operations are always performed over an axis, either the
+        index (default) or the column axis. This behavior is different from
+        `numpy` aggregation functions (`mean`, `median`, `prod`, `sum`, `std`,
+        `var`), where the default is to compute the aggregation of the flattened
+        array, e.g., ``numpy.mean(arr_2d)`` as opposed to
+        ``numpy.mean(arr_2d, axis=0)``.
+
+        `agg` is an alias for `aggregate`. Use the alias.
+
+        Functions that mutate the passed object can produce unexpected
+        behavior or errors and are not supported. See :ref:`gotchas.udf-mutation`
+        for more details.
+
+        A passed user-defined-function will be passed a Series for evaluation.
+
+        If ``func`` defines an index relabeling, ``axis`` must be ``0`` or ``index``.
+
         Examples
         --------
         >>> df = pd.DataFrame({"A": [1, 2, 3], "B": [4, 5, 6], "C": [7, 8, 9]})
@@ -484,36 +521,64 @@ class ExponentialMovingWindow(BaseWindow):
         1  1.666667  4.666667  7.666667
         2  2.428571  5.428571  8.428571
         """
-        ),
-        klass="Series/Dataframe",
-        axis="",
-    )
-    def aggregate(self, func, *args, **kwargs):
         return super().aggregate(func, *args, **kwargs)
 
     agg = aggregate
 
-    @doc(
-        template_header,
-        create_section_header("Parameters"),
-        kwargs_numeric_only,
-        window_agg_numba_parameters(),
-        create_section_header("Returns"),
-        template_returns,
-        create_section_header("See Also"),
-        template_see_also,
-        create_section_header("Notes"),
-        numba_notes.replace("\n", "", 1),
-        window_method="ewm",
-        aggregation_description="(exponential weighted moment) mean",
-        agg_method="mean",
-    )
     def mean(
         self,
         numeric_only: bool = False,
         engine=None,
         engine_kwargs=None,
     ):
+        """
+        Calculate the ewm (exponential weighted moment) mean.
+
+        Parameters
+        ----------
+        numeric_only : bool, default False
+            Include only float, int, boolean columns.
+
+        engine : str, default None
+            * ``'cython'`` : Runs the operation through C-extensions from cython.
+            * ``'numba'`` : Runs the operation through JIT compiled code from numba.
+            * ``None`` : Defaults to ``'cython'`` or globally setting
+              ``compute.use_numba``
+
+        engine_kwargs : dict, default None
+            * For ``'cython'`` engine, there are no accepted ``engine_kwargs``
+            * For ``'numba'`` engine, the engine can accept ``nopython``, ``nogil``
+              and ``parallel`` dictionary keys. The values must either be ``True`` or
+              ``False``. The default ``engine_kwargs`` for the ``'numba'`` engine is
+              ``{{'nopython': True, 'nogil': False, 'parallel': False}}``
+
+        Returns
+        -------
+        Series or DataFrame
+            Return type is the same as the original object with ``np.float64`` dtype.
+
+        See Also
+        --------
+        Series.ewm : Calling ewm with Series data.
+        DataFrame.ewm : Calling ewm with DataFrames.
+        Series.mean : Aggregating mean for Series.
+        DataFrame.mean : Aggregating mean for DataFrame.
+
+        Notes
+        -----
+        See :ref:`window.numba_engine` and :ref:`enhancingperf.numba` for
+        extended documentation and performance considerations for the Numba engine.
+
+        Examples
+        --------
+        >>> ser = pd.Series([1, 2, 3, 4])
+        >>> ser.ewm(alpha=0.2).mean()
+        0    1.000000
+        1    1.555556
+        2    2.147541
+        3    2.775068
+        dtype: float64
+        """
         if maybe_use_numba(engine):
             if self.method == "single":
                 func = generate_numba_ewm_func
@@ -545,29 +610,62 @@ class ExponentialMovingWindow(BaseWindow):
         else:
             raise ValueError("engine must be either 'numba' or 'cython'")
 
-    @doc(
-        template_header,
-        create_section_header("Parameters"),
-        kwargs_numeric_only,
-        window_agg_numba_parameters(),
-        create_section_header("Returns"),
-        template_returns,
-        create_section_header("See Also"),
-        template_see_also,
-        create_section_header("Notes"),
-        numba_notes.replace("\n", "", 1),
-        window_method="ewm",
-        aggregation_description="(exponential weighted moment) sum",
-        agg_method="sum",
-    )
     def sum(
         self,
         numeric_only: bool = False,
         engine=None,
         engine_kwargs=None,
     ):
+        """
+        Calculate the ewm (exponential weighted moment) sum.
+
+        Parameters
+        ----------
+        numeric_only : bool, default False
+            Include only float, int, boolean columns.
+        engine : str, default None
+            * ``'cython'`` : Runs the operation through C-extensions from cython.
+            * ``'numba'`` : Runs the operation through JIT compiled code from numba.
+            * ``None`` : Defaults to ``'cython'`` or globally setting
+              ``compute.use_numba``
+        engine_kwargs : dict, default None
+            * For ``'cython'`` engine, there are no accepted ``engine_kwargs``
+            * For ``'numba'`` engine, the engine can accept ``nopython``, ``nogil``
+              and ``parallel`` dictionary keys. The values must either be ``True`` or
+              ``False``. The default ``engine_kwargs`` for the ``'numba'`` engine is
+              ``{'nopython': True, 'nogil': False, 'parallel': False}``
+
+        Returns
+        -------
+        Series or DataFrame
+            Return type is the same as the original object with ``np.float64`` dtype.
+
+        See Also
+        --------
+        Series.ewm : Calling ewm with Series data.
+        DataFrame.ewm : Calling ewm with DataFrames.
+        Series.sum : Aggregating sum for Series.
+        DataFrame.sum : Aggregating sum for DataFrame.
+
+        Notes
+        -----
+        See :ref:`window.numba_engine` and :ref:`enhancingperf.numba` for extended
+        documentation and performance considerations for the Numba engine.
+
+        Examples
+        --------
+        >>> ser = pd.Series([1, 2, 3, 4])
+        >>> ser.ewm(alpha=0.2).sum()
+        0    1.000
+        1    2.800
+        2    5.240
+        3    8.192
+        dtype: float64
+        """
         if not self.adjust:
             raise NotImplementedError("sum is not implemented with adjust=False")
+        if self.times is not None:
+            raise NotImplementedError("sum is not implemented with times")
         if maybe_use_numba(engine):
             if self.method == "single":
                 func = generate_numba_ewm_func
@@ -599,25 +697,39 @@ class ExponentialMovingWindow(BaseWindow):
         else:
             raise ValueError("engine must be either 'numba' or 'cython'")
 
-    @doc(
-        template_header,
-        create_section_header("Parameters"),
-        dedent(
-            """
+    def std(self, bias: bool = False, numeric_only: bool = False):
+        """
+        Calculate the ewm (exponential weighted moment) standard deviation.
+
+        Parameters
+        ----------
         bias : bool, default False
             Use a standard estimation bias correction.
+        numeric_only : bool, default False
+            Include only float, int, boolean columns.
+
+        Returns
+        -------
+        Series or DataFrame
+            Return type is the same as the original object with ``np.float64`` dtype.
+
+        See Also
+        --------
+        Series.ewm : Calling ewm with Series data.
+        DataFrame.ewm : Calling ewm with DataFrames.
+        Series.std : Aggregating std for Series.
+        DataFrame.std : Aggregating std for DataFrame.
+
+        Examples
+        --------
+        >>> ser = pd.Series([1, 2, 3, 4])
+        >>> ser.ewm(alpha=0.2).std()
+        0         NaN
+        1    0.707107
+        2    0.995893
+        3    1.277320
+        dtype: float64
         """
-        ).replace("\n", "", 1),
-        kwargs_numeric_only,
-        create_section_header("Returns"),
-        template_returns,
-        create_section_header("See Also"),
-        template_see_also[:-1],
-        window_method="ewm",
-        aggregation_description="(exponential weighted moment) standard deviation",
-        agg_method="std",
-    )
-    def std(self, bias: bool = False, numeric_only: bool = False):
         if (
             numeric_only
             and self._selected_obj.ndim == 1
@@ -627,27 +739,45 @@ class ExponentialMovingWindow(BaseWindow):
             raise NotImplementedError(
                 f"{type(self).__name__}.std does not implement numeric_only"
             )
+        if self.times is not None:
+            raise NotImplementedError("std is not implemented with times")
         return zsqrt(self.var(bias=bias, numeric_only=numeric_only))
 
-    @doc(
-        template_header,
-        create_section_header("Parameters"),
-        dedent(
-            """
+    def var(self, bias: bool = False, numeric_only: bool = False):
+        """
+        Calculate the ewm (exponential weighted moment) variance.
+
+        Parameters
+        ----------
         bias : bool, default False
             Use a standard estimation bias correction.
+        numeric_only : bool, default False
+            Include only float, int, boolean columns.
+
+        Returns
+        -------
+        Series or DataFrame
+            Return type is the same as the original object with ``np.float64`` dtype.
+
+        See Also
+        --------
+        Series.ewm : Calling ewm with Series data.
+        DataFrame.ewm : Calling ewm with DataFrames.
+        Series.var : Aggregating var for Series.
+        DataFrame.var : Aggregating var for DataFrame.
+
+        Examples
+        --------
+        >>> ser = pd.Series([1, 2, 3, 4])
+        >>> ser.ewm(alpha=0.2).var()
+        0         NaN
+        1    0.500000
+        2    0.991803
+        3    1.631547
+        dtype: float64
         """
-        ).replace("\n", "", 1),
-        kwargs_numeric_only,
-        create_section_header("Returns"),
-        template_returns,
-        create_section_header("See Also"),
-        template_see_also[:-1],
-        window_method="ewm",
-        aggregation_description="(exponential weighted moment) variance",
-        agg_method="var",
-    )
-    def var(self, bias: bool = False, numeric_only: bool = False):
+        if self.times is not None:
+            raise NotImplementedError("var is not implemented with times")
         window_func = window_aggregations.ewmcov
         wfunc = partial(
             window_func,
@@ -662,11 +792,18 @@ class ExponentialMovingWindow(BaseWindow):
 
         return self._apply(var_func, name="var", numeric_only=numeric_only)
 
-    @doc(
-        template_header,
-        create_section_header("Parameters"),
-        dedent(
-            """
+    def cov(
+        self,
+        other: DataFrame | Series | None = None,
+        pairwise: bool | None = None,
+        bias: bool = False,
+        numeric_only: bool = False,
+    ):
+        """
+        Calculate the ewm (exponential weighted moment) sample covariance.
+
+        Parameters
+        ----------
         other : Series or DataFrame , optional
             If not supplied then will default to self and produce pairwise
             output.
@@ -679,24 +816,35 @@ class ExponentialMovingWindow(BaseWindow):
             observations will be used.
         bias : bool, default False
             Use a standard estimation bias correction.
+        numeric_only : bool, default False
+            Include only float, int, boolean columns.
+
+        Returns
+        -------
+        Series or DataFrame
+            Return type is the same as the original object with ``np.float64`` dtype.
+
+        See Also
+        --------
+        Series.ewm : Calling ewm with Series data.
+        DataFrame.ewm : Calling ewm with DataFrames.
+        Series.cov : Aggregating cov for Series.
+        DataFrame.cov : Aggregating cov for DataFrame.
+
+        Examples
+        --------
+        >>> ser1 = pd.Series([1, 2, 3, 4])
+        >>> ser2 = pd.Series([10, 11, 13, 16])
+        >>> ser1.ewm(alpha=0.2).cov(ser2)
+        0         NaN
+        1    0.500000
+        2    1.524590
+        3    3.408836
+        dtype: float64
         """
-        ).replace("\n", "", 1),
-        kwargs_numeric_only,
-        create_section_header("Returns"),
-        template_returns,
-        create_section_header("See Also"),
-        template_see_also[:-1],
-        window_method="ewm",
-        aggregation_description="(exponential weighted moment) sample covariance",
-        agg_method="cov",
-    )
-    def cov(
-        self,
-        other: DataFrame | Series | None = None,
-        pairwise: bool | None = None,
-        bias: bool = False,
-        numeric_only: bool = False,
-    ):
+        if self.times is not None:
+            raise NotImplementedError("cov is not implemented with times")
+
         from pandas import Series
 
         self._validate_numeric_only("cov", numeric_only)
@@ -736,11 +884,17 @@ class ExponentialMovingWindow(BaseWindow):
             self._selected_obj, other, pairwise, cov_func, numeric_only
         )
 
-    @doc(
-        template_header,
-        create_section_header("Parameters"),
-        dedent(
-            """
+    def corr(
+        self,
+        other: DataFrame | Series | None = None,
+        pairwise: bool | None = None,
+        numeric_only: bool = False,
+    ):
+        """
+        Calculate the ewm (exponential weighted moment) sample correlation.
+
+        Parameters
+        ----------
         other : Series or DataFrame, optional
             If not supplied then will default to self and produce pairwise
             output.
@@ -751,23 +905,35 @@ class ExponentialMovingWindow(BaseWindow):
             output will be a MultiIndex DataFrame in the case of DataFrame
             inputs. In the case of missing elements, only complete pairwise
             observations will be used.
+        numeric_only : bool, default False
+            Include only float, int, boolean columns.
+
+        Returns
+        -------
+        Series or DataFrame
+            Return type is the same as the original object with ``np.float64`` dtype.
+
+        See Also
+        --------
+        Series.ewm : Calling ewm with Series data.
+        DataFrame.ewm : Calling ewm with DataFrames.
+        Series.corr : Aggregating corr for Series.
+        DataFrame.corr : Aggregating corr for DataFrame.
+
+        Examples
+        --------
+        >>> ser1 = pd.Series([1, 2, 3, 4])
+        >>> ser2 = pd.Series([10, 11, 13, 16])
+        >>> ser1.ewm(alpha=0.2).corr(ser2)
+        0         NaN
+        1    1.000000
+        2    0.982821
+        3    0.977802
+        dtype: float64
         """
-        ).replace("\n", "", 1),
-        kwargs_numeric_only,
-        create_section_header("Returns"),
-        template_returns,
-        create_section_header("See Also"),
-        template_see_also[:-1],
-        window_method="ewm",
-        aggregation_description="(exponential weighted moment) sample correlation",
-        agg_method="corr",
-    )
-    def corr(
-        self,
-        other: DataFrame | Series | None = None,
-        pairwise: bool | None = None,
-        numeric_only: bool = False,
-    ):
+        if self.times is not None:
+            raise NotImplementedError("corr is not implemented with times")
+
         from pandas import Series
 
         self._validate_numeric_only("corr", numeric_only)
@@ -814,6 +980,7 @@ class ExponentialMovingWindow(BaseWindow):
         )
 
 
+@set_module("pandas.api.typing")
 class ExponentialMovingWindowGroupby(BaseWindowGroupby, ExponentialMovingWindow):
     """
     Provide an exponential moving window groupby implementation.
@@ -858,7 +1025,6 @@ class OnlineExponentialMovingWindow(ExponentialMovingWindow):
         min_periods: int | None = 0,
         adjust: bool = True,
         ignore_na: bool = False,
-        axis: Axis = 0,
         times: np.ndarray | NDFrame | None = None,
         engine: str = "numba",
         engine_kwargs: dict[str, bool] | None = None,
@@ -878,13 +1044,10 @@ class OnlineExponentialMovingWindow(ExponentialMovingWindow):
             min_periods=min_periods,
             adjust=adjust,
             ignore_na=ignore_na,
-            axis=axis,
             times=times,
             selection=selection,
         )
-        self._mean = EWMMeanState(
-            self._com, self.adjust, self.ignore_na, self.axis, obj.shape
-        )
+        self._mean = EWMMeanState(self._com, self.adjust, self.ignore_na, obj.shape)
         if maybe_use_numba(engine):
             self.engine = engine
             self.engine_kwargs = engine_kwargs
@@ -897,7 +1060,7 @@ class OnlineExponentialMovingWindow(ExponentialMovingWindow):
         """
         self._mean.reset()
 
-    def aggregate(self, func, *args, **kwargs):
+    def aggregate(self, func=None, *args, **kwargs):
         raise NotImplementedError("aggregate is not implemented.")
 
     def std(self, bias: bool = False, *args, **kwargs):
@@ -972,7 +1135,7 @@ class OnlineExponentialMovingWindow(ExponentialMovingWindow):
         if update_times is not None:
             raise NotImplementedError("update_times is not implemented.")
         update_deltas = np.ones(
-            max(self._selected_obj.shape[self.axis - 1] - 1, 0), dtype=np.float64
+            max(self._selected_obj.shape[-1] - 1, 0), dtype=np.float64
         )
         if update is not None:
             if self._mean.last_ewm is None:
