@@ -1,14 +1,16 @@
 from __future__ import annotations
 
-from typing import (
-    TYPE_CHECKING,
-    Any,
+from collections.abc import (
     Callable,
     Hashable,
     Iterable,
-    Literal,
     MutableMapping,
     Sequence,
+)
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Literal,
     TypeVar,
     overload,
 )
@@ -141,9 +143,9 @@ def _range2cols(areas: str) -> list[int]:
 
     Examples
     --------
-    >>> _range2cols('A:E')
+    >>> _range2cols("A:E")
     [0, 1, 2, 3, 4]
-    >>> _range2cols('A,C,Z:AB')
+    >>> _range2cols("A,C,Z:AB")
     [0, 2, 25, 26, 27]
     """
     cols: list[int] = []
@@ -159,23 +161,19 @@ def _range2cols(areas: str) -> list[int]:
 
 
 @overload
-def maybe_convert_usecols(usecols: str | list[int]) -> list[int]:
-    ...
+def maybe_convert_usecols(usecols: str | list[int]) -> list[int]: ...
 
 
 @overload
-def maybe_convert_usecols(usecols: list[str]) -> list[str]:
-    ...
+def maybe_convert_usecols(usecols: list[str]) -> list[str]: ...
 
 
 @overload
-def maybe_convert_usecols(usecols: usecols_func) -> usecols_func:
-    ...
+def maybe_convert_usecols(usecols: usecols_func) -> usecols_func: ...
 
 
 @overload
-def maybe_convert_usecols(usecols: None) -> None:
-    ...
+def maybe_convert_usecols(usecols: None) -> None: ...
 
 
 def maybe_convert_usecols(
@@ -210,13 +208,11 @@ def maybe_convert_usecols(
 
 
 @overload
-def validate_freeze_panes(freeze_panes: tuple[int, int]) -> Literal[True]:
-    ...
+def validate_freeze_panes(freeze_panes: tuple[int, int]) -> Literal[True]: ...
 
 
 @overload
-def validate_freeze_panes(freeze_panes: None) -> Literal[False]:
-    ...
+def validate_freeze_panes(freeze_panes: None) -> Literal[False]: ...
 
 
 def validate_freeze_panes(freeze_panes: tuple[int, int] | None) -> bool:
@@ -302,7 +298,7 @@ def pop_header_name(
     header_name = row[i]
     header_name = None if header_name == "" else header_name
 
-    return header_name, row[:i] + [""] + row[i + 1 :]
+    return header_name, [*row[:i], "", *row[i + 1 :]]
 
 
 def combine_kwargs(engine_kwargs: dict[str, Any] | None, kwargs: dict) -> dict:
