@@ -1,15 +1,31 @@
+from __future__ import annotations
+
 from datetime import time
+import math
+from typing import TYPE_CHECKING
 
 import numpy as np
 
-from pandas._typing import StorageOptions
 from pandas.compat._optional import import_optional_dependency
 
 from pandas.io.excel._base import BaseExcelReader
 
+if TYPE_CHECKING:
+    from xlrd import Book
 
-class XlrdReader(BaseExcelReader):
-    def __init__(self, filepath_or_buffer, storage_options: StorageOptions = None):
+    from pandas._typing import (
+        Scalar,
+        StorageOptions,
+    )
+
+
+class XlrdReader(BaseExcelReader["Book"]):
+    def __init__(
+        self,
+        filepath_or_buffer,
+        storage_options: StorageOptions | None = None,
+        engine_kwargs: dict | None = None,
+    ) -> None:
         """
         Reader using xlrd engine.
 
@@ -18,26 +34,40 @@ class XlrdReader(BaseExcelReader):
         filepath_or_buffer : str, path object or Workbook
             Object to be parsed.
         storage_options : dict, optional
-            passed to fsspec for appropriate URLs (see ``_get_filepath_or_buffer``)
+            Extra options that make sense for a particular storage connection,
+            e.g. host, port, username, password, etc. For HTTP(S) URLs the
+            key-value pairs are forwarded to ``urllib.request.Request`` as
+            header options. For other URLs (e.g. starting with "s3://", and
+            "gcs://") the key-value pairs are forwarded to ``fsspec.open``.
+            Please see ``fsspec`` and ``urllib`` for more details, and for more
+            examples on storage options refer `here <https://pandas.pydata.org/
+            pandas-docs/stable/user_guide/io.html?
+            highlight=storage_options#reading-writing-remote-files>`__.
+        engine_kwargs : dict, optional
+            Arbitrary keyword arguments passed to excel engine.
         """
-        err_msg = "Install xlrd >= 1.0.0 for Excel support"
+        err_msg = "Install xlrd >= 2.0.1 for xls Excel support"
         import_optional_dependency("xlrd", extra=err_msg)
-        super().__init__(filepath_or_buffer, storage_options=storage_options)
+        super().__init__(
+            filepath_or_buffer,
+            storage_options=storage_options,
+            engine_kwargs=engine_kwargs,
+        )
 
     @property
-    def _workbook_class(self):
+    def _workbook_class(self) -> type[Book]:
         from xlrd import Book
 
         return Book
 
-    def load_workbook(self, filepath_or_buffer):
+    def load_workbook(self, filepath_or_buffer, engine_kwargs) -> Book:
         from xlrd import open_workbook
 
         if hasattr(filepath_or_buffer, "read"):
             data = filepath_or_buffer.read()
-            return open_workbook(file_contents=data)
+            return open_workbook(file_contents=data, **engine_kwargs)
         else:
-            return open_workbook(filepath_or_buffer)
+            return open_workbook(filepath_or_buffer, **engine_kwargs)
 
     @property
     def sheet_names(self):
@@ -51,7 +81,9 @@ class XlrdReader(BaseExcelReader):
         self.raise_if_bad_sheet_by_index(index)
         return self.book.sheet_by_index(index)
 
-    def get_sheet_data(self, sheet, convert_float):
+    def get_sheet_data(
+        self, sheet, file_rows_needed: int | None = None
+    ) -> list[list[Scalar]]:
         from xlrd import (
             XL_CELL_BOOLEAN,
             XL_CELL_DATE,
@@ -67,7 +99,6 @@ class XlrdReader(BaseExcelReader):
             converts the contents of the cell into a pandas appropriate object
             """
             if cell_typ == XL_CELL_DATE:
-
                 # Use the newer xlrd datetime handling.
                 try:
                     cell_contents = xldate.xldate_as_datetime(cell_contents, epoch1904)
@@ -92,21 +123,25 @@ class XlrdReader(BaseExcelReader):
                 cell_contents = np.nan
             elif cell_typ == XL_CELL_BOOLEAN:
                 cell_contents = bool(cell_contents)
-            elif convert_float and cell_typ == XL_CELL_NUMBER:
+            elif cell_typ == XL_CELL_NUMBER:
                 # GH5394 - Excel 'numbers' are always floats
                 # it's a minimal perf hit and less surprising
-                val = int(cell_contents)
-                if val == cell_contents:
-                    cell_contents = val
+                if math.isfinite(cell_contents):
+                    # GH54564 - don't attempt to convert NaN/Inf
+                    val = int(cell_contents)
+                    if val == cell_contents:
+                        cell_contents = val
             return cell_contents
 
-        data = []
-
-        for i in range(sheet.nrows):
-            row = [
+        nrows = sheet.nrows
+        if file_rows_needed is not None:
+            nrows = min(nrows, file_rows_needed)
+        return [
+            [
                 _parse_cell(value, typ)
-                for value, typ in zip(sheet.row_values(i), sheet.row_types(i))
+                for value, typ in zip(
+                    sheet.row_values(i), sheet.row_types(i), strict=True
+                )
             ]
-            data.append(row)
-
-        return data
+            for i in range(nrows)
+        ]
