@@ -1,7 +1,7 @@
 import numpy as np
 import pytest
 
-import pandas.util._test_decorators as td
+from pandas.errors import OutOfBoundsDatetime
 
 from pandas import (
     Categorical,
@@ -13,23 +13,41 @@ from pandas import (
     TimedeltaIndex,
     Timestamp,
     date_range,
+    to_datetime,
 )
 import pandas._testing as tm
 from pandas.tests.frame.common import _check_mixed_float
 
 
 class TestFillNA:
-    @td.skip_array_manager_not_yet_implemented
+    def test_fillna_dict_inplace_nonunique_columns(self):
+        df = DataFrame(
+            {"A": [np.nan] * 3, "B": [NaT, Timestamp(1), NaT], "C": [np.nan, "foo", 2]}
+        )
+        df.columns = ["A", "A", "A"]
+        orig = df[:]
+
+        df.fillna({"A": 2}, inplace=True)
+        # The first and third columns can be set inplace, while the second cannot.
+
+        expected = DataFrame(
+            {"A": [2.0] * 3, "B": [2, Timestamp(1), 2], "C": [2, "foo", 2]}
+        )
+        expected.columns = ["A", "A", "A"]
+        tm.assert_frame_equal(df, expected)
+        assert not tm.shares_memory(df.iloc[:, 1], orig.iloc[:, 1])
+
     def test_fillna_on_column_view(self):
         # GH#46149 avoid unnecessary copies
         arr = np.full((40, 50), np.nan)
-        df = DataFrame(arr)
+        df = DataFrame(arr, copy=False)
 
-        df[0].fillna(-1, inplace=True)
-        assert (arr[:, 0] == -1).all()
+        with tm.raises_chained_assignment_error():
+            df[0].fillna(-1, inplace=True)
+        assert np.isnan(arr[:, 0]).all()
 
         # i.e. we didn't create a new 49-column block
-        assert len(df._mgr.arrays) == 1
+        assert len(df._mgr.blocks) == 1
         assert np.shares_memory(df.values, arr)
 
     def test_fillna_datetime(self, datetime_frame):
@@ -40,46 +58,37 @@ class TestFillNA:
         zero_filled = datetime_frame.fillna(0)
         assert (zero_filled.loc[zero_filled.index[:5], "A"] == 0).all()
 
-        padded = datetime_frame.fillna(method="pad")
+        padded = datetime_frame.ffill()
         assert np.isnan(padded.loc[padded.index[:5], "A"]).all()
-        assert (
-            padded.loc[padded.index[-5:], "A"] == padded.loc[padded.index[-5], "A"]
-        ).all()
 
-        msg = "Must specify a fill 'value' or 'method'"
-        with pytest.raises(ValueError, match=msg):
+        msg = r"missing 1 required positional argument: 'value'"
+        with pytest.raises(TypeError, match=msg):
             datetime_frame.fillna()
-        msg = "Cannot specify both 'value' and 'method'"
-        with pytest.raises(ValueError, match=msg):
-            datetime_frame.fillna(5, method="ffill")
-
-    def test_fillna_mixed_type(self, float_string_frame):
 
+    def test_fillna_mixed_type(self, float_string_frame, using_infer_string):
         mf = float_string_frame
         mf.loc[mf.index[5:20], "foo"] = np.nan
         mf.loc[mf.index[-10:], "A"] = np.nan
-        # TODO: make stronger assertion here, GH 25640
-        mf.fillna(value=0)
-        mf.fillna(method="pad")
+
+        result = mf.ffill()
+        assert (
+            result.loc[result.index[-10:], "A"] == result.loc[result.index[-11], "A"]
+        ).all()
+        assert (result.loc[result.index[5:20], "foo"] == "bar").all()
+
+        result = mf.fillna(value=0)
+        assert (result.loc[result.index[-10:], "A"] == 0).all()
+        assert (result.loc[result.index[5:20], "foo"] == 0).all()
 
     def test_fillna_mixed_float(self, mixed_float_frame):
-
         # mixed numeric (but no float16)
         mf = mixed_float_frame.reindex(columns=["A", "B", "D"])
         mf.loc[mf.index[-10:], "A"] = np.nan
         result = mf.fillna(value=0)
         _check_mixed_float(result, dtype={"C": None})
-
-        result = mf.fillna(method="pad")
+        result = mf.ffill()
         _check_mixed_float(result, dtype={"C": None})
 
-    def test_fillna_empty(self):
-        # empty frame (GH#2778)
-        df = DataFrame(columns=["x"])
-        for m in ["pad", "backfill"]:
-            df.x.fillna(method=m, inplace=True)
-            df.x.fillna(method=m)
-
     def test_fillna_different_dtype(self):
         # with different dtype (GH#3386)
         df = DataFrame(
@@ -90,15 +99,17 @@ class TestFillNA:
         expected = DataFrame(
             [["a", "a", "foo", "a"], ["b", "b", "foo", "b"], ["c", "c", "foo", "c"]]
         )
+        # column is originally float (all-NaN) -> filling with string gives object dtype
+        expected[2] = expected[2].astype("object")
         tm.assert_frame_equal(result, expected)
 
-        return_value = df.fillna({2: "foo"}, inplace=True)
+        result = df.fillna({2: "foo"}, inplace=True)
+        assert result is df
         tm.assert_frame_equal(df, expected)
-        assert return_value is None
 
     def test_fillna_limit_and_value(self):
         # limit and value
-        df = DataFrame(np.random.randn(10, 3))
+        df = DataFrame(np.random.default_rng(2).standard_normal((10, 3)))
         df.iloc[2:7, 0] = np.nan
         df.iloc[3:5, 2] = np.nan
 
@@ -135,7 +146,8 @@ class TestFillNA:
                 ]
             }
         )
-        tm.assert_frame_equal(df.fillna(method="pad"), exp)
+        res = df.ffill()
+        tm.assert_frame_equal(res, exp)
 
         df = DataFrame({"A": [NaT, Timestamp("2012-11-11 00:00:00+01:00")]})
         exp = DataFrame(
@@ -146,7 +158,8 @@ class TestFillNA:
                 ]
             }
         )
-        tm.assert_frame_equal(df.fillna(method="bfill"), exp)
+        res = df.bfill()
+        tm.assert_frame_equal(res, exp)
 
     def test_fillna_tzaware_different_column(self):
         # with timezone in another column
@@ -157,7 +170,7 @@ class TestFillNA:
                 "B": [1, 2, np.nan, np.nan],
             }
         )
-        result = df.fillna(method="pad")
+        result = df.ffill()
         expected = DataFrame(
             {
                 "A": date_range("20130101", periods=4, tz="US/Eastern"),
@@ -167,7 +180,6 @@ class TestFillNA:
         tm.assert_frame_equal(result, expected)
 
     def test_na_actions_categorical(self):
-
         cat = Categorical([1, 2, 3, np.nan], categories=[1, 2, 3])
         vals = ["a", "b", np.nan, "d"]
         df = DataFrame({"cats": cat, "vals": vals})
@@ -189,7 +201,7 @@ class TestFillNA:
         with pytest.raises(TypeError, match=msg):
             df.fillna(value={"cats": 4, "vals": "c"})
 
-        res = df.fillna(method="pad")
+        res = df.ffill()
         tm.assert_frame_equal(res, df_exp_fill)
 
         # dropna
@@ -245,24 +257,24 @@ class TestFillNA:
         df = DataFrame({"a": Categorical(idx)})
         tm.assert_frame_equal(df.fillna(value=NaT), df)
 
-    def test_fillna_downcast(self):
-        # GH#15277
-        # infer int64 from float64
-        df = DataFrame({"a": [1.0, np.nan]})
-        result = df.fillna(0, downcast="infer")
-        expected = DataFrame({"a": [1, 0]})
+    def test_fillna_with_categorical_series(self):
+        # https://github.com/pandas-dev/pandas/issues/56329
+        df = DataFrame(
+            {"cats": Categorical(["A", "B", "C"]), "ints": [1.0, 2.0, np.nan]}
+        )
+
+        filler = Series(Categorical([10.0, 20.0, 30.0]))
+        result = df.fillna({"ints": filler})
+
+        expected = DataFrame(
+            {"cats": Categorical(["A", "B", "C"]), "ints": [1.0, 2.0, 30.0]}
+        )
         tm.assert_frame_equal(result, expected)
 
-        # infer int64 from float64 when fillna value is a dict
-        df = DataFrame({"a": [1.0, np.nan]})
-        result = df.fillna({"a": 0}, downcast="infer")
-        expected = DataFrame({"a": [1, 0]})
-        tm.assert_frame_equal(result, expected)
-
-    def test_fillna_downcast_false(self, frame_or_series):
-        # GH#45603 preserve object dtype with downcast=False
+    def test_fillna_no_downcast(self, frame_or_series):
+        # GH#45603 preserve object dtype
         obj = frame_or_series([1, 2, 3], dtype="object")
-        result = obj.fillna("", downcast=False)
+        result = obj.fillna("")
         tm.assert_equal(result, obj)
 
     @pytest.mark.parametrize("columns", [["A", "A", "B"], ["A", "A"]])
@@ -282,15 +294,16 @@ class TestFillNA:
         result = df.dtypes
         expected = Series([np.dtype("object")] * 5, index=[1, 2, 3, 4, 5])
         tm.assert_series_equal(result, expected)
-
         result = df.fillna(1)
-        expected = DataFrame(1, index=["A", "B", "C"], columns=[1, 2, 3, 4, 5])
+        expected = DataFrame(
+            1, index=["A", "B", "C"], columns=[1, 2, 3, 4, 5], dtype=object
+        )
         tm.assert_frame_equal(result, expected)
 
         # empty block
         df = DataFrame(index=range(3), columns=["A", "B"], dtype="float64")
         result = df.fillna("nan")
-        expected = DataFrame("nan", index=range(3), columns=["A", "B"])
+        expected = DataFrame("nan", dtype="object", index=range(3), columns=["A", "B"])
         tm.assert_frame_equal(result, expected)
 
     @pytest.mark.parametrize("val", ["", 1, np.nan, 1.0])
@@ -345,90 +358,64 @@ class TestFillNA:
         tm.assert_frame_equal(result, expected)
 
     def test_ffill(self, datetime_frame):
-        datetime_frame["A"][:5] = np.nan
-        datetime_frame["A"][-5:] = np.nan
+        datetime_frame.loc[datetime_frame.index[:5], "A"] = np.nan
+        datetime_frame.loc[datetime_frame.index[-5:], "A"] = np.nan
 
-        tm.assert_frame_equal(
-            datetime_frame.ffill(), datetime_frame.fillna(method="ffill")
-        )
-
-    def test_ffill_pos_args_deprecation(self):
-        # https://github.com/pandas-dev/pandas/issues/41485
-        df = DataFrame({"a": [1, 2, 3]})
-        msg = (
-            r"In a future version of pandas all arguments of DataFrame.ffill "
-            r"will be keyword-only"
-        )
-        with tm.assert_produces_warning(FutureWarning, match=msg):
-            result = df.ffill(0)
-        expected = DataFrame({"a": [1, 2, 3]})
-        tm.assert_frame_equal(result, expected)
+        alt = datetime_frame.ffill()
+        tm.assert_frame_equal(datetime_frame.ffill(), alt)
 
     def test_bfill(self, datetime_frame):
-        datetime_frame["A"][:5] = np.nan
-        datetime_frame["A"][-5:] = np.nan
+        datetime_frame.loc[datetime_frame.index[:5], "A"] = np.nan
+        datetime_frame.loc[datetime_frame.index[-5:], "A"] = np.nan
 
-        tm.assert_frame_equal(
-            datetime_frame.bfill(), datetime_frame.fillna(method="bfill")
-        )
-
-    def test_bfill_pos_args_deprecation(self):
-        # https://github.com/pandas-dev/pandas/issues/41485
-        df = DataFrame({"a": [1, 2, 3]})
-        msg = (
-            r"In a future version of pandas all arguments of DataFrame.bfill "
-            r"will be keyword-only"
-        )
-        with tm.assert_produces_warning(FutureWarning, match=msg):
-            result = df.bfill(0)
-        expected = DataFrame({"a": [1, 2, 3]})
-        tm.assert_frame_equal(result, expected)
+        alt = datetime_frame.bfill()
+        tm.assert_frame_equal(datetime_frame.bfill(), alt)
 
     def test_frame_pad_backfill_limit(self):
         index = np.arange(10)
-        df = DataFrame(np.random.randn(10, 4), index=index)
+        df = DataFrame(np.random.default_rng(2).standard_normal((10, 4)), index=index)
 
         result = df[:2].reindex(index, method="pad", limit=5)
 
-        expected = df[:2].reindex(index).fillna(method="pad")
+        expected = df[:2].reindex(index).ffill()
         expected.iloc[-3:] = np.nan
         tm.assert_frame_equal(result, expected)
 
         result = df[-2:].reindex(index, method="backfill", limit=5)
 
-        expected = df[-2:].reindex(index).fillna(method="backfill")
+        expected = df[-2:].reindex(index).bfill()
         expected.iloc[:3] = np.nan
         tm.assert_frame_equal(result, expected)
 
     def test_frame_fillna_limit(self):
         index = np.arange(10)
-        df = DataFrame(np.random.randn(10, 4), index=index)
+        df = DataFrame(np.random.default_rng(2).standard_normal((10, 4)), index=index)
 
         result = df[:2].reindex(index)
-        result = result.fillna(method="pad", limit=5)
+        result = result.ffill(limit=5)
 
-        expected = df[:2].reindex(index).fillna(method="pad")
+        expected = df[:2].reindex(index).ffill()
         expected.iloc[-3:] = np.nan
         tm.assert_frame_equal(result, expected)
 
         result = df[-2:].reindex(index)
-        result = result.fillna(method="backfill", limit=5)
+        result = result.bfill(limit=5)
 
-        expected = df[-2:].reindex(index).fillna(method="backfill")
+        expected = df[-2:].reindex(index).bfill()
         expected.iloc[:3] = np.nan
         tm.assert_frame_equal(result, expected)
 
     def test_fillna_skip_certain_blocks(self):
         # don't try to fill boolean, int blocks
 
-        df = DataFrame(np.random.randn(10, 4).astype(int))
+        df = DataFrame(np.random.default_rng(2).standard_normal((10, 4)).astype(int))
 
         # it works!
         df.fillna(np.nan)
 
     @pytest.mark.parametrize("type", [int, float])
     def test_fillna_positive_limit(self, type):
-        df = DataFrame(np.random.randn(10, 4)).astype(type)
+        df = DataFrame(np.random.default_rng(2).standard_normal((10, 4))).astype(type)
 
         msg = "Limit must be greater than 0"
         with pytest.raises(ValueError, match=msg):
@@ -436,32 +423,33 @@ class TestFillNA:
 
     @pytest.mark.parametrize("type", [int, float])
     def test_fillna_integer_limit(self, type):
-        df = DataFrame(np.random.randn(10, 4)).astype(type)
+        df = DataFrame(np.random.default_rng(2).standard_normal((10, 4))).astype(type)
 
         msg = "Limit must be an integer"
         with pytest.raises(ValueError, match=msg):
             df.fillna(0, limit=0.5)
 
     def test_fillna_inplace(self):
-        df = DataFrame(np.random.randn(10, 4))
-        df[1][:4] = np.nan
-        df[3][-4:] = np.nan
+        df = DataFrame(np.random.default_rng(2).standard_normal((10, 4)))
+        df.loc[:4, 1] = np.nan
+        df.loc[-4:, 3] = np.nan
 
         expected = df.fillna(value=0)
         assert expected is not df
 
-        df.fillna(value=0, inplace=True)
+        result = df.fillna(value=0, inplace=True)
+        assert result is df
         tm.assert_frame_equal(df, expected)
 
-        expected = df.fillna(value={0: 0}, inplace=True)
-        assert expected is None
+        result = df.fillna(value={0: 0}, inplace=True)
+        assert result is df
 
-        df[1][:4] = np.nan
-        df[3][-4:] = np.nan
-        expected = df.fillna(method="ffill")
+        df.loc[:4, 1] = np.nan
+        df.loc[-4:, 3] = np.nan
+        expected = df.ffill()
         assert expected is not df
 
-        df.fillna(method="ffill", inplace=True)
+        df.ffill(inplace=True)
         tm.assert_frame_equal(df, expected)
 
     def test_fillna_dict_series(self):
@@ -488,9 +476,48 @@ class TestFillNA:
         expected = df.fillna(df.max().to_dict())
         tm.assert_frame_equal(result, expected)
 
-        # disable this for now
-        with pytest.raises(NotImplementedError, match="column by column"):
-            df.fillna(df.max(1), axis=1)
+    def test_fillna_dict_series_axis_1(self):
+        df = DataFrame(
+            {
+                "a": [np.nan, 1, 2, np.nan, np.nan],
+                "b": [1, 2, 3, np.nan, np.nan],
+                "c": [np.nan, 1, 2, 3, 4],
+            }
+        )
+        result = df.fillna(df.max(axis=1), axis=1)
+        result = df.fillna(df.max(axis=1), axis=1, inplace=True)
+        assert result is df
+        expected = DataFrame(
+            {
+                "a": [1.0, 1.0, 2.0, 3.0, 4.0],
+                "b": [1.0, 2.0, 3.0, 3.0, 4.0],
+                "c": [1.0, 1.0, 2.0, 3.0, 4.0],
+            }
+        )
+        tm.assert_frame_equal(result, expected)
+        tm.assert_frame_equal(df, expected)
+
+    def test_fillna_dict_series_axis_1_mismatch_cols(self):
+        df = DataFrame(
+            {
+                "a": ["abc", "def", np.nan, "ghi", "jkl"],
+                "b": [1, 2, 3, np.nan, np.nan],
+                "c": [np.nan, 1, 2, 3, 4],
+            }
+        )
+        with pytest.raises(ValueError, match="All columns must have the same dtype"):
+            df.fillna(Series({"a": "abc", "b": "def", "c": "hij"}), axis=1)
+
+    def test_fillna_dict_series_axis_1_value_mismatch_with_cols(self):
+        df = DataFrame(
+            {
+                "a": [np.nan, 1, 2, np.nan, np.nan],
+                "b": [1, 2, 3, np.nan, np.nan],
+                "c": [np.nan, 1, 2, 3, 4],
+            }
+        )
+        with pytest.raises(ValueError, match=".* not a suitable type to fill into .*"):
+            df.fillna(Series({"a": "abc", "b": "def", "c": "hij"}), axis=1)
 
     def test_fillna_dataframe(self):
         # GH#8377
@@ -528,22 +555,19 @@ class TestFillNA:
         tm.assert_frame_equal(result, expected)
 
     def test_fillna_columns(self):
-        df = DataFrame(np.random.randn(10, 10))
-        df.values[:, ::2] = np.nan
+        arr = np.random.default_rng(2).standard_normal((10, 10))
+        arr[:, ::2] = np.nan
+        df = DataFrame(arr)
 
-        result = df.fillna(method="ffill", axis=1)
-        expected = df.T.fillna(method="pad").T
+        result = df.ffill(axis=1)
+        expected = df.T.ffill().T
         tm.assert_frame_equal(result, expected)
 
         df.insert(6, "foo", 5)
-        result = df.fillna(method="ffill", axis=1)
-        expected = df.astype(float).fillna(method="ffill", axis=1)
+        result = df.ffill(axis=1)
+        expected = df.astype(float).ffill(axis=1)
         tm.assert_frame_equal(result, expected)
 
-    def test_fillna_invalid_method(self, float_frame):
-        with pytest.raises(ValueError, match="ffil"):
-            float_frame.fillna(method="ffil")
-
     def test_fillna_invalid_value(self, float_frame):
         # list
         msg = '"value" parameter must be a scalar or dict, but you passed a "{}"'
@@ -562,43 +586,15 @@ class TestFillNA:
 
     def test_fillna_col_reordering(self):
         cols = ["COL." + str(i) for i in range(5, 0, -1)]
-        data = np.random.rand(20, 5)
+        data = np.random.default_rng(2).random((20, 5))
         df = DataFrame(index=range(20), columns=cols, data=data)
-        filled = df.fillna(method="ffill")
+        filled = df.ffill()
         assert df.columns.tolist() == filled.columns.tolist()
 
-    def test_fill_corner(self, float_frame, float_string_frame):
-        mf = float_string_frame
-        mf.loc[mf.index[5:20], "foo"] = np.nan
-        mf.loc[mf.index[-10:], "A"] = np.nan
-
-        filled = float_string_frame.fillna(value=0)
-        assert (filled.loc[filled.index[5:20], "foo"] == 0).all()
-        del float_string_frame["foo"]
-
-        empty_float = float_frame.reindex(columns=[])
-
-        # TODO(wesm): unused?
-        result = empty_float.fillna(value=0)  # noqa
-
-    def test_fillna_downcast_dict(self):
-        # GH#40809
-        df = DataFrame({"col1": [1, np.nan]})
-        result = df.fillna({"col1": 2}, downcast={"col1": "int64"})
-        expected = DataFrame({"col1": [1, 2]})
-        tm.assert_frame_equal(result, expected)
-
-    def test_fillna_pos_args_deprecation(self):
-        # https://github.com/pandas-dev/pandas/issues/41485
-        df = DataFrame({"a": [1, 2, 3, np.nan]}, dtype=float)
-        msg = (
-            r"In a future version of pandas all arguments of DataFrame.fillna "
-            r"except for the argument 'value' will be keyword-only"
-        )
-        with tm.assert_produces_warning(FutureWarning, match=msg):
-            result = df.fillna(0, None, None)
-        expected = DataFrame({"a": [1, 2, 3, 0]}, dtype=float)
-        tm.assert_frame_equal(result, expected)
+    def test_fill_empty(self, float_frame):
+        df = float_frame.reindex(columns=[])
+        result = df.fillna(value=0)
+        tm.assert_frame_equal(result, df)
 
     def test_fillna_with_columns_and_limit(self):
         # GH40989
@@ -636,6 +632,18 @@ class TestFillNA:
         tm.assert_frame_equal(result, expected)
         tm.assert_frame_equal(result2, expected2)
 
+    def test_fillna_datetime_inplace(self):
+        # GH#48863
+        df = DataFrame(
+            {
+                "date1": to_datetime(["2018-05-30", None]),
+                "date2": to_datetime(["2018-09-30", None]),
+            }
+        )
+        expected = df.copy()
+        df.fillna(np.nan, inplace=True)
+        tm.assert_frame_equal(df, expected)
+
     def test_fillna_inplace_with_columns_limit_and_value(self):
         # GH40989
         df = DataFrame(
@@ -651,9 +659,72 @@ class TestFillNA:
         expected = df.fillna(axis=1, value=100, limit=1)
         assert expected is not df
 
-        df.fillna(axis=1, value=100, limit=1, inplace=True)
+        result = df.fillna(axis=1, value=100, limit=1, inplace=True)
+        assert result is df
         tm.assert_frame_equal(df, expected)
 
+    @pytest.mark.parametrize("val", [-1, {"x": -1, "y": -1}])
+    def test_inplace_dict_update_view(self, val):
+        # GH#47188
+        df = DataFrame({"x": [np.nan, 2], "y": [np.nan, 2]})
+        df_orig = df.copy()
+        result_view = df[:]
+        df.fillna(val, inplace=True)
+        expected = DataFrame({"x": [-1, 2.0], "y": [-1.0, 2]})
+        tm.assert_frame_equal(df, expected)
+        tm.assert_frame_equal(result_view, df_orig)
+
+    def test_single_block_df_with_horizontal_axis(self):
+        # GH 47713
+        df = DataFrame(
+            {
+                "col1": [5, 0, np.nan, 10, np.nan],
+                "col2": [7, np.nan, np.nan, 5, 3],
+                "col3": [12, np.nan, 1, 2, 0],
+                "col4": [np.nan, 1, 1, np.nan, 18],
+            }
+        )
+        result = df.fillna(50, limit=1, axis=1)
+        expected = DataFrame(
+            [
+                [5.0, 7.0, 12.0, 50.0],
+                [0.0, 50.0, np.nan, 1.0],
+                [50.0, np.nan, 1.0, 1.0],
+                [10.0, 5.0, 2.0, 50.0],
+                [50.0, 3.0, 0.0, 18.0],
+            ],
+            columns=["col1", "col2", "col3", "col4"],
+        )
+        tm.assert_frame_equal(result, expected)
+
+    def test_fillna_with_multi_index_frame(self):
+        # GH 47649
+        pdf = DataFrame(
+            {
+                ("x", "a"): [np.nan, 2.0, 3.0],
+                ("x", "b"): [1.0, 2.0, np.nan],
+                ("y", "c"): [1.0, 2.0, np.nan],
+            }
+        )
+        expected = DataFrame(
+            {
+                ("x", "a"): [-1.0, 2.0, 3.0],
+                ("x", "b"): [1.0, 2.0, -1.0],
+                ("y", "c"): [1.0, 2.0, np.nan],
+            }
+        )
+        tm.assert_frame_equal(pdf.fillna({"x": -1}), expected)
+        tm.assert_frame_equal(pdf.fillna({"x": -1, ("x", "b"): -2}), expected)
+
+        expected = DataFrame(
+            {
+                ("x", "a"): [-1.0, 2.0, 3.0],
+                ("x", "b"): [1.0, 2.0, -2.0],
+                ("y", "c"): [1.0, 2.0, np.nan],
+            }
+        )
+        tm.assert_frame_equal(pdf.fillna({("x", "b"): -2, "x": -1}), expected)
+
 
 def test_fillna_nonconsolidated_frame():
     # https://github.com/pandas-dev/pandas/issues/36495
@@ -665,6 +736,121 @@ def test_fillna_nonconsolidated_frame():
         ],
         columns=["i1", "i2", "i3", "f1"],
     )
-    df_nonconsol = df.pivot("i1", "i2")
+    df_nonconsol = df.pivot(index="i1", columns="i2")
     result = df_nonconsol.fillna(0)
     assert result.isna().sum().sum() == 0
+
+
+def test_fillna_nones_inplace():
+    # GH 48480
+    df = DataFrame(
+        [[None, None], [None, None]],
+        columns=["A", "B"],
+    )
+    result = df.fillna(value={"A": 1, "B": 2}, inplace=True)
+    assert result is df
+
+    expected = DataFrame([[1, 2], [1, 2]], columns=["A", "B"], dtype=object)
+    tm.assert_frame_equal(df, expected)
+
+
+@pytest.mark.parametrize(
+    "data, expected_data, method, kwargs",
+    (
+        (
+            [np.nan, np.nan, 3, np.nan, np.nan, np.nan, 7, np.nan, np.nan],
+            [np.nan, np.nan, 3.0, 3.0, 3.0, 3.0, 7.0, np.nan, np.nan],
+            "ffill",
+            {"limit_area": "inside"},
+        ),
+        (
+            [np.nan, np.nan, 3, np.nan, np.nan, np.nan, 7, np.nan, np.nan],
+            [np.nan, np.nan, 3.0, 3.0, np.nan, np.nan, 7.0, np.nan, np.nan],
+            "ffill",
+            {"limit_area": "inside", "limit": 1},
+        ),
+        (
+            [np.nan, np.nan, 3, np.nan, np.nan, np.nan, 7, np.nan, np.nan],
+            [np.nan, np.nan, 3.0, np.nan, np.nan, np.nan, 7.0, 7.0, 7.0],
+            "ffill",
+            {"limit_area": "outside"},
+        ),
+        (
+            [np.nan, np.nan, 3, np.nan, np.nan, np.nan, 7, np.nan, np.nan],
+            [np.nan, np.nan, 3.0, np.nan, np.nan, np.nan, 7.0, 7.0, np.nan],
+            "ffill",
+            {"limit_area": "outside", "limit": 1},
+        ),
+        (
+            [np.nan, np.nan, np.nan, np.nan, np.nan, np.nan, np.nan],
+            [np.nan, np.nan, np.nan, np.nan, np.nan, np.nan, np.nan],
+            "ffill",
+            {"limit_area": "outside", "limit": 1},
+        ),
+        (
+            range(5),
+            range(5),
+            "ffill",
+            {"limit_area": "outside", "limit": 1},
+        ),
+        (
+            [np.nan, np.nan, 3, np.nan, np.nan, np.nan, 7, np.nan, np.nan],
+            [np.nan, np.nan, 3.0, 7.0, 7.0, 7.0, 7.0, np.nan, np.nan],
+            "bfill",
+            {"limit_area": "inside"},
+        ),
+        (
+            [np.nan, np.nan, 3, np.nan, np.nan, np.nan, 7, np.nan, np.nan],
+            [np.nan, np.nan, 3.0, np.nan, np.nan, 7.0, 7.0, np.nan, np.nan],
+            "bfill",
+            {"limit_area": "inside", "limit": 1},
+        ),
+        (
+            [np.nan, np.nan, 3, np.nan, np.nan, np.nan, 7, np.nan, np.nan],
+            [3.0, 3.0, 3.0, np.nan, np.nan, np.nan, 7.0, np.nan, np.nan],
+            "bfill",
+            {"limit_area": "outside"},
+        ),
+        (
+            [np.nan, np.nan, 3, np.nan, np.nan, np.nan, 7, np.nan, np.nan],
+            [np.nan, 3.0, 3.0, np.nan, np.nan, np.nan, 7.0, np.nan, np.nan],
+            "bfill",
+            {"limit_area": "outside", "limit": 1},
+        ),
+    ),
+)
+def test_ffill_bfill_limit_area(data, expected_data, method, kwargs):
+    # GH#56492
+    df = DataFrame(data)
+    expected = DataFrame(expected_data)
+    result = getattr(df, method)(**kwargs)
+    tm.assert_frame_equal(result, expected)
+
+
+@pytest.mark.parametrize("test_frame", [True, False])
+@pytest.mark.parametrize("dtype", ["float", "object"])
+def test_fillna_with_none_object(test_frame, dtype):
+    # GH#57723
+    obj = Series([1, np.nan, 3], dtype=dtype)
+    if test_frame:
+        obj = obj.to_frame()
+    result = obj.fillna(value=None)
+    expected = Series([1, None, 3], dtype=dtype)
+    if test_frame:
+        expected = expected.to_frame()
+    tm.assert_equal(result, expected)
+
+
+def test_fillna_out_of_bounds_datetime():
+    # GH#61208
+    df = DataFrame(
+        {
+            "datetime": date_range("1/1/2011", periods=3, freq="h", unit="ns"),
+            "value": [1, 2, 3],
+        }
+    )
+    df.iloc[0, 0] = None
+
+    msg = "Cannot cast 0001-01-01 00:00:00 to unit='ns' without overflow"
+    with pytest.raises(OutOfBoundsDatetime, match=msg):
+        df.fillna(Timestamp("0001-01-01"))
