@@ -1,5 +1,3 @@
-import random
-
 import numpy as np
 import pytest
 
@@ -12,7 +10,6 @@ from pandas import (
     date_range,
 )
 import pandas._testing as tm
-from pandas.util.version import Version
 
 
 class TestDataFrameSortValues:
@@ -97,7 +94,9 @@ class TestDataFrameSortValues:
 
     def test_sort_values_inplace(self):
         frame = DataFrame(
-            np.random.randn(4, 4), index=[1, 2, 3, 4], columns=["A", "B", "C", "D"]
+            np.random.default_rng(2).standard_normal((4, 4)),
+            index=[1, 2, 3, 4],
+            columns=["A", "B", "C", "D"],
         )
 
         sorted_df = frame.copy()
@@ -129,9 +128,11 @@ class TestDataFrameSortValues:
     def test_sort_values_multicolumn(self):
         A = np.arange(5).repeat(20)
         B = np.tile(np.arange(5), 20)
-        random.shuffle(A)
-        random.shuffle(B)
-        frame = DataFrame({"A": A, "B": B, "C": np.random.randn(100)})
+        np.random.default_rng(2).shuffle(A)
+        np.random.default_rng(2).shuffle(B)
+        frame = DataFrame(
+            {"A": A, "B": B, "C": np.random.default_rng(2).standard_normal(100)}
+        )
 
         result = frame.sort_values(by=["A", "B"])
         indexer = np.lexsort((frame["B"], frame["A"]))
@@ -168,7 +169,7 @@ class TestDataFrameSortValues:
                 "a": pd.Series([18446637057563306014, 1162265347240853609]),
                 "b": pd.Series([1, 2]),
             },
-            index=pd.Index([1, 0]),
+            index=range(1, -1, -1),
         )
 
         tm.assert_frame_equal(result, expected)
@@ -274,9 +275,9 @@ class TestDataFrameSortValues:
         # All rows with NaN in col "B" only have unique values in "A", therefore,
         # only the rows with NaNs in "A" have to be treated individually:
         expected_idx = (
-            [11, 12, 2] + expected_idx_non_na
+            [11, 12, 2, *expected_idx_non_na]
             if na_position == "first"
-            else expected_idx_non_na + [2, 11, 12]
+            else [*expected_idx_non_na, 2, 11, 12]
         )
         expected = df.take(expected_idx)
         sorted_df = df.sort_values(
@@ -329,21 +330,15 @@ class TestDataFrameSortValues:
         df2 = df.sort_values(by=["C", "B"])
         tm.assert_frame_equal(df1, df2)
 
-    def test_sort_values_frame_column_inplace_sort_exception(
-        self, float_frame, using_copy_on_write
-    ):
+    def test_sort_values_frame_column_inplace_sort_exception(self, float_frame):
         s = float_frame["A"]
         float_frame_orig = float_frame.copy()
-        if using_copy_on_write:
-            # INFO(CoW) Series is a new object, so can be changed inplace
-            # without modifying original datafame
-            s.sort_values(inplace=True)
-            tm.assert_series_equal(s, float_frame_orig["A"].sort_values())
-            # column in dataframe is not changed
-            tm.assert_frame_equal(float_frame, float_frame_orig)
-        else:
-            with pytest.raises(ValueError, match="This Series is a view"):
-                s.sort_values(inplace=True)
+        # INFO(CoW) Series is a new object, so can be changed inplace
+        # without modifying original datafame
+        s.sort_values(inplace=True)
+        tm.assert_series_equal(s, float_frame_orig["A"].sort_values())
+        # column in dataframe is not changed
+        tm.assert_frame_equal(float_frame, float_frame_orig)
 
         cp = s.copy()
         cp.sort_values()  # it works!
@@ -364,7 +359,7 @@ class TestDataFrameSortValues:
         df_reversed = DataFrame(
             {"int": int_values[::-1], "float": float_values[::-1]},
             columns=["int", "float"],
-            index=[1, 0],
+            index=range(1, -1, -1),
         )
 
         # NaT is not a "na" for int64 columns, so na_position must not
@@ -389,7 +384,7 @@ class TestDataFrameSortValues:
         df_reversed = DataFrame(
             {"datetime": [NaT, Timestamp("2016-01-01")], "float": float_values[::-1]},
             columns=["datetime", "float"],
-            index=[1, 0],
+            index=range(1, -1, -1),
         )
 
         df_sorted = df.sort_values(["datetime", "float"], na_position="first")
@@ -544,19 +539,19 @@ class TestDataFrameSortValues:
     @pytest.mark.parametrize(
         "original_dict, sorted_dict, ignore_index, output_index",
         [
-            ({"A": [1, 2, 3]}, {"A": [3, 2, 1]}, True, [0, 1, 2]),
-            ({"A": [1, 2, 3]}, {"A": [3, 2, 1]}, False, [2, 1, 0]),
+            ({"A": [1, 2, 3]}, {"A": [3, 2, 1]}, True, range(3)),
+            ({"A": [1, 2, 3]}, {"A": [3, 2, 1]}, False, range(2, -1, -1)),
             (
                 {"A": [1, 2, 3], "B": [2, 3, 4]},
                 {"A": [3, 2, 1], "B": [4, 3, 2]},
                 True,
-                [0, 1, 2],
+                range(3),
             ),
             (
                 {"A": [1, 2, 3], "B": [2, 3, 4]},
                 {"A": [3, 2, 1], "B": [4, 3, 2]},
                 False,
-                [2, 1, 0],
+                range(2, -1, -1),
             ),
         ],
     )
@@ -596,25 +591,6 @@ class TestDataFrameSortValues:
         result = expected.sort_values(["A", "date"])
         tm.assert_frame_equal(result, expected)
 
-    def test_sort_values_item_cache(self, using_array_manager, using_copy_on_write):
-        # previous behavior incorrect retained an invalid _item_cache entry
-        df = DataFrame(np.random.randn(4, 3), columns=["A", "B", "C"])
-        df["D"] = df["A"] * 2
-        ser = df["A"]
-        if not using_array_manager:
-            assert len(df._mgr.blocks) == 2
-
-        df.sort_values(by="A")
-
-        if using_copy_on_write:
-            ser.iloc[0] = 99
-            assert df.iloc[0, 0] == df["A"][0]
-            assert df.iloc[0, 0] != 99
-        else:
-            ser.values[0] = 99
-            assert df.iloc[0, 0] == df["A"][0]
-            assert df.iloc[0, 0] == 99
-
     def test_sort_values_reshaping(self):
         # GH 39426
         values = list(range(21))
@@ -638,11 +614,20 @@ class TestDataFrameSortValues:
         expected = DataFrame({"A": [10, 20], "B": [1, 5]})
         tm.assert_frame_equal(result, expected)
 
+    def test_sort_by_column_named_none(self):
+        # GH#61512
+        df = DataFrame([[3, 1], [2, 2]], columns=[None, "C1"])
+        result = df.sort_values(by=None)
+        expected = DataFrame([[2, 2], [3, 1]], columns=[None, "C1"], index=[1, 0])
+        tm.assert_frame_equal(result, expected)
+
 
 class TestDataFrameSortKey:  # test key sorting (issue 27237)
     def test_sort_values_inplace_key(self, sort_by_key):
         frame = DataFrame(
-            np.random.randn(4, 4), index=[1, 2, 3, 4], columns=["A", "B", "C", "D"]
+            np.random.default_rng(2).standard_normal((4, 4)),
+            index=[1, 2, 3, 4],
+            columns=["A", "B", "C", "D"],
         )
 
         sorted_df = frame.copy()
@@ -843,21 +828,13 @@ def sort_names(request):
     return request.param
 
 
-@pytest.fixture(params=[True, False])
-def ascending(request):
-    return request.param
-
-
 class TestSortValuesLevelAsStr:
     def test_sort_index_level_and_column_label(
         self, df_none, df_idx, sort_names, ascending, request
     ):
         # GH#14353
-        if (
-            Version(np.__version__) >= Version("1.25")
-            and request.node.callspec.id == "df_idx0-inner-True"
-        ):
-            request.node.add_marker(
+        if request.node.callspec.id == "df_idx0-inner-True":
+            request.applymarker(
                 pytest.mark.xfail(
                     reason=(
                         "pandas default unstable sorting of duplicates"
@@ -900,16 +877,15 @@ class TestSortValuesLevelAsStr:
         # Compute result by transposing and sorting on axis=1.
         result = df_idx.T.sort_values(by=sort_names, ascending=ascending, axis=1)
 
-        if Version(np.__version__) >= Version("1.25"):
-            request.node.add_marker(
-                pytest.mark.xfail(
-                    reason=(
-                        "pandas default unstable sorting of duplicates"
-                        "issue with numpy>=1.25 with AVX instructions"
-                    ),
-                    strict=False,
-                )
+        request.applymarker(
+            pytest.mark.xfail(
+                reason=(
+                    "pandas default unstable sorting of duplicates"
+                    "issue with numpy>=1.25 with AVX instructions"
+                ),
+                strict=False,
             )
+        )
 
         tm.assert_frame_equal(result, expected)
 
@@ -921,7 +897,6 @@ class TestSortValuesLevelAsStr:
         with pytest.raises(ValueError, match=msg):
             df.sort_values(by="D", ascending="False")
 
-    @pytest.mark.parametrize("ascending", [False, 0, 1, True])
     def test_sort_values_validate_ascending_functional(self, ascending):
         df = DataFrame({"D": [23, 7, 21]})
         indexer = df["D"].argsort().values
