@@ -1,4 +1,5 @@
 import operator
+from tokenize import TokenError
 
 import numpy as np
 import pytest
@@ -58,26 +59,26 @@ class TestCompat:
         result = df.query("A>0")
         tm.assert_frame_equal(result, expected1)
         result = df.eval("A+1")
-        tm.assert_series_equal(result, expected2, check_names=False)
+        tm.assert_series_equal(result, expected2)
 
     def test_query_None(self, df, expected1, expected2):
         result = df.query("A>0", engine=None)
         tm.assert_frame_equal(result, expected1)
         result = df.eval("A+1", engine=None)
-        tm.assert_series_equal(result, expected2, check_names=False)
+        tm.assert_series_equal(result, expected2)
 
     def test_query_python(self, df, expected1, expected2):
         result = df.query("A>0", engine="python")
         tm.assert_frame_equal(result, expected1)
         result = df.eval("A+1", engine="python")
-        tm.assert_series_equal(result, expected2, check_names=False)
+        tm.assert_series_equal(result, expected2)
 
     def test_query_numexpr(self, df, expected1, expected2):
         if NUMEXPR_INSTALLED:
             result = df.query("A>0", engine="numexpr")
             tm.assert_frame_equal(result, expected1)
             result = df.eval("A+1", engine="numexpr")
-            tm.assert_series_equal(result, expected2, check_names=False)
+            tm.assert_series_equal(result, expected2)
         else:
             msg = (
                 r"'numexpr' is not installed or an unsupported version. "
@@ -158,6 +159,17 @@ class TestDataFrameEval:
         with pytest.raises(ValueError, match=msg):
             df.query("")
 
+    def test_query_duplicate_column_name(self, engine, parser):
+        df = DataFrame({"A": range(3), "B": range(3), "C": range(3)}).rename(
+            columns={"B": "A"}
+        )
+
+        res = df.query("C == 1", engine=engine, parser=parser)
+
+        expect = DataFrame([[1, 1, 1]], columns=["A", "A", "C"], index=[1])
+
+        tm.assert_frame_equal(res, expect)
+
     def test_eval_resolvers_as_list(self):
         # GH 14095
         df = DataFrame(
@@ -188,6 +200,20 @@ class TestDataFrameEval:
         expected = DataFrame({"a1": ["Y", "N"], "c": [True, False]})
         tm.assert_frame_equal(res, expected)
 
+    def test_using_numpy(self, engine, parser):
+        # GH 58041
+        skip_if_no_pandas_parser(parser)
+        df = Series([0.2, 1.5, 2.8], name="a").to_frame()
+        res = df.eval("@np.floor(a)", engine=engine, parser=parser)
+        expected = np.floor(df["a"])
+        tm.assert_series_equal(expected, res)
+
+    def test_eval_simple(self, engine, parser):
+        df = Series([0.2, 1.5, 2.8], name="a").to_frame()
+        res = df.eval("a", engine=engine, parser=parser)
+        expected = df["a"]
+        tm.assert_series_equal(expected, res)
+
     def test_extension_array_eval(self, engine, parser, request):
         # GH#58748
         if engine == "numexpr":
@@ -495,7 +521,10 @@ class TestDataFrameQueryNumExprPandas:
     def test_date_query_with_non_date(self, engine, parser):
         n = 10
         df = DataFrame(
-            {"dates": date_range("1/1/2012", periods=n), "nondate": np.arange(n)}
+            {
+                "dates": date_range("1/1/2012", periods=n, unit="ns"),
+                "nondate": np.arange(n),
+            }
         )
 
         result = df.query("dates == nondate", parser=parser, engine=engine)
@@ -749,7 +778,7 @@ class TestDataFrameQueryNumExprPandas:
         # https://github.com/pandas-dev/pandas/issues/29463
         tz = tz_aware_fixture
         df_index = date_range(
-            start="2019-01-01", freq="1d", periods=10, tz=tz, name="time"
+            start="2019-01-01", freq="1D", periods=10, tz=tz, name="time"
         )
         expected = DataFrame(index=df_index)
         df = DataFrame(index=df_index)
@@ -757,7 +786,6 @@ class TestDataFrameQueryNumExprPandas:
         tm.assert_frame_equal(result, expected)
 
         expected = DataFrame(df_index)
-        expected.columns = expected.columns.astype(object)
         result = df.reset_index().query('"2018-01-03 00:00:00+00" < time')
         tm.assert_frame_equal(result, expected)
 
@@ -966,11 +994,11 @@ class TestDataFrameQueryStrings:
             rhs = lhs[::-1]
 
             eq, ne = "==", "!="
-            ops = 2 * ([eq] + [ne])
+            ops = 2 * ([eq, ne])
             msg = r"'(Not)?In' nodes are not implemented"
 
-            for lhs, op, rhs in zip(lhs, ops, rhs):
-                ex = f"{lhs} {op} {rhs}"
+            for lh, op_, rh in zip(lhs, ops, rhs):
+                ex = f"{lh} {op_} {rh}"
                 with pytest.raises(NotImplementedError, match=msg):
                     df.query(
                         ex,
@@ -1007,11 +1035,11 @@ class TestDataFrameQueryStrings:
             rhs = lhs[::-1]
 
             eq, ne = "==", "!="
-            ops = 2 * ([eq] + [ne])
+            ops = 2 * ([eq, ne])
             msg = r"'(Not)?In' nodes are not implemented"
 
-            for lhs, op, rhs in zip(lhs, ops, rhs):
-                ex = f"{lhs} {op} {rhs}"
+            for lh, ops_, rh in zip(lhs, ops, rhs):
+                ex = f"{lh} {ops_} {rh}"
                 with pytest.raises(NotImplementedError, match=msg):
                     df.query(ex, engine=engine, parser=parser)
         else:
@@ -1157,6 +1185,7 @@ class TestDataFrameQueryStrings:
         df_expected = DataFrame({"a": expected}, dtype="string")
         df_expected.index = df_expected.index.astype("int64")
         df = DataFrame({"a": in_list}, dtype="string")
+        df.index = Index(list(df.index), dtype=df.index.dtype)
         res1 = df.query("a == 'asdf'", parser=parser, engine=engine)
         res2 = df[df["a"] == "asdf"]
         res3 = df.query("a <= 'asdf'", parser=parser, engine=engine)
@@ -1200,7 +1229,7 @@ class TestDataFrameQueryBacktickQuoting:
         by backticks. The last two columns cannot be escaped by backticks
         and should raise a ValueError.
         """
-        yield DataFrame(
+        return DataFrame(
             {
                 "A": [1, 2, 3],
                 "B B": [3, 2, 1],
@@ -1222,6 +1251,8 @@ class TestDataFrameQueryBacktickQuoting:
                 "it's": [6, 3, 1],
                 "that's": [9, 1, 8],
                 "☺": [8, 7, 6],
+                "xy （z）": [1, 2, 3],  # noqa: RUF001
+                "xy （z\\uff09": [4, 5, 6],  # noqa: RUF001
                 "foo#bar": [2, 4, 5],
                 1: [5, 7, 9],
             }
@@ -1307,6 +1338,11 @@ class TestDataFrameQueryBacktickQuoting:
         expect = df[" A"] + df["  "]
         tm.assert_series_equal(res, expect)
 
+    def test_ints(self, df):
+        res = df.query("`1` == 7")
+        expect = df[df[1] == 7]
+        tm.assert_frame_equal(res, expect)
+
     def test_lots_of_operators_string(self, df):
         res = df.query("`  &^ :!€$?(} >    <++*''  ` > 4")
         expect = df[df["  &^ :!€$?(} >    <++*''  "] > 4]
@@ -1317,20 +1353,155 @@ class TestDataFrameQueryBacktickQuoting:
         with pytest.raises(AttributeError, match=message):
             df.eval("@pd.thing")
 
-    def test_failing_quote(self, df):
-        msg = r"(Could not convert ).*( to a valid Python identifier.)"
-        with pytest.raises(SyntaxError, match=msg):
-            df.query("`it's` > `that's`")
+    def test_quote(self, df):
+        res = df.query("`it's` > `that's`")
+        expect = df[df["it's"] > df["that's"]]
+        tm.assert_frame_equal(res, expect)
 
-    def test_failing_character_outside_range(self, df):
-        msg = r"(Could not convert ).*( to a valid Python identifier.)"
-        with pytest.raises(SyntaxError, match=msg):
-            df.query("`☺` > 4")
+    def test_character_outside_range_smiley(self, df):
+        res = df.query("`☺` > 4")
+        expect = df[df["☺"] > 4]
+        tm.assert_frame_equal(res, expect)
 
-    def test_failing_hashtag(self, df):
-        msg = "Failed to parse backticks"
-        with pytest.raises(SyntaxError, match=msg):
-            df.query("`foo#bar` > 4")
+    def test_character_outside_range_2_byte_parens(self, df):
+        # GH 49633
+        res = df.query("`xy （z）` == 2")  # noqa: RUF001
+        expect = df[df["xy （z）"] == 2]  # noqa: RUF001
+        tm.assert_frame_equal(res, expect)
+
+    def test_character_outside_range_and_actual_backslash(self, df):
+        # GH 49633
+        res = df.query("`xy （z\\uff09` == 2")  # noqa: RUF001
+        expect = df[df["xy \uff08z\\uff09"] == 2]
+        tm.assert_frame_equal(res, expect)
+
+    def test_hashtag(self, df):
+        res = df.query("`foo#bar` > 4")
+        expect = df[df["foo#bar"] > 4]
+        tm.assert_frame_equal(res, expect)
+
+    def test_expr_with_column_name_with_hashtag_character(self):
+        # GH 59285
+        df = DataFrame((1, 2, 3), columns=["a#"])
+        result = df.query("`a#` < 2")
+        expected = df[df["a#"] < 2]
+        tm.assert_frame_equal(result, expected)
+
+    def test_expr_with_comment(self):
+        # GH 59285
+        df = DataFrame((1, 2, 3), columns=["a#"])
+        result = df.query("`a#` < 2  # This is a comment")
+        expected = df[df["a#"] < 2]
+        tm.assert_frame_equal(result, expected)
+
+    def test_expr_with_column_name_with_backtick_and_hash(self):
+        # GH 59285
+        df = DataFrame((1, 2, 3), columns=["a`#b"])
+        result = df.query("`a``#b` < 2")
+        expected = df[df["a`#b"] < 2]
+        tm.assert_frame_equal(result, expected)
+
+    def test_expr_with_column_name_with_backtick(self):
+        # GH 59285
+        df = DataFrame({"a`b": (1, 2, 3), "ab": (4, 5, 6)})
+        result = df.query("`a``b` < 2")
+        # Note: Formatting checks may wrongly consider the above ``inline code``.
+        expected = df[df["a`b"] < 2]
+        tm.assert_frame_equal(result, expected)
+
+    def test_expr_with_string_with_backticks(self):
+        # GH 59285
+        df = DataFrame(("`", "`````", "``````````"), columns=["#backticks"])
+        result = df.query("'```' < `#backticks`")
+        expected = df["```" < df["#backticks"]]
+        tm.assert_frame_equal(result, expected)
+
+    def test_expr_with_string_with_backticked_substring_same_as_column_name(self):
+        # GH 59285
+        df = DataFrame(("`", "`````", "``````````"), columns=["#backticks"])
+        result = df.query("'`#backticks`' < `#backticks`")
+        expected = df["`#backticks`" < df["#backticks"]]
+        tm.assert_frame_equal(result, expected)
+
+    @pytest.mark.parametrize(
+        "col1,col2,expr",
+        [
+            ("it's", "that's", "`it's` < `that's`"),
+            ('it"s', 'that"s', '`it"s` < `that"s`'),
+            ("it's", 'that\'s "nice"', "`it's` < `that's \"nice\"`"),
+            ("it's", "that's #cool", "`it's` < `that's #cool` # This is a comment"),
+        ],
+    )
+    def test_expr_with_column_names_with_special_characters(self, col1, col2, expr):
+        # GH 59285
+        df = DataFrame(
+            [
+                {col1: 1, col2: 2},
+                {col1: 3, col2: 4},
+                {col1: -1, col2: -2},
+                {col1: -3, col2: -4},
+            ]
+        )
+        result = df.query(expr)
+        expected = df[df[col1] < df[col2]]
+        tm.assert_frame_equal(result, expected)
+
+    def test_expr_with_no_backticks(self):
+        # GH 59285
+        df = DataFrame(("aaa", "vvv", "zzz"), columns=["column_name"])
+        result = df.query("'value' < column_name")
+        expected = df["value" < df["column_name"]]
+        tm.assert_frame_equal(result, expected)
+
+    def test_expr_with_no_quotes_and_backtick_is_unmatched(self):
+        # GH 59285
+        df = DataFrame((1, 5, 10), columns=["column-name"])
+        with pytest.raises((SyntaxError, TokenError), match="invalid syntax"):
+            df.query("5 < `column-name")
+
+    def test_expr_with_no_quotes_and_backtick_is_matched(self):
+        # GH 59285
+        df = DataFrame((1, 5, 10), columns=["column-name"])
+        result = df.query("5 < `column-name`")
+        expected = df[5 < df["column-name"]]
+        tm.assert_frame_equal(result, expected)
+
+    def test_expr_with_backtick_opened_before_quote_and_backtick_is_unmatched(self):
+        # GH 59285
+        df = DataFrame((1, 5, 10), columns=["It's"])
+        with pytest.raises(
+            (SyntaxError, TokenError), match="unterminated string literal"
+        ):
+            df.query("5 < `It's")
+
+    def test_expr_with_backtick_opened_before_quote_and_backtick_is_matched(self):
+        # GH 59285
+        df = DataFrame((1, 5, 10), columns=["It's"])
+        result = df.query("5 < `It's`")
+        expected = df[5 < df["It's"]]
+        tm.assert_frame_equal(result, expected)
+
+    def test_expr_with_quote_opened_before_backtick_and_quote_is_unmatched(self):
+        # GH 59285
+        df = DataFrame(("aaa", "vvv", "zzz"), columns=["column-name"])
+        with pytest.raises(
+            (SyntaxError, TokenError), match="unterminated string literal"
+        ):
+            df.query("`column-name` < 'It`s that\\'s \"quote\" #hash")
+
+    def test_expr_with_quote_opened_before_backtick_and_quote_is_matched_at_end(self):
+        # GH 59285
+        df = DataFrame(("aaa", "vvv", "zzz"), columns=["column-name"])
+        result = df.query("`column-name` < 'It`s that\\'s \"quote\" #hash'")
+        expected = df[df["column-name"] < 'It`s that\'s "quote" #hash']
+        tm.assert_frame_equal(result, expected)
+
+    def test_expr_with_quote_opened_before_backtick_and_quote_is_matched_in_mid(self):
+        # GH 59285
+        df = DataFrame(("aaa", "vvv", "zzz"), columns=["column-name"])
+        result = df.query("'It`s that\\'s \"quote\" #hash' < `column-name`")
+        expected = df['It`s that\'s "quote" #hash' < df["column-name"]]
+        tm.assert_frame_equal(result, expected)
 
     def test_call_non_named_expression(self, df):
         """
@@ -1399,12 +1570,12 @@ class TestDataFrameQueryBacktickQuoting:
         if dtype == "int64[pyarrow]":
             pytest.importorskip("pyarrow")
         # GH#50261
-        df = DataFrame({"a": Series([1, 2], dtype=dtype)})
+        df = DataFrame({"a": [1, 2]}, dtype=dtype)
         ref = {2}  # noqa: F841
         warning = RuntimeWarning if dtype == "Int64" and NUMEXPR_INSTALLED else None
         with tm.assert_produces_warning(warning):
             result = df.query("a in @ref")
-        expected = DataFrame({"a": Series([2], dtype=dtype, index=[1])})
+        expected = DataFrame({"a": [2]}, index=range(1, 2), dtype=dtype)
         tm.assert_frame_equal(result, expected)
 
     @pytest.mark.parametrize("engine", ["python", "numexpr"])
@@ -1423,8 +1594,8 @@ class TestDataFrameQueryBacktickQuoting:
             result = df.query("A == B", engine=engine)
         expected = DataFrame(
             {
-                "A": Series([1, 2], dtype="Int64", index=[0, 2]),
-                "B": Series([1, 2], dtype=dtype, index=[0, 2]),
+                "A": Series([1, 2], dtype="Int64", index=range(0, 4, 2)),
+                "B": Series([1, 2], dtype=dtype, index=range(0, 4, 2)),
             }
         )
         tm.assert_frame_equal(result, expected)
