@@ -8,6 +8,11 @@ from datetime import (
 import numpy as np
 import pytest
 
+from pandas._libs import index as libindex
+from pandas.compat.numpy import np_long
+from pandas.errors import Pandas4Warning
+import pandas.util._test_decorators as td
+
 import pandas as pd
 from pandas import (
     DatetimeIndex,
@@ -29,49 +34,46 @@ class TestGetItem:
         # GH4226
         st = Timestamp("2013-07-01 00:00:00", tz="America/Los_Angeles")
         et = Timestamp("2013-07-02 00:00:00", tz="America/Los_Angeles")
-        dr = date_range(st, et, freq="H", name="timebucket")
+        dr = date_range(st, et, freq="h", name="timebucket")
         assert dr[1:].name == dr.name
 
-    def test_getitem(self):
-        idx1 = date_range("2011-01-01", "2011-01-31", freq="D", name="idx")
-        idx2 = date_range(
-            "2011-01-01", "2011-01-31", freq="D", tz="Asia/Tokyo", name="idx"
+    @pytest.mark.parametrize("tz", [None, "Asia/Tokyo"])
+    def test_getitem(self, tz):
+        idx = date_range("2011-01-01", "2011-01-31", freq="D", tz=tz, name="idx")
+
+        result = idx[0]
+        assert result == Timestamp("2011-01-01", tz=idx.tz)
+
+        result = idx[0:5]
+        expected = date_range(
+            "2011-01-01", "2011-01-05", freq="D", tz=idx.tz, name="idx"
         )
+        tm.assert_index_equal(result, expected)
+        assert result.freq == expected.freq
 
-        for idx in [idx1, idx2]:
-            result = idx[0]
-            assert result == Timestamp("2011-01-01", tz=idx.tz)
+        result = idx[0:10:2]
+        expected = date_range(
+            "2011-01-01", "2011-01-09", freq="2D", tz=idx.tz, name="idx"
+        )
+        tm.assert_index_equal(result, expected)
+        assert result.freq == expected.freq
 
-            result = idx[0:5]
-            expected = date_range(
-                "2011-01-01", "2011-01-05", freq="D", tz=idx.tz, name="idx"
-            )
-            tm.assert_index_equal(result, expected)
-            assert result.freq == expected.freq
+        result = idx[-20:-5:3]
+        expected = date_range(
+            "2011-01-12", "2011-01-24", freq="3D", tz=idx.tz, name="idx"
+        )
+        tm.assert_index_equal(result, expected)
+        assert result.freq == expected.freq
 
-            result = idx[0:10:2]
-            expected = date_range(
-                "2011-01-01", "2011-01-09", freq="2D", tz=idx.tz, name="idx"
-            )
-            tm.assert_index_equal(result, expected)
-            assert result.freq == expected.freq
-
-            result = idx[-20:-5:3]
-            expected = date_range(
-                "2011-01-12", "2011-01-24", freq="3D", tz=idx.tz, name="idx"
-            )
-            tm.assert_index_equal(result, expected)
-            assert result.freq == expected.freq
-
-            result = idx[4::-1]
-            expected = DatetimeIndex(
-                ["2011-01-05", "2011-01-04", "2011-01-03", "2011-01-02", "2011-01-01"],
-                freq="-1D",
-                tz=idx.tz,
-                name="idx",
-            )
-            tm.assert_index_equal(result, expected)
-            assert result.freq == expected.freq
+        result = idx[4::-1]
+        expected = DatetimeIndex(
+            ["2011-01-05", "2011-01-04", "2011-01-03", "2011-01-02", "2011-01-01"],
+            dtype=idx.dtype,
+            freq="-1D",
+            name="idx",
+        )
+        tm.assert_index_equal(result, expected)
+        assert result.freq == expected.freq
 
     @pytest.mark.parametrize("freq", ["B", "C"])
     def test_dti_business_getitem(self, freq):
@@ -91,7 +93,7 @@ class TestGetItem:
         assert fancy_indexed.freq is None
 
         # 32-bit vs. 64-bit platforms
-        assert rng[4] == rng[np.int_(4)]
+        assert rng[4] == rng[np_long(4)]
 
     @pytest.mark.parametrize("freq", ["B", "C"])
     def test_dti_business_getitem_matplotlib_hackaround(self, freq):
@@ -101,7 +103,7 @@ class TestGetItem:
             rng[:, None]
 
     def test_getitem_int_list(self):
-        dti = date_range(start="1/1/2005", end="12/1/2005", freq="M")
+        dti = date_range(start="1/1/2005", end="12/1/2005", freq="ME")
         dti2 = dti[[1, 3, 5]]
 
         v1 = dti2[0]
@@ -117,6 +119,17 @@ class TestGetItem:
 
 
 class TestWhere:
+    @pytest.mark.parametrize("is_td", [True, False])
+    def test_where_freq_invalidation(self, is_td):
+        # GH#24555
+        index = date_range("20130101", periods=3, tz="US/Eastern")
+        if is_td:
+            index = index - index[0]
+        other = Index([pd.NaT, pd.NaT, *index[2:].tolist()])
+
+        result = index.where(notna(other), other)
+        assert result.freq is None
+
     def test_where_doesnt_retain_freq(self):
         dti = date_range("20130101", periods=3, freq="D", name="idx")
         cond = [True, True, False]
@@ -135,54 +148,54 @@ class TestWhere:
             tm.assert_index_equal(result, expected)
 
         i2 = i.copy()
-        i2 = Index([pd.NaT, pd.NaT] + i[2:].tolist())
+        i2 = Index([pd.NaT, pd.NaT, *i[2:].tolist()])
         result = i.where(notna(i2), i2)
         tm.assert_index_equal(result, i2)
 
         i2 = i.copy()
-        i2 = Index([pd.NaT, pd.NaT] + i[2:].tolist())
+        i2 = Index([pd.NaT, pd.NaT, *i[2:].tolist()])
         result = i.where(notna(i2), i2._values)
         tm.assert_index_equal(result, i2)
 
     def test_where_invalid_dtypes(self):
-        dti = date_range("20130101", periods=3, tz="US/Eastern")
+        dti = date_range("20130101", periods=3, tz="US/Eastern", unit="ns")
 
         tail = dti[2:].tolist()
-        i2 = Index([pd.NaT, pd.NaT] + tail)
+        i2 = Index([pd.NaT, pd.NaT, *tail])
 
         mask = notna(i2)
 
         # passing tz-naive ndarray to tzaware DTI
         result = dti.where(mask, i2.values)
-        expected = Index([pd.NaT.asm8, pd.NaT.asm8] + tail, dtype=object)
+        expected = Index([pd.NaT.asm8, pd.NaT.asm8, *tail], dtype=object)
         tm.assert_index_equal(result, expected)
 
         # passing tz-aware DTI to tznaive DTI
         naive = dti.tz_localize(None)
         result = naive.where(mask, i2)
-        expected = Index([i2[0], i2[1]] + naive[2:].tolist(), dtype=object)
+        expected = Index([i2[0], i2[1], *naive[2:].tolist()], dtype=object)
         tm.assert_index_equal(result, expected)
 
         pi = i2.tz_localize(None).to_period("D")
         result = dti.where(mask, pi)
-        expected = Index([pi[0], pi[1]] + tail, dtype=object)
+        expected = Index([pi[0], pi[1], *tail], dtype=object)
         tm.assert_index_equal(result, expected)
 
         tda = i2.asi8.view("timedelta64[ns]")
         result = dti.where(mask, tda)
-        expected = Index([tda[0], tda[1]] + tail, dtype=object)
+        expected = Index([tda[0], tda[1], *tail], dtype=object)
         assert isinstance(expected[0], np.timedelta64)
         tm.assert_index_equal(result, expected)
 
         result = dti.where(mask, i2.asi8)
-        expected = Index([pd.NaT._value, pd.NaT._value] + tail, dtype=object)
+        expected = Index([pd.NaT._value, pd.NaT._value, *tail], dtype=object)
         assert isinstance(expected[0], int)
         tm.assert_index_equal(result, expected)
 
         # non-matching scalar
         td = pd.Timedelta(days=4)
         result = dti.where(mask, td)
-        expected = Index([td, td] + tail, dtype=object)
+        expected = Index([td, td, *tail], dtype=object)
         assert expected[0] is td
         tm.assert_index_equal(result, expected)
 
@@ -205,70 +218,75 @@ class TestWhere:
         tm.assert_index_equal(result, expected)
 
         i2 = i.copy()
-        i2 = Index([pd.NaT, pd.NaT] + i[2:].tolist())
+        i2 = Index([pd.NaT, pd.NaT, *i[2:].tolist()])
         result = i.where(notna(i2))
         expected = i2
         tm.assert_index_equal(result, expected)
 
 
 class TestTake:
+    @pytest.mark.parametrize("tzstr", ["US/Eastern", "dateutil/US/Eastern"])
+    def test_dti_take_dont_lose_meta(self, tzstr):
+        rng = date_range("1/1/2000", periods=20, tz=tzstr)
+
+        result = rng.take(range(5))
+        assert result.tz == rng.tz
+        assert result.freq == rng.freq
+
     def test_take_nan_first_datetime(self):
         index = DatetimeIndex([pd.NaT, Timestamp("20130101"), Timestamp("20130102")])
         result = index.take([-1, 0, 1])
         expected = DatetimeIndex([index[-1], index[0], index[1]])
         tm.assert_index_equal(result, expected)
 
-    def test_take(self):
+    @pytest.mark.parametrize("tz", [None, "Asia/Tokyo"])
+    def test_take(self, tz):
         # GH#10295
-        idx1 = date_range("2011-01-01", "2011-01-31", freq="D", name="idx")
-        idx2 = date_range(
-            "2011-01-01", "2011-01-31", freq="D", tz="Asia/Tokyo", name="idx"
+        idx = date_range("2011-01-01", "2011-01-31", freq="D", name="idx", tz=tz)
+
+        result = idx.take([0])
+        assert result == Timestamp("2011-01-01", tz=idx.tz)
+
+        result = idx.take([0, 1, 2])
+        expected = date_range(
+            "2011-01-01", "2011-01-03", freq="D", tz=idx.tz, name="idx"
         )
+        tm.assert_index_equal(result, expected)
+        assert result.freq == expected.freq
 
-        for idx in [idx1, idx2]:
-            result = idx.take([0])
-            assert result == Timestamp("2011-01-01", tz=idx.tz)
+        result = idx.take([0, 2, 4])
+        expected = date_range(
+            "2011-01-01", "2011-01-05", freq="2D", tz=idx.tz, name="idx"
+        )
+        tm.assert_index_equal(result, expected)
+        assert result.freq == expected.freq
 
-            result = idx.take([0, 1, 2])
-            expected = date_range(
-                "2011-01-01", "2011-01-03", freq="D", tz=idx.tz, name="idx"
-            )
-            tm.assert_index_equal(result, expected)
-            assert result.freq == expected.freq
+        result = idx.take([7, 4, 1])
+        expected = date_range(
+            "2011-01-08", "2011-01-02", freq="-3D", tz=idx.tz, name="idx"
+        )
+        tm.assert_index_equal(result, expected)
+        assert result.freq == expected.freq
 
-            result = idx.take([0, 2, 4])
-            expected = date_range(
-                "2011-01-01", "2011-01-05", freq="2D", tz=idx.tz, name="idx"
-            )
-            tm.assert_index_equal(result, expected)
-            assert result.freq == expected.freq
+        result = idx.take([3, 2, 5])
+        expected = DatetimeIndex(
+            ["2011-01-04", "2011-01-03", "2011-01-06"],
+            dtype=idx.dtype,
+            freq=None,
+            name="idx",
+        )
+        tm.assert_index_equal(result, expected)
+        assert result.freq is None
 
-            result = idx.take([7, 4, 1])
-            expected = date_range(
-                "2011-01-08", "2011-01-02", freq="-3D", tz=idx.tz, name="idx"
-            )
-            tm.assert_index_equal(result, expected)
-            assert result.freq == expected.freq
-
-            result = idx.take([3, 2, 5])
-            expected = DatetimeIndex(
-                ["2011-01-04", "2011-01-03", "2011-01-06"],
-                freq=None,
-                tz=idx.tz,
-                name="idx",
-            )
-            tm.assert_index_equal(result, expected)
-            assert result.freq is None
-
-            result = idx.take([-3, 2, 5])
-            expected = DatetimeIndex(
-                ["2011-01-29", "2011-01-03", "2011-01-06"],
-                freq=None,
-                tz=idx.tz,
-                name="idx",
-            )
-            tm.assert_index_equal(result, expected)
-            assert result.freq is None
+        result = idx.take([-3, 2, 5])
+        expected = DatetimeIndex(
+            ["2011-01-29", "2011-01-03", "2011-01-06"],
+            dtype=idx.dtype,
+            freq=None,
+            name="idx",
+        )
+        tm.assert_index_equal(result, expected)
+        assert result.freq is None
 
     def test_take_invalid_kwargs(self):
         idx = date_range("2011-01-01", "2011-01-31", freq="D", name="idx")
@@ -299,11 +317,11 @@ class TestTake:
         idx = date_range(
             start="2010-01-01 09:00",
             end="2010-02-01 09:00",
-            freq="H",
+            freq="h",
             tz=tz,
             name="idx",
         )
-        expected = DatetimeIndex(dates, freq=None, name="idx", tz=tz)
+        expected = DatetimeIndex(dates, freq=None, name="idx", dtype=idx.dtype)
 
         taken1 = idx.take([5, 6, 8, 12])
         taken2 = idx[[5, 6, 8, 12]]
@@ -333,8 +351,7 @@ class TestTake:
         tm.assert_index_equal(result, expected)
 
         msg = (
-            "When allow_fill=True and fill_value is not None, "
-            "all indices must be >= -1"
+            "When allow_fill=True and fill_value is not None, all indices must be >= -1"
         )
         with pytest.raises(ValueError, match=msg):
             idx.take(np.array([1, 0, -2]), fill_value=True)
@@ -370,8 +387,7 @@ class TestTake:
         tm.assert_index_equal(result, expected)
 
         msg = (
-            "When allow_fill=True and fill_value is not None, "
-            "all indices must be >= -1"
+            "When allow_fill=True and fill_value is not None, all indices must be >= -1"
         )
         with pytest.raises(ValueError, match=msg):
             idx.take(np.array([1, 0, -2]), fill_value=True)
@@ -405,7 +421,7 @@ class TestGetLoc:
 
     def test_get_loc_time_obj(self):
         # time indexing
-        idx = date_range("2000-01-01", periods=24, freq="H")
+        idx = date_range("2000-01-01", periods=24, freq="h")
 
         result = idx.get_loc(time(12))
         expected = np.array([12])
@@ -415,19 +431,19 @@ class TestGetLoc:
         expected = np.array([])
         tm.assert_numpy_array_equal(result, expected, check_dtype=False)
 
-    def test_get_loc_time_obj2(self):
+    @pytest.mark.parametrize("offset", [-10, 10])
+    def test_get_loc_time_obj2(self, monkeypatch, offset):
         # GH#8667
-
-        from pandas._libs.index import _SIZE_CUTOFF
-
-        ns = _SIZE_CUTOFF + np.array([-100, 100], dtype=np.int64)
+        size_cutoff = 50
+        n = size_cutoff + offset
         key = time(15, 11, 30)
         start = key.hour * 3600 + key.minute * 60 + key.second
         step = 24 * 3600
 
-        for n in ns:
-            idx = date_range("2014-11-26", periods=n, freq="S")
-            ts = pd.Series(np.random.randn(n), index=idx)
+        with monkeypatch.context():
+            monkeypatch.setattr(libindex, "_SIZE_CUTOFF", size_cutoff)
+            idx = date_range("2014-11-26", periods=n, freq="s")
+            ts = pd.Series(np.random.default_rng(2).standard_normal(n), index=idx)
             locs = np.arange(start, n, step, dtype=np.intp)
 
             result = ts.index.get_loc(key)
@@ -510,6 +526,26 @@ class TestContains:
 
 
 class TestGetIndexer:
+    @td.skip_if_no("pyarrow")
+    @pytest.mark.parametrize("as_td", [True, False])
+    def test_get_indexer_pyarrow(self, as_td):
+        # GH#62277
+        index = date_range("2016-01-01", periods=3)
+        target = index.astype("timestamp[ns][pyarrow]")[::-1]
+        if as_td:
+            # Test duration dtypes while we're here
+            index = index - index[0]
+            target = target - target[-1]
+
+        result = index.get_indexer(target)
+
+        expected = np.array([2, 1, 0], dtype=np.intp)
+        tm.assert_numpy_array_equal(result, expected)
+
+        # Reversed op should work the same
+        result2 = target.get_indexer(index)
+        tm.assert_numpy_array_equal(result2, expected)
+
     def test_get_indexer_date_objs(self):
         rng = date_range("1/1/2000", periods=20)
 
@@ -601,7 +637,7 @@ class TestGetIndexer:
 class TestMaybeCastSliceBound:
     def test_maybe_cast_slice_bounds_empty(self):
         # GH#14354
-        empty_idx = date_range(freq="1H", periods=0, end="2015")
+        empty_idx = date_range(freq="1h", periods=0, end="2015", unit="ns")
 
         right = empty_idx._maybe_cast_slice_bound("2015-01-02", "right")
         exp = Timestamp("2015-01-02 23:59:59.999999999")
@@ -620,6 +656,32 @@ class TestMaybeCastSliceBound:
 
 
 class TestGetSliceBounds:
+    @pytest.mark.parametrize("as_td", [True, False])
+    def test_get_slice_bound_mismatched_unit(self, as_td):
+        # GH#63262
+        index = date_range(start="2000-01-01", freq="h", periods=8)
+
+        td = pd.Timedelta(1)
+        ts = Timestamp("2000-01-01 01:00:00")
+        start = ts - td
+        stop = ts + td
+        if as_td:
+            index = index - Timestamp(0).as_unit("us")
+            start = start - Timestamp(0).as_unit("us")
+            stop = stop - Timestamp(0).as_unit("us")
+
+        left = index.get_slice_bound(start, side="left")
+        assert left == 1
+        right = index.get_slice_bound(stop, side="right")
+        assert right == 2
+
+        # The user-facing behavior is slicing with .loc, so let's test that
+        #  explicitly while we're here.
+        ser = pd.Series(1, index=index)
+        result = ser.loc[start:stop]
+        expected = ser.iloc[1:2]
+        tm.assert_series_equal(result, expected)
+
     @pytest.mark.parametrize("box", [date, datetime, Timestamp])
     @pytest.mark.parametrize("side, expected", [("left", 4), ("right", 5)])
     def test_get_slice_bounds_datetime_within(
@@ -630,13 +692,16 @@ class TestGetSliceBounds:
         index = bdate_range("2000-01-03", "2000-02-11").tz_localize(tz)
         key = box(year=2000, month=1, day=7)
 
-        if tz is not None:
-            with pytest.raises(TypeError, match="Cannot compare tz-naive"):
-                # GH#36148 we require tzawareness-compat as of 2.0
-                index.get_slice_bound(key, side=side)
-        else:
-            result = index.get_slice_bound(key, side=side)
-            assert result == expected
+        warn = None if box is not date else Pandas4Warning
+        msg = "Slicing with a datetime.date object is deprecated"
+        with tm.assert_produces_warning(warn, match=msg):
+            if tz is not None:
+                with pytest.raises(TypeError, match="Cannot compare tz-naive"):
+                    # GH#36148 we require tzawareness-compat as of 2.0
+                    index.get_slice_bound(key, side=side)
+            else:
+                result = index.get_slice_bound(key, side=side)
+                assert result == expected
 
     @pytest.mark.parametrize("box", [datetime, Timestamp])
     @pytest.mark.parametrize("side", ["left", "right"])
