@@ -1,15 +1,11 @@
 import numpy as np
 import pytest
 
-from pandas.errors import SettingWithCopyError
-import pandas.util._test_decorators as td
-
 import pandas as pd
 from pandas import (
     DataFrame,
     MultiIndex,
     Series,
-    Timestamp,
     date_range,
     isna,
     notna,
@@ -91,11 +87,11 @@ class TestMultiIndexSetItem:
             np.random.default_rng(2).random((12, 4)), index=idx, columns=cols
         )
 
-        subidx = MultiIndex.from_tuples(
-            [("A", Timestamp("2015-01-01")), ("A", Timestamp("2015-02-01"))]
+        subidx = MultiIndex.from_arrays(
+            [["A", "A"], date_range("2015-01-01", "2015-02-01", freq="MS")]
         )
-        subcols = MultiIndex.from_tuples(
-            [("foo", Timestamp("2016-01-01")), ("foo", Timestamp("2016-02-01"))]
+        subcols = MultiIndex.from_arrays(
+            [["foo", "foo"], date_range("2016-01-01", "2016-02-01", freq="MS")]
         )
 
         vals = DataFrame(
@@ -127,9 +123,6 @@ class TestMultiIndexSetItem:
             expected=copy,
         )
 
-    # TODO(ArrayManager) df.loc["bar"] *= 2 doesn't raise an error but results in
-    # all NaNs -> doesn't work in the "split" path (also for BlockManager actually)
-    @td.skip_array_manager_not_yet_implemented
     def test_multiindex_setitem(self):
         # GH 3738
         # setting with a multi-index right hand side
@@ -175,7 +168,7 @@ class TestMultiIndexSetItem:
         )
 
         expected = df_orig.copy()
-        expected.iloc[[0, 2, 3]] *= 2
+        expected.iloc[[0, 1, 3]] *= 2
 
         idx = pd.IndexSlice
         df = df_orig.copy()
@@ -201,7 +194,7 @@ class TestMultiIndexSetItem:
         df.loc[4, "d"] = arr
         tm.assert_series_equal(df.loc[4, "d"], Series(arr, index=[8, 10], name="d"))
 
-    def test_multiindex_assignment_single_dtype(self, using_copy_on_write):
+    def test_multiindex_assignment_single_dtype(self):
         # GH3777 part 2b
         # single dtype
         arr = np.array([0.0, 1.0])
@@ -212,7 +205,6 @@ class TestMultiIndexSetItem:
             index=[[4, 4, 8], [8, 10, 12]],
             dtype=np.int64,
         )
-        view = df["c"].iloc[:2].values
 
         # arr can be losslessly cast to int, so this setitem is inplace
         df.loc[4, "c"] = arr
@@ -220,18 +212,12 @@ class TestMultiIndexSetItem:
         result = df.loc[4, "c"]
         tm.assert_series_equal(result, exp)
 
-        # extra check for inplace-ness
-        if not using_copy_on_write:
-            tm.assert_numpy_array_equal(view, exp.values)
-
         # arr + 0.5 cannot be cast losslessly to int, so we upcast
-        with tm.assert_produces_warning(
-            FutureWarning, match="item of incompatible dtype"
-        ):
+        with pytest.raises(TypeError, match="Invalid value"):
             df.loc[4, "c"] = arr + 0.5
-        result = df.loc[4, "c"]
-        exp = exp + 0.5
-        tm.assert_series_equal(result, exp)
+        # Upcast so that we can add .5
+        df = df.astype({"c": "float64"})
+        df.loc[4, "c"] = arr + 0.5
 
         # scalar ok
         df.loc[4, "c"] = 10
@@ -386,6 +372,7 @@ class TestMultiIndexSetItem:
         expected = df.loc[2000, 1, 6][["A", "B", "C"]]
         tm.assert_series_equal(result, expected)
 
+    @pytest.mark.filterwarnings("ignore:Setting a value on a view:FutureWarning")
     def test_loc_getitem_setitem_slice_integers(self, frame_or_series):
         index = MultiIndex(
             levels=[[0, 1, 2], [0, 2]], codes=[[0, 0, 1, 1, 2, 2], [0, 1, 0, 1, 0, 1]]
@@ -416,9 +403,7 @@ class TestMultiIndexSetItem:
         reindexed = dft.reindex(columns=[("foo", "two")])
         tm.assert_series_equal(reindexed["foo", "two"], s > s.median())
 
-    def test_set_column_scalar_with_loc(
-        self, multiindex_dataframe_random_data, using_copy_on_write
-    ):
+    def test_set_column_scalar_with_loc(self, multiindex_dataframe_random_data):
         frame = multiindex_dataframe_random_data
         subset = frame.index[[1, 4, 5]]
 
@@ -428,11 +413,8 @@ class TestMultiIndexSetItem:
         frame_original = frame.copy()
         col = frame["B"]
         col[subset] = 97
-        if using_copy_on_write:
-            # chained setitem doesn't work with CoW
-            tm.assert_frame_equal(frame, frame_original)
-        else:
-            assert (frame.loc[subset, "B"] == 97).all()
+        # chained setitem doesn't work with CoW
+        tm.assert_frame_equal(frame, frame_original)
 
     def test_nonunique_assignment_1750(self):
         df = DataFrame(
@@ -508,54 +490,49 @@ class TestSetitemWithExpansionMultiIndex:
         )
         tm.assert_frame_equal(df, expected)
 
+    def test_setitem_enlargement_multiindex_with_none(self):
+        # GH#59153
+        # enlarging a DataFrame with a MultiIndex containing None values
+        index = MultiIndex.from_tuples(
+            [("A", "a1"), ("A", "a2"), ("B", "b1"), ("B", None)]
+        )
+        df = DataFrame([(0.0, 6.0), (1.0, 5.0), (2.0, 4.0), (3.0, 7.0)], index=index)
+        df.loc[("A", None), :] = [12.0, 13.0]
+        expected_index = MultiIndex.from_tuples(
+            [("A", "a1"), ("A", "a2"), ("B", "b1"), ("B", None), ("A", None)]
+        )
+        expected = DataFrame(
+            [[0.0, 6.0], [1.0, 5.0], [2.0, 4.0], [3.0, 7.0], [12.0, 13.0]],
+            index=expected_index,
+            columns=[0, 1],
+        )
+        tm.assert_frame_equal(df, expected, check_index_type=False)
 
-@td.skip_array_manager_invalid_test  # df["foo"] select multiple columns -> .values
-# is not a view
-def test_frame_setitem_view_direct(
-    multiindex_dataframe_random_data, using_copy_on_write
-):
+
+def test_frame_setitem_view_direct(multiindex_dataframe_random_data):
     # this works because we are modifying the underlying array
     # really a no-no
     df = multiindex_dataframe_random_data.T
-    if using_copy_on_write:
-        with pytest.raises(ValueError, match="read-only"):
-            df["foo"].values[:] = 0
-        assert (df["foo"].values != 0).all()
-    else:
+    with pytest.raises(ValueError, match="read-only"):
         df["foo"].values[:] = 0
-        assert (df["foo"].values == 0).all()
+    assert (df["foo"].values != 0).all()
 
 
-def test_frame_setitem_copy_raises(
-    multiindex_dataframe_random_data, using_copy_on_write
-):
+def test_frame_setitem_copy_raises(multiindex_dataframe_random_data):
     # will raise/warn as its chained assignment
     df = multiindex_dataframe_random_data.T
-    if using_copy_on_write:
-        with tm.raises_chained_assignment_error():
-            df["foo"]["one"] = 2
-    else:
-        msg = "A value is trying to be set on a copy of a slice from a DataFrame"
-        with pytest.raises(SettingWithCopyError, match=msg):
-            df["foo"]["one"] = 2
+    with tm.raises_chained_assignment_error():
+        df["foo"]["one"] = 2
 
 
-def test_frame_setitem_copy_no_write(
-    multiindex_dataframe_random_data, using_copy_on_write
-):
+def test_frame_setitem_copy_no_write(multiindex_dataframe_random_data):
     frame = multiindex_dataframe_random_data.T
     expected = frame
     df = frame.copy()
-    if using_copy_on_write:
-        with tm.raises_chained_assignment_error():
-            df["foo"]["one"] = 2
-    else:
-        msg = "A value is trying to be set on a copy of a slice from a DataFrame"
-        with pytest.raises(SettingWithCopyError, match=msg):
-            df["foo"]["one"] = 2
+    with tm.raises_chained_assignment_error():
+        df["foo"]["one"] = 2
 
-    result = df
-    tm.assert_frame_equal(result, expected)
+    tm.assert_frame_equal(df, expected)
 
 
 def test_frame_setitem_partial_multiindex():
