@@ -11,12 +11,12 @@ from pathlib import Path
 import platform
 import re
 from urllib.error import URLError
+import uuid
 from zipfile import BadZipFile
 
 import numpy as np
 import pytest
 
-from pandas.compat import is_platform_windows
 import pandas.util._test_decorators as td
 
 import pandas as pd
@@ -29,9 +29,6 @@ from pandas import (
 )
 import pandas._testing as tm
 
-if is_platform_windows():
-    pytestmark = pytest.mark.single_cpu
-
 read_ext_params = [".xls", ".xlsx", ".xlsm", ".xlsb", ".ods"]
 engine_params = [
     # Add any engines to test here
@@ -87,7 +84,7 @@ def _transfer_marks(engine, read_ext):
     engine gives us a pytest.param object with some marks, read_ext is just
     a string.  We need to generate a new pytest.param inheriting the marks.
     """
-    values = engine.values + (read_ext,)
+    values = (*engine.values, read_ext)
     new_param = pytest.param(values, marks=engine.marks)
     return new_param
 
@@ -120,6 +117,13 @@ def read_ext(engine_and_read_ext):
     return read_ext
 
 
+@pytest.fixture
+def tmp_excel(read_ext, tmp_path):
+    tmp = tmp_path / f"{uuid.uuid4()}{read_ext}"
+    tmp.touch()
+    return str(tmp)
+
+
 @pytest.fixture
 def df_ref(datapath):
     """
@@ -131,10 +135,10 @@ def df_ref(datapath):
 
 
 def get_exp_unit(read_ext: str, engine: str | None) -> str:
-    return "ns"
+    return "us"
 
 
-def adjust_expected(expected: DataFrame, read_ext: str, engine: str) -> None:
+def adjust_expected(expected: DataFrame, read_ext: str, engine: str | None) -> None:
     expected.index.name = None
     unit = get_exp_unit(read_ext, engine)
     # error: "Index" has no attribute "as_unit"
@@ -151,6 +155,36 @@ def xfail_datetimes_with_pyxlsb(engine, request):
 
 
 class TestReaders:
+    @pytest.mark.parametrize("col", [[True, None, False], [True], [True, False]])
+    def test_read_excel_type_check(self, col, tmp_excel, read_ext):
+        # GH 58159
+        if read_ext in (".xlsb", ".xls"):
+            pytest.skip(f"No engine for filetype: '{read_ext}'")
+        df = DataFrame({"bool_column": col}, dtype="boolean")
+        df.to_excel(tmp_excel, index=False)
+        df2 = pd.read_excel(tmp_excel, dtype={"bool_column": "boolean"})
+        tm.assert_frame_equal(df, df2)
+
+    def test_pass_none_type(self, datapath):
+        # GH 58159
+        f_path = datapath("io", "data", "excel", "test_none_type.xlsx")
+
+        with pd.ExcelFile(f_path) as excel:
+            parsed = pd.read_excel(
+                excel,
+                sheet_name="Sheet1",
+                keep_default_na=True,
+                na_values=["nan", "None", "abcd"],
+                dtype="boolean",
+                engine="openpyxl",
+            )
+        expected = DataFrame(
+            {"Test": [True, None, False, None, False, None, True]},
+            dtype="boolean",
+        )
+
+        tm.assert_frame_equal(parsed, expected)
+
     @pytest.fixture(autouse=True)
     def cd_and_set_engine(self, engine, datapath, monkeypatch):
         """
@@ -560,25 +594,21 @@ class TestReaders:
         [
             (
                 None,
-                DataFrame(
-                    {
-                        "a": [1, 2, 3, 4],
-                        "b": [2.5, 3.5, 4.5, 5.5],
-                        "c": [1, 2, 3, 4],
-                        "d": [1.0, 2.0, np.nan, 4.0],
-                    }
-                ),
+                {
+                    "a": [1, 2, 3, 4],
+                    "b": [2.5, 3.5, 4.5, 5.5],
+                    "c": [1, 2, 3, 4],
+                    "d": [1.0, 2.0, np.nan, 4.0],
+                },
             ),
             (
                 {"a": "float64", "b": "float32", "c": str, "d": str},
-                DataFrame(
-                    {
-                        "a": Series([1, 2, 3, 4], dtype="float64"),
-                        "b": Series([2.5, 3.5, 4.5, 5.5], dtype="float32"),
-                        "c": Series(["001", "002", "003", "004"], dtype="str"),
-                        "d": Series(["1", "2", np.nan, "4"], dtype="str"),
-                    },
-                ),
+                {
+                    "a": Series([1, 2, 3, 4], dtype="float64"),
+                    "b": Series([2.5, 3.5, 4.5, 5.5], dtype="float32"),
+                    "c": Series(["001", "002", "003", "004"], dtype="str"),
+                    "d": Series(["1", "2", np.nan, "4"], dtype="str"),
+                },
             ),
         ],
     )
@@ -587,9 +617,10 @@ class TestReaders:
         basename = "testdtype"
 
         actual = pd.read_excel(basename + read_ext, dtype=dtype)
+        expected = DataFrame(expected)
         tm.assert_frame_equal(actual, expected)
 
-    def test_dtype_backend(self, read_ext, dtype_backend, engine):
+    def test_dtype_backend(self, read_ext, dtype_backend, engine, tmp_excel):
         # GH#36712
         if read_ext in (".xlsb", ".xls"):
             pytest.skip(f"No engine for filetype: '{read_ext}'")
@@ -608,11 +639,10 @@ class TestReaders:
                 "j": Series([pd.NA, pd.NA], dtype="Int64"),
             }
         )
-        with tm.ensure_clean(read_ext) as file_path:
-            df.to_excel(file_path, sheet_name="test", index=False)
-            result = pd.read_excel(
-                file_path, sheet_name="test", dtype_backend=dtype_backend
-            )
+        df.to_excel(tmp_excel, sheet_name="test", index=False)
+        result = pd.read_excel(
+            tmp_excel, sheet_name="test", dtype_backend=dtype_backend
+        )
         if dtype_backend == "pyarrow":
             import pyarrow as pa
 
@@ -624,6 +654,10 @@ class TestReaders:
                     for col in df.columns
                 }
             )
+
+            # pandas uses large_string by default, but pyarrow infers string
+            expected["d"] = expected["d"].astype(pd.ArrowDtype(pa.string()))
+            expected["h"] = expected["h"].astype(pd.ArrowDtype(pa.string()))
             # pyarrow by default infers timestamp resolution as us, not ns
             expected["i"] = ArrowExtensionArray(
                 expected["i"].array._pa_array.cast(pa.timestamp(unit="us"))
@@ -637,50 +671,48 @@ class TestReaders:
 
         tm.assert_frame_equal(result, expected)
 
-    def test_dtype_backend_and_dtype(self, read_ext):
+    def test_dtype_backend_and_dtype(self, read_ext, tmp_excel):
         # GH#36712
         if read_ext in (".xlsb", ".xls"):
             pytest.skip(f"No engine for filetype: '{read_ext}'")
 
         df = DataFrame({"a": [np.nan, 1.0], "b": [2.5, np.nan]})
-        with tm.ensure_clean(read_ext) as file_path:
-            df.to_excel(file_path, sheet_name="test", index=False)
-            result = pd.read_excel(
-                file_path,
-                sheet_name="test",
-                dtype_backend="numpy_nullable",
-                dtype="float64",
-            )
+        df.to_excel(tmp_excel, sheet_name="test", index=False)
+        result = pd.read_excel(
+            tmp_excel,
+            sheet_name="test",
+            dtype_backend="numpy_nullable",
+            dtype="float64",
+        )
         tm.assert_frame_equal(result, df)
 
-    def test_dtype_backend_string(self, read_ext, string_storage):
+    def test_dtype_backend_string(self, read_ext, string_storage, tmp_excel):
         # GH#36712
         if read_ext in (".xlsb", ".xls"):
             pytest.skip(f"No engine for filetype: '{read_ext}'")
 
+        df = DataFrame(
+            {
+                "a": np.array(["a", "b"], dtype=np.object_),
+                "b": np.array(["x", pd.NA], dtype=np.object_),
+            }
+        )
+        df.to_excel(tmp_excel, sheet_name="test", index=False)
+
         with pd.option_context("mode.string_storage", string_storage):
-            df = DataFrame(
-                {
-                    "a": np.array(["a", "b"], dtype=np.object_),
-                    "b": np.array(["x", pd.NA], dtype=np.object_),
-                }
+            result = pd.read_excel(
+                tmp_excel, sheet_name="test", dtype_backend="numpy_nullable"
             )
 
-            with tm.ensure_clean(read_ext) as file_path:
-                df.to_excel(file_path, sheet_name="test", index=False)
-                result = pd.read_excel(
-                    file_path, sheet_name="test", dtype_backend="numpy_nullable"
-                )
-
-            expected = DataFrame(
-                {
-                    "a": Series(["a", "b"], dtype=pd.StringDtype(string_storage)),
-                    "b": Series(["x", None], dtype=pd.StringDtype(string_storage)),
-                }
-            )
-            # the storage of the str columns' Index is also affected by the
-            # string_storage setting -> ignore that for checking the result
-            tm.assert_frame_equal(result, expected, check_column_type=False)
+        expected = DataFrame(
+            {
+                "a": Series(["a", "b"], dtype=pd.StringDtype(string_storage)),
+                "b": Series(["x", None], dtype=pd.StringDtype(string_storage)),
+            }
+        )
+        # the storage of the str columns' Index is also affected by the
+        # string_storage setting -> ignore that for checking the result
+        tm.assert_frame_equal(result, expected, check_column_type=False)
 
     @pytest.mark.parametrize("dtypes, exp_value", [({}, 1), ({"a.1": "int64"}, 1)])
     def test_dtype_mangle_dup_cols(self, read_ext, dtypes, exp_value):
@@ -879,8 +911,7 @@ class TestReaders:
 
             error = XLRDError
             msg = (
-                "Unsupported format, or corrupt file: Expected BOF "
-                "record; found b'foo'"
+                "Unsupported format, or corrupt file: Expected BOF record; found b'foo'"
             )
         elif engine == "calamine":
             from python_calamine import CalamineError
@@ -904,29 +935,27 @@ class TestReaders:
 
     @td.skip_if_not_us_locale
     @pytest.mark.single_cpu
-    def test_read_from_s3_url(self, read_ext, s3_public_bucket, s3so):
-        # Bucket created in tests/io/conftest.py
+    def test_read_from_s3_url(self, read_ext, s3_bucket_public, s3so):
         with open("test1" + read_ext, "rb") as f:
-            s3_public_bucket.put_object(Key="test1" + read_ext, Body=f)
+            s3_bucket_public.put_object(Key="test1" + read_ext, Body=f)
 
-        url = f"s3://{s3_public_bucket.name}/test1" + read_ext
+        url = f"s3://{s3_bucket_public.name}/test1" + read_ext
 
         url_table = pd.read_excel(url, storage_options=s3so)
         local_table = pd.read_excel("test1" + read_ext)
         tm.assert_frame_equal(url_table, local_table)
 
     @pytest.mark.single_cpu
-    def test_read_from_s3_object(self, read_ext, s3_public_bucket, s3so):
+    def test_read_from_s3_object(self, read_ext, s3_bucket_public, s3so):
         # GH 38788
-        # Bucket created in tests/io/conftest.py
         with open("test1" + read_ext, "rb") as f:
-            s3_public_bucket.put_object(Key="test1" + read_ext, Body=f)
+            s3_bucket_public.put_object(Key="test1" + read_ext, Body=f)
 
         import s3fs
 
         s3 = s3fs.S3FileSystem(**s3so)
 
-        with s3.open(f"s3://{s3_public_bucket.name}/test1" + read_ext) as f:
+        with s3.open(f"s3://{s3_bucket_public.name}/test1" + read_ext) as f:
             url_table = pd.read_excel(f)
 
         local_table = pd.read_excel("test1" + read_ext)
@@ -957,19 +986,6 @@ class TestReaders:
 
         tm.assert_frame_equal(expected, actual)
 
-    @td.skip_if_no("py.path")
-    def test_read_from_py_localpath(self, read_ext):
-        # GH12655
-        from py.path import local as LocalPath
-
-        str_path = os.path.join("test1" + read_ext)
-        expected = pd.read_excel(str_path, sheet_name="Sheet1", index_col=0)
-
-        path_obj = LocalPath().join("test1" + read_ext)
-        actual = pd.read_excel(path_obj, sheet_name="Sheet1", index_col=0)
-
-        tm.assert_frame_equal(expected, actual)
-
     def test_close_from_py_localpath(self, read_ext):
         # GH31467
         str_path = os.path.join("test1" + read_ext)
@@ -1064,7 +1080,7 @@ class TestReaders:
         tm.assert_frame_equal(actual, expected)
 
         # "mi_column_name" sheet
-        expected.index = list(range(4))
+        expected.index = range(4)
         expected.columns = mi.set_names(["c1", "c2"])
         actual = pd.read_excel(
             mi_file, sheet_name="mi_column_name", header=[0, 1], index_col=0
@@ -1116,7 +1132,6 @@ class TestReaders:
         mi = MultiIndex.from_product([["foo", "bar"], ["a", "b"]], names=["c1", "c2"])
 
         unit = get_exp_unit(read_ext, engine)
-
         expected = DataFrame(
             [
                 [1, 2.5, pd.Timestamp("2015-01-01"), True],
@@ -1452,17 +1467,10 @@ class TestReaders:
 
 
 class TestExcelFileRead:
-    def test_deprecate_bytes_input(self, engine, read_ext):
+    def test_raises_bytes_input(self, engine, read_ext):
         # GH 53830
-        msg = (
-            "Passing bytes to 'read_excel' is deprecated and "
-            "will be removed in a future version. To read from a "
-            "byte string, wrap it in a `BytesIO` object."
-        )
-
-        with tm.assert_produces_warning(
-            FutureWarning, match=msg, raise_on_extra_warnings=False
-        ):
+        msg = "Expected file path name or file-like object"
+        with pytest.raises(TypeError, match=msg):
             with open("test1" + read_ext, "rb") as f:
                 pd.read_excel(f.read(), engine=engine)
 
@@ -1681,6 +1689,7 @@ class TestExcelFileRead:
             actual = pd.read_excel(excel, header=[0, 1], index_col=0, engine=engine)
 
         unit = get_exp_unit(read_ext, engine)
+
         dti = pd.DatetimeIndex(["2020-02-29", "2020-03-01"], dtype=f"M8[{unit}]")
         expected_column_index = MultiIndex.from_arrays(
             [dti[:1], dti[1:]],
@@ -1712,7 +1721,7 @@ class TestExcelFileRead:
         with pd.ExcelFile("chartsheet" + read_ext) as excel:
             assert excel.sheet_names == ["Sheet1"]
 
-    def test_corrupt_files_closed(self, engine, read_ext):
+    def test_corrupt_files_closed(self, engine, tmp_excel):
         # GH41778
         errors = (BadZipFile,)
         if engine is None:
@@ -1726,10 +1735,9 @@ class TestExcelFileRead:
 
             errors = (CalamineError,)
 
-        with tm.ensure_clean(f"corrupt{read_ext}") as file:
-            Path(file).write_text("corrupt", encoding="utf-8")
-            with tm.assert_produces_warning(False):
-                try:
-                    pd.ExcelFile(file, engine=engine)
-                except errors:
-                    pass
+        Path(tmp_excel).write_text("corrupt", encoding="utf-8")
+        with tm.assert_produces_warning(False):
+            try:
+                pd.ExcelFile(tmp_excel, engine=engine)
+            except errors:
+                pass
