@@ -2,21 +2,22 @@
 Tests that work on both the Python and C engines but do not have a
 specific classification into the other test modules.
 """
+
 from io import StringIO
 
-import numpy as np
 import pytest
 
-from pandas import (
-    DataFrame,
-    option_context,
-)
+from pandas import DataFrame
 import pandas._testing as tm
 
+pytestmark = pytest.mark.filterwarnings(
+    "ignore:Passing a BlockManager to DataFrame:DeprecationWarning"
+)
+
 xfail_pyarrow = pytest.mark.usefixtures("pyarrow_xfail")
 
 
-@xfail_pyarrow
+@xfail_pyarrow  # AssertionError: DataFrame.index are different
 @pytest.mark.parametrize("na_filter", [True, False])
 def test_inf_parsing(all_parsers, na_filter):
     parser = all_parsers
@@ -40,7 +41,7 @@ j,-inF"""
     tm.assert_frame_equal(result, expected)
 
 
-@xfail_pyarrow
+@xfail_pyarrow  # AssertionError: DataFrame.index are different
 @pytest.mark.parametrize("na_filter", [True, False])
 def test_infinity_parsing(all_parsers, na_filter):
     parser = all_parsers
@@ -56,13 +57,3 @@ c,+Infinity
     )
     result = parser.read_csv(StringIO(data), index_col=0, na_filter=na_filter)
     tm.assert_frame_equal(result, expected)
-
-
-def test_read_csv_with_use_inf_as_na(all_parsers):
-    # https://github.com/pandas-dev/pandas/issues/35493
-    parser = all_parsers
-    data = "1.0\nNaN\n3.0"
-    with option_context("use_inf_as_na", True):
-        result = parser.read_csv(StringIO(data), header=None)
-    expected = DataFrame([1.0, np.nan, 3.0])
-    tm.assert_frame_equal(result, expected)
