@@ -5,7 +5,12 @@ these tests out of this module as soon as the Python parser can accept
 further arguments when parsing.
 """
 
-from io import BytesIO, StringIO, TextIOWrapper
+from decimal import Decimal
+from io import (
+    BytesIO,
+    StringIO,
+    TextIOWrapper,
+)
 import mmap
 import os
 import tarfile
@@ -13,11 +18,18 @@ import tarfile
 import numpy as np
 import pytest
 
-from pandas.errors import ParserError
+from pandas.compat import WASM
+from pandas.errors import (
+    ParserError,
+    ParserWarning,
+)
 import pandas.util._test_decorators as td
 
-from pandas import DataFrame, concat
-import pandas.util.testing as tm
+from pandas import (
+    DataFrame,
+    concat,
+)
+import pandas._testing as tm
 
 
 @pytest.mark.parametrize(
@@ -35,32 +47,12 @@ def test_buffer_overflow(c_parser_only, malformed):
         parser.read_csv(StringIO(malformed))
 
 
-def test_buffer_rd_bytes(c_parser_only):
-    # see gh-12098: src->buffer in the C parser can be freed twice leading
-    # to a segfault if a corrupt gzip file is read with 'read_csv', and the
-    # buffer is filled more than once before gzip raises an Exception.
-
-    data = (
-        "\x1F\x8B\x08\x00\x00\x00\x00\x00\x00\x03\xED\xC3\x41\x09"
-        "\x00\x00\x08\x00\xB1\xB7\xB6\xBA\xFE\xA5\xCC\x21\x6C\xB0"
-        "\xA6\x4D" + "\x55" * 267 + "\x7D\xF7\x00\x91\xE0\x47\x97\x14\x38\x04\x00"
-        "\x1f\x8b\x08\x00VT\x97V\x00\x03\xed]\xefO"
-    )
-    parser = c_parser_only
-
-    for _ in range(100):
-        try:
-            parser.read_csv(StringIO(data), compression="gzip", delim_whitespace=True)
-        except Exception:
-            pass
-
-
 def test_delim_whitespace_custom_terminator(c_parser_only):
     # See gh-12912
     data = "a b c~1 2 3~4 5 6~7 8 9"
     parser = c_parser_only
 
-    df = parser.read_csv(StringIO(data), lineterminator="~", delim_whitespace=True)
+    df = parser.read_csv(StringIO(data), lineterminator="~", sep=r"\s+")
     expected = DataFrame([[1, 2, 3], [4, 5, 6], [7, 8, 9]], columns=["a", "b", "c"])
     tm.assert_frame_equal(df, expected)
 
@@ -97,14 +89,16 @@ nan 2
 3.0 3
 """
     # fallback casting, but not castable
-    with pytest.raises(ValueError, match="cannot safely convert"):
-        parser.read_csv(
-            StringIO(data),
-            sep=r"\s+",
-            header=None,
-            names=["a", "b"],
-            dtype={"a": np.int32},
-        )
+    if not WASM:  # no fp exception support in wasm
+        with pytest.raises(ValueError, match="cannot safely convert"):
+            with tm.assert_produces_warning(RuntimeWarning, check_stacklevel=False):
+                parser.read_csv(
+                    StringIO(data),
+                    sep=r"\s+",
+                    header=None,
+                    names=["a", "b"],
+                    dtype={"a": np.int32},
+                )
 
 
 @pytest.mark.parametrize(
@@ -116,73 +110,76 @@ nan 2
                 "the dtype datetime64 is not supported for parsing, "
                 "pass this column using parse_dates instead"
             ),
-            dict(dtype={"A": "datetime64", "B": "float64"}),
+            {"dtype": {"A": "datetime64", "B": "float64"}},
         ),
         (
             (
                 "the dtype datetime64 is not supported for parsing, "
                 "pass this column using parse_dates instead"
             ),
-            dict(dtype={"A": "datetime64", "B": "float64"}, parse_dates=["B"]),
+            {"dtype": {"A": "datetime64", "B": "float64"}, "parse_dates": ["B"]},
         ),
         (
             "the dtype timedelta64 is not supported for parsing",
-            dict(dtype={"A": "timedelta64", "B": "float64"}),
+            {"dtype": {"A": "timedelta64", "B": "float64"}},
+        ),
+        (
+            f"the dtype {tm.ENDIAN}U8 is not supported for parsing",
+            {"dtype": {"A": "U8"}},
         ),
-        ("the dtype <U8 is not supported for parsing", dict(dtype={"A": "U8"})),
     ],
-    ids=["dt64-0", "dt64-1", "td64", "<U8"],
+    ids=["dt64-0", "dt64-1", "td64", f"{tm.ENDIAN}U8"],
 )
-def test_unsupported_dtype(c_parser_only, match, kwargs):
+def test_unsupported_dtype(c_parser_only, match, kwargs, temp_file):
     parser = c_parser_only
     df = DataFrame(
-        np.random.rand(5, 2), columns=list("AB"), index=["1A", "1B", "1C", "1D", "1E"]
+        np.random.default_rng(2).random((5, 2)),
+        columns=list("AB"),
+        index=["1A", "1B", "1C", "1D", "1E"],
     )
 
-    with tm.ensure_clean("__unsupported_dtype__.csv") as path:
-        df.to_csv(path)
+    df.to_csv(temp_file)
 
-        with pytest.raises(TypeError, match=match):
-            parser.read_csv(path, index_col=0, **kwargs)
+    with pytest.raises(TypeError, match=match):
+        parser.read_csv(temp_file, index_col=0, **kwargs)
 
 
 @td.skip_if_32bit
-def test_precise_conversion(c_parser_only):
-    from decimal import Decimal
-
+@pytest.mark.slow
+# test numbers between 1 and 2
+@pytest.mark.parametrize("num", np.linspace(1.0, 2.0, num=21))
+def test_precise_conversion(c_parser_only, num):
     parser = c_parser_only
 
     normal_errors = []
     precise_errors = []
 
-    # test numbers between 1 and 2
-    for num in np.linspace(1.0, 2.0, num=500):
-        # 25 decimal digits of precision
-        text = "a\n{0:.25}".format(num)
+    def error(val: float, actual_val: Decimal) -> Decimal:
+        return abs(Decimal(f"{val:.100}") - actual_val)
 
-        normal_val = float(parser.read_csv(StringIO(text))["a"][0])
-        precise_val = float(
-            parser.read_csv(StringIO(text), float_precision="high")["a"][0]
-        )
-        roundtrip_val = float(
-            parser.read_csv(StringIO(text), float_precision="round_trip")["a"][0]
-        )
-        actual_val = Decimal(text[2:])
+    # 25 decimal digits of precision
+    text = f"a\n{num:.25}"
 
-        def error(val):
-            return abs(Decimal("{0:.100}".format(val)) - actual_val)
+    normal_val = float(
+        parser.read_csv(StringIO(text), float_precision="legacy")["a"][0]
+    )
+    precise_val = float(parser.read_csv(StringIO(text), float_precision="high")["a"][0])
+    roundtrip_val = float(
+        parser.read_csv(StringIO(text), float_precision="round_trip")["a"][0]
+    )
+    actual_val = Decimal(text[2:])
 
-        normal_errors.append(error(normal_val))
-        precise_errors.append(error(precise_val))
+    normal_errors.append(error(normal_val, actual_val))
+    precise_errors.append(error(precise_val, actual_val))
 
-        # round-trip should match float()
-        assert roundtrip_val == float(text[2:])
+    # round-trip should match float()
+    assert roundtrip_val == float(text[2:])
 
     assert sum(precise_errors) <= sum(normal_errors)
     assert max(precise_errors) <= max(normal_errors)
 
 
-def test_usecols_dtypes(c_parser_only):
+def test_usecols_dtypes(c_parser_only, using_infer_string):
     parser = c_parser_only
     data = """\
 1,2,3
@@ -207,8 +204,12 @@ def test_usecols_dtypes(c_parser_only):
         dtype={"b": int, "c": float},
     )
 
-    assert (result.dtypes == [object, np.int, np.float]).all()
-    assert (result2.dtypes == [object, np.float]).all()
+    if using_infer_string:
+        assert (result.dtypes == ["string", int, float]).all()
+        assert (result2.dtypes == ["string", float]).all()
+    else:
+        assert (result.dtypes == [object, int, float]).all()
+        assert (result2.dtypes == [object, float]).all()
 
 
 def test_disable_bool_parsing(c_parser_only):
@@ -286,7 +287,9 @@ def test_tokenize_CR_with_quoting(c_parser_only):
     tm.assert_frame_equal(result, expected)
 
 
-def test_grow_boundary_at_cap(c_parser_only):
+@pytest.mark.slow
+@pytest.mark.parametrize("count", [3 * 2**n for n in range(6)])
+def test_grow_boundary_at_cap(c_parser_only, count):
     # See gh-12494
     #
     # Cause of error was that the C parser
@@ -295,21 +298,18 @@ def test_grow_boundary_at_cap(c_parser_only):
     # to capacity, which would later cause a
     # buffer overflow error when checking the
     # EOF terminator of the CSV stream.
+    # 3 * 2^n commas was observed to break the parser
     parser = c_parser_only
 
-    def test_empty_header_read(count):
-        s = StringIO("," * count)
-        expected = DataFrame(
-            columns=["Unnamed: {i}".format(i=i) for i in range(count + 1)]
-        )
+    with StringIO("," * count) as s:
+        expected = DataFrame(columns=[f"Unnamed: {i}" for i in range(count + 1)])
         df = parser.read_csv(s)
-        tm.assert_frame_equal(df, expected)
-
-    for cnt in range(1, 101):
-        test_empty_header_read(cnt)
+    tm.assert_frame_equal(df, expected)
 
 
-def test_parse_trim_buffers(c_parser_only):
+@pytest.mark.slow
+@pytest.mark.parametrize("encoding", [None, "utf-8"])
+def test_parse_trim_buffers(c_parser_only, encoding):
     # This test is part of a bugfix for gh-13703. It attempts to
     # to stress the system memory allocator, to cause it to move the
     # stream buffer and either let the OS reclaim the region, or let
@@ -320,6 +320,9 @@ def test_parse_trim_buffers(c_parser_only):
     # times it fails due to memory corruption, which causes the
     # loaded DataFrame to differ from the expected one.
 
+    # Also force 'utf-8' encoding, so that `_string_convert` would take
+    # a different execution branch.
+
     parser = c_parser_only
 
     # Generate a large mixed-type CSV file on-the-fly (one record is
@@ -375,25 +378,16 @@ def test_parse_trim_buffers(c_parser_only):
     )
 
     # Iterate over the CSV file in chunks of `chunksize` lines
-    chunks_ = parser.read_csv(
-        StringIO(csv_data), header=None, dtype=object, chunksize=chunksize
-    )
-    result = concat(chunks_, axis=0, ignore_index=True)
-
-    # Check for data corruption if there was no segfault
-    tm.assert_frame_equal(result, expected)
-
-    # This extra test was added to replicate the fault in gh-5291.
-    # Force 'utf-8' encoding, so that `_string_convert` would take
-    # a different execution branch.
-    chunks_ = parser.read_csv(
+    with parser.read_csv(
         StringIO(csv_data),
         header=None,
         dtype=object,
         chunksize=chunksize,
-        encoding="utf_8",
-    )
-    result = concat(chunks_, axis=0, ignore_index=True)
+        encoding=encoding,
+    ) as chunks_:
+        result = concat(chunks_, axis=0, ignore_index=True)
+
+    # Check for data corruption if there was no segfault
     tm.assert_frame_equal(result, expected)
 
 
@@ -465,7 +459,7 @@ def test_data_after_quote(c_parser_only):
     tm.assert_frame_equal(result, expected)
 
 
-def test_comment_whitespace_delimited(c_parser_only, capsys):
+def test_comment_whitespace_delimited(c_parser_only):
     parser = c_parser_only
     test_input = """\
 1 2
@@ -478,18 +472,17 @@ def test_comment_whitespace_delimited(c_parser_only, capsys):
 8# 1 field, NaN
 9 2 3 # skipped line
 # comment"""
-    df = parser.read_csv(
-        StringIO(test_input),
-        comment="#",
-        header=None,
-        delimiter="\\s+",
-        skiprows=0,
-        error_bad_lines=False,
-    )
-    captured = capsys.readouterr()
-    # skipped lines 2, 3, 4, 9
-    for line_num in (2, 3, 4, 9):
-        assert "Skipping line {}".format(line_num) in captured.err
+    with tm.assert_produces_warning(
+        ParserWarning, match="Skipping line", check_stacklevel=False
+    ):
+        df = parser.read_csv(
+            StringIO(test_input),
+            comment="#",
+            header=None,
+            delimiter="\\s+",
+            skiprows=0,
+            on_bad_lines="warn",
+        )
     expected = DataFrame([[1, 2], [5, 2], [6, 2], [7, np.nan], [8, np.nan]])
     tm.assert_frame_equal(df, expected)
 
@@ -516,8 +509,8 @@ def test_file_like_no_next(c_parser_only):
 
 def test_buffer_rd_bytes_bad_unicode(c_parser_only):
     # see gh-22748
-    t = BytesIO(b"\xB0")
-    t = TextIOWrapper(t, encoding="ascii", errors="surrogateescape")
+    t = BytesIO(b"\xb0")
+    t = TextIOWrapper(t, encoding="UTF-8", errors="surrogateescape")
     msg = "'utf-8' codec can't encode character"
     with pytest.raises(UnicodeError, match=msg):
         c_parser_only.read_csv(t, encoding="UTF-8")
@@ -541,21 +534,6 @@ def test_read_tarfile(c_parser_only, csv_dir_path, tar_suffix):
         tm.assert_frame_equal(out, expected)
 
 
-@pytest.mark.high_memory
-def test_bytes_exceed_2gb(c_parser_only):
-    # see gh-16798
-    #
-    # Read from a "CSV" that has a column larger than 2GB.
-    parser = c_parser_only
-
-    if parser.low_memory:
-        pytest.skip("not a high_memory test")
-
-    csv = StringIO("strings\n" + "\n".join(["x" * (1 << 20) for _ in range(2100)]))
-    df = parser.read_csv(csv)
-    assert not df.empty
-
-
 def test_chunk_whitespace_on_boundary(c_parser_only):
     # see gh-9735: this issue is C parser-specific (bug when
     # parsing whitespace and characters at chunk boundary)
@@ -571,29 +549,96 @@ def test_chunk_whitespace_on_boundary(c_parser_only):
     tm.assert_frame_equal(result, expected)
 
 
+@pytest.mark.skipif(WASM, reason="limited file system access on WASM")
 def test_file_handles_mmap(c_parser_only, csv1):
     # gh-14418
     #
     # Don't close user provided file handles.
     parser = c_parser_only
 
-    with open(csv1, "r") as f:
-        m = mmap.mmap(f.fileno(), 0, access=mmap.ACCESS_READ)
-        parser.read_csv(m)
-
-        assert not m.closed
-        m.close()
+    with open(csv1, encoding="utf-8") as f:
+        with mmap.mmap(f.fileno(), 0, access=mmap.ACCESS_READ) as m:
+            parser.read_csv(m)
+            assert not m.closed
 
 
-def test_file_binary_mode(c_parser_only):
+def test_file_binary_mode(c_parser_only, temp_file):
     # see gh-23779
     parser = c_parser_only
     expected = DataFrame([[1, 2, 3], [4, 5, 6]])
 
-    with tm.ensure_clean() as path:
-        with open(path, "w") as f:
-            f.write("1,2,3\n4,5,6")
+    path = temp_file
+    with open(path, "w", encoding="utf-8") as f:
+        f.write("1,2,3\n4,5,6")
 
-        with open(path, "rb") as f:
-            result = parser.read_csv(f, header=None)
-            tm.assert_frame_equal(result, expected)
+    with open(path, "rb") as f:
+        result = parser.read_csv(f, header=None)
+        tm.assert_frame_equal(result, expected)
+
+
+def test_unix_style_breaks(c_parser_only, temp_file):
+    # GH 11020
+    parser = c_parser_only
+    path = temp_file
+    with open(path, "w", newline="\n", encoding="utf-8") as f:
+        f.write("blah\n\ncol_1,col_2,col_3\n\n")
+    result = parser.read_csv(path, skiprows=2, encoding="utf-8", engine="c")
+    expected = DataFrame(columns=["col_1", "col_2", "col_3"])
+    tm.assert_frame_equal(result, expected)
+
+
+@pytest.mark.parametrize("float_precision", [None, "legacy", "high", "round_trip"])
+@pytest.mark.parametrize(
+    "data,thousands,decimal",
+    [
+        (
+            """A|B|C
+1|2,334.01|5
+10|13|10.
+""",
+            ",",
+            ".",
+        ),
+        (
+            """A|B|C
+1|2.334,01|5
+10|13|10,
+""",
+            ".",
+            ",",
+        ),
+    ],
+)
+def test_1000_sep_with_decimal(
+    c_parser_only, data, thousands, decimal, float_precision
+):
+    parser = c_parser_only
+    expected = DataFrame({"A": [1, 10], "B": [2334.01, 13], "C": [5, 10.0]})
+
+    result = parser.read_csv(
+        StringIO(data),
+        sep="|",
+        thousands=thousands,
+        decimal=decimal,
+        float_precision=float_precision,
+    )
+    tm.assert_frame_equal(result, expected)
+
+
+def test_float_precision_options(c_parser_only):
+    # GH 17154, 36228
+    parser = c_parser_only
+    s = "foo\n243.164\n"
+    df = parser.read_csv(StringIO(s))
+    df2 = parser.read_csv(StringIO(s), float_precision="high")
+
+    tm.assert_frame_equal(df, df2)
+
+    df3 = parser.read_csv(StringIO(s), float_precision="legacy")
+
+    assert not df.iloc[0, 0] == df3.iloc[0, 0]
+
+    msg = "Unrecognized float_precision option: junk"
+
+    with pytest.raises(ValueError, match=msg):
+        parser.read_csv(StringIO(s), float_precision="junk")
