@@ -4,6 +4,7 @@ stated as a Python-specific issue, the goal is to eventually move as many of
 these tests out of this module as soon as the C parser can accept further
 arguments when parsing.
 """
+
 from __future__ import annotations
 
 import csv
@@ -157,7 +158,9 @@ also also skip this
 @pytest.mark.parametrize(
     "compression,klass", [("gzip", "GzipFile"), ("bz2", "BZ2File")]
 )
-def test_decompression_regex_sep(python_parser_only, csv1, compression, klass):
+def test_decompression_regex_sep(
+    temp_file, python_parser_only, csv1, compression, klass
+):
     # see gh-6607
     parser = python_parser_only
 
@@ -170,12 +173,11 @@ def test_decompression_regex_sep(python_parser_only, csv1, compression, klass):
     module = pytest.importorskip(compression)
     klass = getattr(module, klass)
 
-    with tm.ensure_clean() as path:
-        with klass(path, mode="wb") as tmp:
-            tmp.write(data)
+    with klass(temp_file, mode="wb") as tmp:
+        tmp.write(data)
 
-        result = parser.read_csv(path, sep="::", compression=compression)
-        tm.assert_frame_equal(result, expected)
+    result = parser.read_csv(temp_file, sep="::", compression=compression)
+    tm.assert_frame_equal(result, expected)
 
 
 def test_read_csv_buglet_4x_multi_index(python_parser_only):
@@ -274,7 +276,7 @@ def test_multi_char_sep_quotes(python_parser_only, quoting):
             parser.read_csv(StringIO(data), quoting=quoting, **kwargs)
 
 
-def test_none_delimiter(python_parser_only, capsys):
+def test_none_delimiter(python_parser_only):
     # see gh-13374 and gh-17465
     parser = python_parser_only
     data = "a,b,c\n0,1,2\n3,4,5,6\n7,8,9"
@@ -283,12 +285,14 @@ def test_none_delimiter(python_parser_only, capsys):
     # We expect the third line in the data to be
     # skipped because it is malformed, but we do
     # not expect any errors to occur.
-    result = parser.read_csv(StringIO(data), header=0, sep=None, on_bad_lines="warn")
+    with tm.assert_produces_warning(
+        ParserWarning, match="Skipping line 3", check_stacklevel=False
+    ):
+        result = parser.read_csv(
+            StringIO(data), header=0, sep=None, on_bad_lines="warn"
+        )
     tm.assert_frame_equal(result, expected)
 
-    captured = capsys.readouterr()
-    assert "Skipping line 3" in captured.err
-
 
 @pytest.mark.parametrize("data", ['a\n1\n"b"a', 'a,b,c\ncat,foo,bar\ndog,foo,"baz'])
 @pytest.mark.parametrize("skipfooter", [0, 1])
@@ -429,7 +433,7 @@ def test_on_bad_lines_callable_not_expected_length(python_parser_only):
     bad_sio = StringIO(data)
 
     result = parser.read_csv_check_warnings(
-        ParserWarning, "Length of header or names", bad_sio, on_bad_lines=lambda x: x
+        ParserWarning, "from bad_lines callable", bad_sio, on_bad_lines=lambda x: x
     )
     expected = DataFrame({"a": [1, 2, 3], "b": [2, 3, 4]})
     tm.assert_frame_equal(result, expected)
@@ -518,6 +522,8 @@ a;b;c
             "c": [0, 4000, 131],
         }
     )
+    if dtype["a"] == object:
+        expected["a"] = expected["a"].astype(object)
     tm.assert_frame_equal(result, expected)
 
 
@@ -526,21 +532,17 @@ a;b;c
     [
         (
             {"a": str, "b": np.float64, "c": np.int64},
-            DataFrame(
-                {
-                    "b": [16000.1, 0, 23000],
-                    "c": [0, 4001, 131],
-                }
-            ),
+            {
+                "b": [16000.1, 0, 23000],
+                "c": [0, 4001, 131],
+            },
         ),
         (
             str,
-            DataFrame(
-                {
-                    "b": ["16,000.1", "0", "23,000"],
-                    "c": ["0", "4,001", "131"],
-                }
-            ),
+            {
+                "b": ["16,000.1", "0", "23,000"],
+                "c": ["0", "4,001", "131"],
+            },
         ),
     ],
 )
@@ -558,5 +560,84 @@ def test_no_thousand_convert_for_non_numeric_cols(python_parser_only, dtype, exp
         dtype=dtype,
         thousands=",",
     )
+    expected = DataFrame(expected)
     expected.insert(0, "a", ["0000,7995", "3,03,001,00514", "4923,600,041"])
     tm.assert_frame_equal(result, expected)
+
+
+@pytest.mark.parametrize("index_col", [None, 0])
+def test_on_bad_lines_callable_warns_and_truncates_with_index_col(
+    python_parser_only, index_col
+):
+    # GH#61837
+    parser = python_parser_only
+    data = "id,field_1,field_2\n101,A,B\n102,C,D,E\n103,F,G\n"
+
+    def fixer(bad_line):
+        return [*list(bad_line), "EXTRA1", "EXTRA2"]
+
+    result = parser.read_csv_check_warnings(
+        ParserWarning,
+        "from bad_lines callable",
+        StringIO(data),
+        on_bad_lines=fixer,
+        index_col=index_col,
+    )
+
+    if index_col is None:
+        expected = DataFrame(
+            {
+                "id": [101, 102, 103],
+                "field_1": ["A", "C", "F"],
+                "field_2": ["B", "D", "G"],
+            }
+        )
+    else:
+        expected = DataFrame(
+            {"field_1": ["A", "C", "F"], "field_2": ["B", "D", "G"]},
+            index=Index([101, 102, 103], name="id"),
+        )
+
+    tm.assert_frame_equal(result, expected)
+
+
+def test_read_csv_leading_quote_skip(python_parser_only):
+    # GH 62739
+    tbl = """\
+    "
+a b
+1 3
+"""
+    parser = python_parser_only
+    result = parser.read_csv(
+        StringIO(tbl),
+        delimiter=" ",
+        skiprows=1,
+    )
+    expected = DataFrame({"a": [1], "b": [3]})
+    tm.assert_frame_equal(result, expected)
+
+
+def test_read_csv_unclosed_double_quote_in_data_still_errors(python_parser_only):
+    # GH 62739
+    tbl = """\
+a b
+"
+1 3
+"""
+    parser = python_parser_only
+    with pytest.raises(ParserError, match="unexpected end of data"):
+        parser.read_csv(StringIO(tbl), delimiter=" ", skiprows=1)
+
+
+def test_read_csv_skiprows_zero(python_parser_only):
+    # GH 62739
+    tbl = """\
+"
+a b
+1 3
+"""
+    parser = python_parser_only
+    # don't skip anything
+    with pytest.raises(ParserError, match="unexpected end of data"):
+        parser.read_csv(StringIO(tbl), delimiter=" ", skiprows=0, engine="python")
