@@ -17,7 +17,6 @@ from pandas import (
     Index,
 )
 import pandas._testing as tm
-from pandas.tests.io.test_compression import _compression_to_extension
 
 from pandas.io.common import get_handle
 from pandas.io.xml import read_xml
@@ -34,7 +33,7 @@ from pandas.io.xml import read_xml
 # [X] - KeyError: "...is not included in namespaces"
 # [X] - KeyError: "no valid column"
 # [X] - ValueError: "To use stylesheet, you need lxml installed..."
-# []  - OSError: (NEED PERMISSOIN ISSUE, DISK FULL, ETC.)
+# []  - OSError: (NEED PERMISSION ISSUE, DISK FULL, ETC.)
 # [X] - FileNotFoundError: "No such file or directory"
 # [X] - PermissionError: "Forbidden"
 
@@ -42,7 +41,7 @@ from pandas.io.xml import read_xml
 # [X] - TypeError: "...is not a valid type for attr_cols"
 # [X] - TypeError: "...is not a valid type for elem_cols"
 # [X] - LookupError: "unknown encoding"
-# []  - OSError: (NEED PERMISSOIN ISSUE, DISK FULL, ETC.)
+# []  - OSError: (NEED PERMISSION ISSUE, DISK FULL, ETC.)
 # [X] - FileNotFoundError: "No such file or directory"
 # [X] - KeyError: "...is not included in namespaces"
 # [X] - KeyError: "no valid column"
@@ -56,60 +55,69 @@ from pandas.io.xml import read_xml
 # [X] - XSLTParseError: "failed to compile"
 # [X] - PermissionError: "Forbidden"
 
-geom_df = DataFrame(
-    {
-        "shape": ["square", "circle", "triangle"],
-        "degrees": [360, 360, 180],
-        "sides": [4, np.nan, 3],
-    }
-)
 
-planet_df = DataFrame(
-    {
-        "planet": [
-            "Mercury",
-            "Venus",
-            "Earth",
-            "Mars",
-            "Jupiter",
-            "Saturn",
-            "Uranus",
-            "Neptune",
-        ],
-        "type": [
-            "terrestrial",
-            "terrestrial",
-            "terrestrial",
-            "terrestrial",
-            "gas giant",
-            "gas giant",
-            "ice giant",
-            "ice giant",
-        ],
-        "location": [
-            "inner",
-            "inner",
-            "inner",
-            "inner",
-            "outer",
-            "outer",
-            "outer",
-            "outer",
-        ],
-        "mass": [
-            0.330114,
-            4.86747,
-            5.97237,
-            0.641712,
-            1898.187,
-            568.3174,
-            86.8127,
-            102.4126,
-        ],
-    }
-)
+@pytest.fixture
+def geom_df():
+    return DataFrame(
+        {
+            "shape": ["square", "circle", "triangle"],
+            "degrees": [360, 360, 180],
+            "sides": [4, np.nan, 3],
+        }
+    )
 
-from_file_expected = """\
+
+@pytest.fixture
+def planet_df():
+    return DataFrame(
+        {
+            "planet": [
+                "Mercury",
+                "Venus",
+                "Earth",
+                "Mars",
+                "Jupiter",
+                "Saturn",
+                "Uranus",
+                "Neptune",
+            ],
+            "type": [
+                "terrestrial",
+                "terrestrial",
+                "terrestrial",
+                "terrestrial",
+                "gas giant",
+                "gas giant",
+                "ice giant",
+                "ice giant",
+            ],
+            "location": [
+                "inner",
+                "inner",
+                "inner",
+                "inner",
+                "outer",
+                "outer",
+                "outer",
+                "outer",
+            ],
+            "mass": [
+                0.330114,
+                4.86747,
+                5.97237,
+                0.641712,
+                1898.187,
+                568.3174,
+                86.8127,
+                102.4126,
+            ],
+        }
+    )
+
+
+@pytest.fixture
+def from_file_expected():
+    return """\
 <?xml version='1.0' encoding='utf-8'?>
 <data>
   <row>
@@ -146,7 +154,6 @@ def equalize_decl(doc):
             '<?xml version="1.0" encoding="utf-8"?',
             "<?xml version='1.0' encoding='utf-8'?",
         )
-
     return doc
 
 
@@ -163,37 +170,30 @@ def parser(request):
 # FILE OUTPUT
 
 
-def test_file_output_str_read(datapath, parser):
-    filename = datapath("io", "data", "xml", "books.xml")
-    df_file = read_xml(filename, parser=parser)
+def test_file_output_str_read(xml_books, parser, from_file_expected, temp_file):
+    df_file = read_xml(xml_books, parser=parser)
 
-    with tm.ensure_clean("test.xml") as path:
-        df_file.to_xml(path, parser=parser)
-        with open(path, "rb") as f:
-            output = f.read().decode("utf-8").strip()
+    df_file.to_xml(temp_file, parser=parser)
+    output = temp_file.read_text(encoding="utf-8").strip()
 
-        output = equalize_decl(output)
+    output = equalize_decl(output)
 
-        assert output == from_file_expected
+    assert output == from_file_expected
 
 
-def test_file_output_bytes_read(datapath, parser):
-    filename = datapath("io", "data", "xml", "books.xml")
-    df_file = read_xml(filename, parser=parser)
+def test_file_output_bytes_read(xml_books, parser, from_file_expected, temp_file):
+    df_file = read_xml(xml_books, parser=parser)
 
-    with tm.ensure_clean("test.xml") as path:
-        df_file.to_xml(path, parser=parser)
-        with open(path, "rb") as f:
-            output = f.read().decode("utf-8").strip()
+    df_file.to_xml(temp_file, parser=parser)
+    output = temp_file.read_text(encoding="utf-8").strip()
 
-        output = equalize_decl(output)
+    output = equalize_decl(output)
 
-        assert output == from_file_expected
+    assert output == from_file_expected
 
 
-def test_str_output(datapath, parser):
-    filename = datapath("io", "data", "xml", "books.xml")
-    df_file = read_xml(filename, parser=parser)
+def test_str_output(xml_books, parser, from_file_expected):
+    df_file = read_xml(xml_books, parser=parser)
 
     output = df_file.to_xml(parser=parser)
     output = equalize_decl(output)
@@ -201,7 +201,7 @@ def test_str_output(datapath, parser):
     assert output == from_file_expected
 
 
-def test_wrong_file_path(parser):
+def test_wrong_file_path(parser, geom_df):
     path = "/my/fake/path/output.xml"
 
     with pytest.raises(
@@ -214,7 +214,7 @@ def test_wrong_file_path(parser):
 # INDEX
 
 
-def test_index_false(datapath, parser):
+def test_index_false(xml_books, parser, temp_file):
     expected = """\
 <?xml version='1.0' encoding='utf-8'?>
 <data>
@@ -241,20 +241,17 @@ def test_index_false(datapath, parser):
   </row>
 </data>"""
 
-    filename = datapath("io", "data", "xml", "books.xml")
-    df_file = read_xml(filename, parser=parser)
+    df_file = read_xml(xml_books, parser=parser)
 
-    with tm.ensure_clean("test.xml") as path:
-        df_file.to_xml(path, index=False, parser=parser)
-        with open(path, "rb") as f:
-            output = f.read().decode("utf-8").strip()
+    df_file.to_xml(temp_file, index=False, parser=parser)
+    output = temp_file.read_text(encoding="utf-8").strip()
 
-        output = equalize_decl(output)
+    output = equalize_decl(output)
 
-        assert output == expected
+    assert output == expected
 
 
-def test_index_false_rename_row_root(datapath, parser):
+def test_index_false_rename_row_root(xml_books, parser, temp_file):
     expected = """\
 <?xml version='1.0' encoding='utf-8'?>
 <books>
@@ -281,25 +278,20 @@ def test_index_false_rename_row_root(datapath, parser):
   </book>
 </books>"""
 
-    filename = datapath("io", "data", "xml", "books.xml")
-    df_file = read_xml(filename, parser=parser)
+    df_file = read_xml(xml_books, parser=parser)
 
-    with tm.ensure_clean("test.xml") as path:
-        df_file.to_xml(
-            path, index=False, root_name="books", row_name="book", parser=parser
-        )
-        with open(path, "rb") as f:
-            output = f.read().decode("utf-8").strip()
+    df_file.to_xml(
+        temp_file, index=False, root_name="books", row_name="book", parser=parser
+    )
+    output = temp_file.read_text(encoding="utf-8").strip()
 
-        output = equalize_decl(output)
+    output = equalize_decl(output)
 
-        assert output == expected
+    assert output == expected
 
 
-@pytest.mark.parametrize(
-    "offset_index", [list(range(10, 13)), [str(i) for i in range(10, 13)]]
-)
-def test_index_false_with_offset_input_index(parser, offset_index):
+@pytest.mark.parametrize("typ", [int, str])
+def test_index_false_with_offset_input_index(parser, typ, geom_df):
     """
     Tests that the output does not contain the `<index>` field when the index of the
     input Dataframe has an offset.
@@ -326,7 +318,7 @@ def test_index_false_with_offset_input_index(parser, offset_index):
     <sides>3.0</sides>
   </row>
 </data>"""
-
+    offset_index = [typ(i) for i in range(10, 13)]
     offset_geom_df = geom_df.copy()
     offset_geom_df.index = Index(offset_index)
     output = offset_geom_df.to_xml(index=False, parser=parser)
@@ -361,21 +353,21 @@ na_expected = """\
 </data>"""
 
 
-def test_na_elem_output(parser):
+def test_na_elem_output(parser, geom_df):
     output = geom_df.to_xml(parser=parser)
     output = equalize_decl(output)
 
     assert output == na_expected
 
 
-def test_na_empty_str_elem_option(parser):
+def test_na_empty_str_elem_option(parser, geom_df):
     output = geom_df.to_xml(na_rep="", parser=parser)
     output = equalize_decl(output)
 
     assert output == na_expected
 
 
-def test_na_empty_elem_option(parser):
+def test_na_empty_elem_option(parser, geom_df):
     expected = """\
 <?xml version='1.0' encoding='utf-8'?>
 <data>
@@ -408,7 +400,7 @@ def test_na_empty_elem_option(parser):
 # ATTR_COLS
 
 
-def test_attrs_cols_nan_output(parser):
+def test_attrs_cols_nan_output(parser, geom_df):
     expected = """\
 <?xml version='1.0' encoding='utf-8'?>
 <data>
@@ -423,7 +415,7 @@ def test_attrs_cols_nan_output(parser):
     assert output == expected
 
 
-def test_attrs_cols_prefix(parser):
+def test_attrs_cols_prefix(parser, geom_df):
     expected = """\
 <?xml version='1.0' encoding='utf-8'?>
 <doc:data xmlns:doc="http://example.xom">
@@ -446,12 +438,12 @@ doc:degrees="180" doc:sides="3.0"/>
     assert output == expected
 
 
-def test_attrs_unknown_column(parser):
+def test_attrs_unknown_column(parser, geom_df):
     with pytest.raises(KeyError, match=("no valid column")):
         geom_df.to_xml(attr_cols=["shape", "degree", "sides"], parser=parser)
 
 
-def test_attrs_wrong_type(parser):
+def test_attrs_wrong_type(parser, geom_df):
     with pytest.raises(TypeError, match=("is not a valid type for attr_cols")):
         geom_df.to_xml(attr_cols='"shape", "degree", "sides"', parser=parser)
 
@@ -459,7 +451,7 @@ def test_attrs_wrong_type(parser):
 # ELEM_COLS
 
 
-def test_elems_cols_nan_output(parser):
+def test_elems_cols_nan_output(parser, geom_df):
     elems_cols_expected = """\
 <?xml version='1.0' encoding='utf-8'?>
 <data>
@@ -488,17 +480,17 @@ def test_elems_cols_nan_output(parser):
     assert output == elems_cols_expected
 
 
-def test_elems_unknown_column(parser):
+def test_elems_unknown_column(parser, geom_df):
     with pytest.raises(KeyError, match=("no valid column")):
         geom_df.to_xml(elem_cols=["shape", "degree", "sides"], parser=parser)
 
 
-def test_elems_wrong_type(parser):
+def test_elems_wrong_type(parser, geom_df):
     with pytest.raises(TypeError, match=("is not a valid type for elem_cols")):
         geom_df.to_xml(elem_cols='"shape", "degree", "sides"', parser=parser)
 
 
-def test_elems_and_attrs_cols(parser):
+def test_elems_and_attrs_cols(parser, geom_df):
     elems_cols_expected = """\
 <?xml version='1.0' encoding='utf-8'?>
 <data>
@@ -530,7 +522,7 @@ def test_elems_and_attrs_cols(parser):
 # HIERARCHICAL COLUMNS
 
 
-def test_hierarchical_columns(parser):
+def test_hierarchical_columns(parser, planet_df):
     expected = """\
 <?xml version='1.0' encoding='utf-8'?>
 <data>
@@ -577,7 +569,7 @@ def test_hierarchical_columns(parser):
     assert output == expected
 
 
-def test_hierarchical_attrs_columns(parser):
+def test_hierarchical_attrs_columns(parser, planet_df):
     expected = """\
 <?xml version='1.0' encoding='utf-8'?>
 <data>
@@ -607,7 +599,7 @@ sum_mass="2667.54" mean_mass="333.44"/>
 # MULTIINDEX
 
 
-def test_multi_index(parser):
+def test_multi_index(parser, planet_df):
     expected = """\
 <?xml version='1.0' encoding='utf-8'?>
 <data>
@@ -646,7 +638,7 @@ def test_multi_index(parser):
     assert output == expected
 
 
-def test_multi_index_attrs_cols(parser):
+def test_multi_index_attrs_cols(parser, planet_df):
     expected = """\
 <?xml version='1.0' encoding='utf-8'?>
 <data>
@@ -672,7 +664,7 @@ sum="189.23" mean="94.61"/>
 # NAMESPACE
 
 
-def test_default_namespace(parser):
+def test_default_namespace(parser, geom_df):
     expected = """\
 <?xml version='1.0' encoding='utf-8'?>
 <data xmlns="http://example.com">
@@ -702,10 +694,43 @@ def test_default_namespace(parser):
     assert output == expected
 
 
+def test_unused_namespaces(parser, geom_df):
+    expected = """\
+<?xml version='1.0' encoding='utf-8'?>
+<data xmlns:oth="http://other.org" xmlns:ex="http://example.com">
+  <row>
+    <index>0</index>
+    <shape>square</shape>
+    <degrees>360</degrees>
+    <sides>4.0</sides>
+  </row>
+  <row>
+    <index>1</index>
+    <shape>circle</shape>
+    <degrees>360</degrees>
+    <sides/>
+  </row>
+  <row>
+    <index>2</index>
+    <shape>triangle</shape>
+    <degrees>180</degrees>
+    <sides>3.0</sides>
+  </row>
+</data>"""
+
+    output = geom_df.to_xml(
+        namespaces={"oth": "http://other.org", "ex": "http://example.com"},
+        parser=parser,
+    )
+    output = equalize_decl(output)
+
+    assert output == expected
+
+
 # PREFIX
 
 
-def test_namespace_prefix(parser):
+def test_namespace_prefix(parser, geom_df):
     expected = """\
 <?xml version='1.0' encoding='utf-8'?>
 <doc:data xmlns:doc="http://example.com">
@@ -737,17 +762,17 @@ def test_namespace_prefix(parser):
     assert output == expected
 
 
-def test_missing_prefix_in_nmsp(parser):
+def test_missing_prefix_in_nmsp(parser, geom_df):
     with pytest.raises(KeyError, match=("doc is not included in namespaces")):
         geom_df.to_xml(
             namespaces={"": "http://example.com"}, prefix="doc", parser=parser
         )
 
 
-def test_namespace_prefix_and_default(parser):
+def test_namespace_prefix_and_default(parser, geom_df):
     expected = """\
 <?xml version='1.0' encoding='utf-8'?>
-<doc:data xmlns="http://example.com" xmlns:doc="http://other.org">
+<doc:data xmlns:doc="http://other.org" xmlns="http://example.com">
   <doc:row>
     <doc:index>0</doc:index>
     <doc:shape>square</doc:shape>
@@ -775,13 +800,6 @@ def test_namespace_prefix_and_default(parser):
     )
     output = equalize_decl(output)
 
-    if output is not None:
-        # etree and lxml differs on order of namespace prefixes
-        output = output.replace(
-            'xmlns:doc="http://other.org" xmlns="http://example.com"',
-            'xmlns="http://example.com" xmlns:doc="http://other.org"',
-        )
-
     assert output == expected
 
 
@@ -823,9 +841,8 @@ encoding_expected = """\
 </data>"""
 
 
-def test_encoding_option_str(datapath, parser):
-    filename = datapath("io", "data", "xml", "baby_names.xml")
-    df_file = read_xml(filename, parser=parser, encoding="ISO-8859-1").head(5)
+def test_encoding_option_str(xml_baby_names, parser):
+    df_file = read_xml(xml_baby_names, parser=parser, encoding="ISO-8859-1").head(5)
 
     output = df_file.to_xml(encoding="ISO-8859-1", parser=parser)
 
@@ -839,26 +856,22 @@ def test_encoding_option_str(datapath, parser):
     assert output == encoding_expected
 
 
-@td.skip_if_no("lxml")
-def test_correct_encoding_file(datapath):
-    filename = datapath("io", "data", "xml", "baby_names.xml")
-    df_file = read_xml(filename, encoding="ISO-8859-1", parser="lxml")
+def test_correct_encoding_file(xml_baby_names, temp_file):
+    pytest.importorskip("lxml")
+    df_file = read_xml(xml_baby_names, encoding="ISO-8859-1", parser="lxml")
 
-    with tm.ensure_clean("test.xml") as path:
-        df_file.to_xml(path, index=False, encoding="ISO-8859-1", parser="lxml")
+    df_file.to_xml(temp_file, index=False, encoding="ISO-8859-1", parser="lxml")
 
 
-@td.skip_if_no("lxml")
 @pytest.mark.parametrize("encoding", ["UTF-8", "UTF-16", "ISO-8859-1"])
-def test_wrong_encoding_option_lxml(datapath, parser, encoding):
-    filename = datapath("io", "data", "xml", "baby_names.xml")
-    df_file = read_xml(filename, encoding="ISO-8859-1", parser="lxml")
+def test_wrong_encoding_option_lxml(xml_baby_names, parser, encoding, temp_file):
+    pytest.importorskip("lxml")
+    df_file = read_xml(xml_baby_names, encoding="ISO-8859-1", parser="lxml")
 
-    with tm.ensure_clean("test.xml") as path:
-        df_file.to_xml(path, index=False, encoding=encoding, parser=parser)
+    df_file.to_xml(temp_file, index=False, encoding=encoding, parser=parser)
 
 
-def test_misspelled_encoding(parser):
+def test_misspelled_encoding(parser, geom_df):
     with pytest.raises(LookupError, match=("unknown encoding")):
         geom_df.to_xml(encoding="uft-8", parser=parser)
 
@@ -866,8 +879,8 @@ def test_misspelled_encoding(parser):
 # PRETTY PRINT
 
 
-@td.skip_if_no("lxml")
-def test_xml_declaration_pretty_print():
+def test_xml_declaration_pretty_print(geom_df):
+    pytest.importorskip("lxml")
     expected = """\
 <data>
   <row>
@@ -895,7 +908,7 @@ def test_xml_declaration_pretty_print():
     assert output == expected
 
 
-def test_no_pretty_print_with_decl(parser):
+def test_no_pretty_print_with_decl(parser, geom_df):
     expected = (
         "<?xml version='1.0' encoding='utf-8'?>\n"
         "<data><row><index>0</index><shape>square</shape>"
@@ -916,7 +929,7 @@ def test_no_pretty_print_with_decl(parser):
     assert output == expected
 
 
-def test_no_pretty_print_no_decl(parser):
+def test_no_pretty_print_no_decl(parser, geom_df):
     expected = (
         "<data><row><index>0</index><shape>square</shape>"
         "<degrees>360</degrees><sides>4.0</sides></row><row>"
@@ -939,14 +952,14 @@ def test_no_pretty_print_no_decl(parser):
 
 
 @td.skip_if_installed("lxml")
-def test_default_parser_no_lxml():
+def test_default_parser_no_lxml(geom_df):
     with pytest.raises(
         ImportError, match=("lxml not found, please install or use the etree parser.")
     ):
         geom_df.to_xml()
 
 
-def test_unknown_parser():
+def test_unknown_parser(geom_df):
     with pytest.raises(
         ValueError, match=("Values for parser can only be lxml or etree.")
     ):
@@ -979,21 +992,23 @@ xsl_expected = """\
 </data>"""
 
 
-@td.skip_if_no("lxml")
-def test_stylesheet_file_like(datapath, mode):
-    xsl = datapath("io", "data", "xml", "row_field_output.xsl")
-
-    with open(xsl, mode) as f:
+def test_stylesheet_file_like(xsl_row_field_output, mode, geom_df):
+    pytest.importorskip("lxml")
+    with open(
+        xsl_row_field_output, mode, encoding="utf-8" if mode == "r" else None
+    ) as f:
         assert geom_df.to_xml(stylesheet=f) == xsl_expected
 
 
-@td.skip_if_no("lxml")
-def test_stylesheet_io(datapath, mode):
-    xsl_path = datapath("io", "data", "xml", "row_field_output.xsl")
+def test_stylesheet_io(xsl_row_field_output, mode, geom_df):
+    # note: By default the bodies of untyped functions are not checked,
+    # consider using --check-untyped-defs
+    pytest.importorskip("lxml")
+    xsl_obj: BytesIO | StringIO  # type: ignore[annotation-unchecked]
 
-    xsl_obj: BytesIO | StringIO
-
-    with open(xsl_path, mode) as f:
+    with open(
+        xsl_row_field_output, mode, encoding="utf-8" if mode == "r" else None
+    ) as f:
         if mode == "rb":
             xsl_obj = BytesIO(f.read())
         else:
@@ -1004,35 +1019,30 @@ def test_stylesheet_io(datapath, mode):
     assert output == xsl_expected
 
 
-@td.skip_if_no("lxml")
-def test_stylesheet_buffered_reader(datapath, mode):
-    xsl = datapath("io", "data", "xml", "row_field_output.xsl")
-
-    with open(xsl, mode) as f:
-        xsl_obj = f.read()
-
-    output = geom_df.to_xml(stylesheet=xsl_obj)
+def test_stylesheet_buffered_reader(xsl_row_field_output, mode, geom_df):
+    pytest.importorskip("lxml")
+    with open(
+        xsl_row_field_output, mode, encoding="utf-8" if mode == "r" else None
+    ) as f:
+        output = geom_df.to_xml(stylesheet=f)
 
     assert output == xsl_expected
 
 
-@td.skip_if_no("lxml")
-def test_stylesheet_wrong_path():
-    from lxml.etree import XMLSyntaxError
+def test_stylesheet_wrong_path(geom_df):
+    pytest.importorskip("lxml.etree")
 
-    xsl = os.path.join("data", "xml", "row_field_output.xslt")
+    xsl = os.path.join("does", "not", "exist", "row_field_output.xslt")
 
     with pytest.raises(
-        XMLSyntaxError,
-        match=("Start tag expected, '<' not found"),
+        FileNotFoundError, match=r"\[Errno 2\] No such file or director"
     ):
         geom_df.to_xml(stylesheet=xsl)
 
 
-@td.skip_if_no("lxml")
-@pytest.mark.parametrize("val", ["", b""])
-def test_empty_string_stylesheet(val):
-    from lxml.etree import XMLSyntaxError
+@pytest.mark.parametrize("val", [StringIO(""), BytesIO(b"")])
+def test_empty_string_stylesheet(val, geom_df):
+    lxml_etree = pytest.importorskip("lxml.etree")
 
     msg = "|".join(
         [
@@ -1043,13 +1053,12 @@ def test_empty_string_stylesheet(val):
         ]
     )
 
-    with pytest.raises(XMLSyntaxError, match=msg):
+    with pytest.raises(lxml_etree.XMLSyntaxError, match=msg):
         geom_df.to_xml(stylesheet=val)
 
 
-@td.skip_if_no("lxml")
-def test_incorrect_xsl_syntax():
-    from lxml.etree import XMLSyntaxError
+def test_incorrect_xsl_syntax(geom_df):
+    lxml_etree = pytest.importorskip("lxml.etree")
 
     xsl = """\
 <xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
@@ -1072,13 +1081,14 @@ def test_incorrect_xsl_syntax():
     </xsl:template>
 </xsl:stylesheet>"""
 
-    with pytest.raises(XMLSyntaxError, match=("Opening and ending tag mismatch")):
-        geom_df.to_xml(stylesheet=xsl)
+    with pytest.raises(
+        lxml_etree.XMLSyntaxError, match="Opening and ending tag mismatch"
+    ):
+        geom_df.to_xml(stylesheet=StringIO(xsl))
 
 
-@td.skip_if_no("lxml")
-def test_incorrect_xsl_eval():
-    from lxml.etree import XSLTParseError
+def test_incorrect_xsl_eval(geom_df):
+    lxml_etree = pytest.importorskip("lxml.etree")
 
     xsl = """\
 <xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
@@ -1101,13 +1111,12 @@ def test_incorrect_xsl_eval():
     </xsl:template>
 </xsl:stylesheet>"""
 
-    with pytest.raises(XSLTParseError, match=("failed to compile")):
-        geom_df.to_xml(stylesheet=xsl)
+    with pytest.raises(lxml_etree.XSLTParseError, match="failed to compile"):
+        geom_df.to_xml(stylesheet=StringIO(xsl))
 
 
-@td.skip_if_no("lxml")
-def test_incorrect_xsl_apply():
-    from lxml.etree import XSLTApplyError
+def test_incorrect_xsl_apply(geom_df, temp_file):
+    lxml_etree = pytest.importorskip("lxml.etree")
 
     xsl = """\
 <xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
@@ -1121,12 +1130,11 @@ def test_incorrect_xsl_apply():
     </xsl:template>
 </xsl:stylesheet>"""
 
-    with pytest.raises(XSLTApplyError, match=("Cannot resolve URI")):
-        with tm.ensure_clean("test.xml") as path:
-            geom_df.to_xml(path, stylesheet=xsl)
+    with pytest.raises(lxml_etree.XSLTApplyError, match="Cannot resolve URI"):
+        geom_df.to_xml(temp_file, stylesheet=StringIO(xsl))
 
 
-def test_stylesheet_with_etree():
+def test_stylesheet_with_etree(geom_df):
     xsl = """\
 <xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
     <xsl:output method="xml" encoding="utf-8" indent="yes" />
@@ -1138,14 +1146,12 @@ def test_stylesheet_with_etree():
         </xsl:copy>
     </xsl:template>"""
 
-    with pytest.raises(
-        ValueError, match=("To use stylesheet, you need lxml installed")
-    ):
-        geom_df.to_xml(parser="etree", stylesheet=xsl)
+    with pytest.raises(ValueError, match="To use stylesheet, you need lxml installed"):
+        geom_df.to_xml(parser="etree", stylesheet=StringIO(xsl))
 
 
-@td.skip_if_no("lxml")
-def test_style_to_csv():
+def test_style_to_csv(geom_df):
+    pytest.importorskip("lxml")
     xsl = """\
 <xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
     <xsl:output method="text" indent="yes" />
@@ -1168,13 +1174,13 @@ def test_style_to_csv():
 
     if out_csv is not None:
         out_csv = out_csv.strip()
-    out_xml = geom_df.to_xml(stylesheet=xsl)
+    out_xml = geom_df.to_xml(stylesheet=StringIO(xsl))
 
     assert out_csv == out_xml
 
 
-@td.skip_if_no("lxml")
-def test_style_to_string():
+def test_style_to_string(geom_df):
+    pytest.importorskip("lxml")
     xsl = """\
 <xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
     <xsl:output method="text" indent="yes" />
@@ -1202,13 +1208,13 @@ def test_style_to_string():
 </xsl:stylesheet>"""
 
     out_str = geom_df.to_string()
-    out_xml = geom_df.to_xml(na_rep="NaN", stylesheet=xsl)
+    out_xml = geom_df.to_xml(na_rep="NaN", stylesheet=StringIO(xsl))
 
     assert out_xml == out_str
 
 
-@td.skip_if_no("lxml")
-def test_style_to_json():
+def test_style_to_json(geom_df):
+    pytest.importorskip("lxml")
     xsl = """\
 <xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
     <xsl:output method="text" indent="yes" />
@@ -1247,7 +1253,7 @@ def test_style_to_json():
 </xsl:stylesheet>"""
 
     out_json = geom_df.to_json()
-    out_xml = geom_df.to_xml(stylesheet=xsl)
+    out_xml = geom_df.to_xml(stylesheet=StringIO(xsl))
 
     assert out_json == out_xml
 
@@ -1279,33 +1285,35 @@ geom_xml = """\
 </data>"""
 
 
-def test_compression_output(parser, compression_only):
-    with tm.ensure_clean() as path:
-        geom_df.to_xml(path, parser=parser, compression=compression_only)
+def test_compression_output(parser, compression_only, geom_df, temp_file):
+    path = temp_file
+    geom_df.to_xml(path, parser=parser, compression=compression_only)
 
-        with get_handle(
-            path,
-            "r",
-            compression=compression_only,
-        ) as handle_obj:
-            output = handle_obj.handle.read()
+    with get_handle(
+        path,
+        "r",
+        compression=compression_only,
+    ) as handle_obj:
+        output = handle_obj.handle.read()
 
     output = equalize_decl(output)
 
     assert geom_xml == output.strip()
 
 
-def test_filename_and_suffix_comp(parser, compression_only):
-    compfile = "xml." + _compression_to_extension[compression_only]
-    with tm.ensure_clean(filename=compfile) as path:
-        geom_df.to_xml(path, parser=parser, compression=compression_only)
+def test_filename_and_suffix_comp(
+    parser, compression_only, geom_df, compression_to_extension, tmp_path
+):
+    compfile = "xml." + compression_to_extension[compression_only]
+    path = tmp_path / compfile
+    geom_df.to_xml(path, parser=parser, compression=compression_only)
 
-        with get_handle(
-            path,
-            "r",
-            compression=compression_only,
-        ) as handle_obj:
-            output = handle_obj.handle.read()
+    with get_handle(
+        path,
+        "r",
+        compression=compression_only,
+    ) as handle_obj:
+        output = handle_obj.handle.read()
 
     output = equalize_decl(output)
 
@@ -1326,24 +1334,24 @@ def test_ea_dtypes(any_numeric_ea_dtype, parser):
     assert equalize_decl(result).strip() == expected
 
 
-def test_unsuported_compression(parser):
+def test_unsupported_compression(parser, geom_df, temp_file):
     with pytest.raises(ValueError, match="Unrecognized compression type"):
-        with tm.ensure_clean() as path:
-            geom_df.to_xml(path, parser=parser, compression="7z")
+        path = temp_file
+        geom_df.to_xml(path, parser=parser, compression="7z")
 
 
 # STORAGE OPTIONS
 
 
 @pytest.mark.single_cpu
-@td.skip_if_no("s3fs")
-@td.skip_if_no("lxml")
-def test_s3_permission_output(parser, s3_resource):
-    # s3_resource hosts pandas-test
-    import s3fs
+def test_s3_permission_output(parser, s3_bucket_public, geom_df):
+    s3fs = pytest.importorskip("s3fs")
+    pytest.importorskip("lxml")
 
-    with pytest.raises(PermissionError, match="Access Denied"):
+    with tm.external_error_raised((PermissionError, FileNotFoundError)):
         fs = s3fs.S3FileSystem(anon=True)
-        fs.ls("pandas-test")
+        fs.ls(s3_bucket_public.name)
 
-        geom_df.to_xml("s3://pandas-test/geom.xml", compression="zip", parser=parser)
+        geom_df.to_xml(
+            f"s3://{s3_bucket_public.name}/geom.xml", compression="zip", parser=parser
+        )
