@@ -1,3 +1,5 @@
+import pickle
+
 import numpy as np
 import pytest
 
@@ -59,7 +61,7 @@ class TestMisc:
         tm.assert_numpy_array_equal(result, expected)
 
         # case that can't be cast to td64ns
-        td = Timedelta(np.timedelta64(400, "Y"))
+        td = Timedelta(np.timedelta64(146000, "D"))
         assert hash(td) == hash(td.as_unit("ms"))
         assert hash(td) == hash(td.as_unit("us"))
         mapping1 = {td: 1}
@@ -279,7 +281,78 @@ def test_cache_readonly_preserve_docstrings():
     assert Index.hasnans.__doc__ is not None
 
 
-def test_no_default_pickle():
+def test_no_default_pickle(temp_file):
     # GH#40397
-    obj = tm.round_trip_pickle(lib.no_default)
+    obj = tm.round_trip_pickle(lib.no_default, temp_file)
     assert obj is lib.no_default
+
+
+def test_ensure_string_array_copy():
+    # ensure the original array is not modified in case of copy=False with
+    # pickle-roundtripped object dtype array
+    # https://github.com/pandas-dev/pandas/issues/54654
+    arr = np.array(["a", None], dtype=object)
+    arr = pickle.loads(pickle.dumps(arr))
+    result = lib.ensure_string_array(arr, copy=False)
+    assert not np.shares_memory(arr, result)
+    assert arr[1] is None
+    assert result[1] is np.nan
+
+
+def test_ensure_string_array_list_of_lists():
+    # GH#61155: ensure list of lists doesn't get converted to string
+    arr = [list("test"), list("word")]
+    result = lib.ensure_string_array(arr)
+
+    # Each item in result should still be a list, not a stringified version
+    expected = np.array(["['t', 'e', 's', 't']", "['w', 'o', 'r', 'd']"], dtype=object)
+    tm.assert_numpy_array_equal(result, expected)
+
+
+def test_item_from_zerodim_for_subclasses():
+    # GH#62981 Ensure item_from_zerodim preserves subclasses of ndarray
+    # Define a custom ndarray subclass
+    class TestArray(np.ndarray):
+        def __new__(cls, input_array):
+            return np.asarray(input_array).view(cls)
+
+        def __array_finalize__(self, obj) -> None:
+            self._is_test_array = True
+
+    # Define test data
+    val_0_dim = 1
+    val_1_dim = [1, 2, 3]
+
+    # 0-dim and 1-dim numpy arrays
+    arr_0_dim = np.array(val_0_dim)
+    arr_1_dim = np.array(val_1_dim)
+
+    # 0-dim and 1-dim TestArray arrays
+    test_arr_0_dim = TestArray(val_0_dim)
+    test_arr_1_dim = TestArray(val_1_dim)
+
+    # Check that behavior did not change for regular numpy arrays
+    # Test with regular numpy 0-dim array
+    result = lib.item_from_zerodim(arr_0_dim)
+    expected = val_0_dim
+    assert result == expected
+    assert np.isscalar(result)
+
+    # Test with regular numpy 1-dim array
+    result = lib.item_from_zerodim(arr_1_dim)
+    expected = arr_1_dim
+    tm.assert_numpy_array_equal(result, expected)
+    assert isinstance(result, np.ndarray)
+
+    # Check that behaviour for subclasses now is as expected
+    # Test with TestArray 0-dim array
+    result = lib.item_from_zerodim(test_arr_0_dim)
+    expected = test_arr_0_dim
+    assert result == expected
+    assert isinstance(result, TestArray)
+
+    # Test with TestArray 1-dim array
+    result = lib.item_from_zerodim(test_arr_1_dim)
+    expected = test_arr_1_dim
+    assert np.all(result == expected)
+    assert isinstance(result, TestArray)
