@@ -4,14 +4,18 @@ import pytest
 from pandas.core.dtypes.concat import union_categoricals
 
 import pandas as pd
-from pandas import Categorical, CategoricalIndex, Series
-from pandas.util import testing as tm
+from pandas import (
+    Categorical,
+    CategoricalIndex,
+    Series,
+)
+import pandas._testing as tm
 
 
 class TestUnionCategoricals:
-    def test_union_categorical(self):
-        # GH 13361
-        data = [
+    @pytest.mark.parametrize(
+        "a, b, combined",
+        [
             (list("abc"), list("abd"), list("abcabd")),
             ([0, 1, 2], [2, 3, 4], [0, 1, 2, 2, 3, 4]),
             ([0, 1.2, 2], [2, 3.4, 4], [0, 1.2, 2, 2, 3.4, 4]),
@@ -35,14 +39,16 @@ class TestUnionCategoricals:
                 pd.period_range("2014-01-06", "2014-01-07"),
                 pd.period_range("2014-01-01", "2014-01-07"),
             ),
-        ]
-
-        for a, b, combined in data:
-            for box in [Categorical, CategoricalIndex, Series]:
-                result = union_categoricals([box(Categorical(a)), box(Categorical(b))])
-                expected = Categorical(combined)
-                tm.assert_categorical_equal(result, expected, check_category_order=True)
+        ],
+    )
+    @pytest.mark.parametrize("box", [Categorical, CategoricalIndex, Series])
+    def test_union_categorical(self, a, b, combined, box):
+        # GH 13361
+        result = union_categoricals([box(Categorical(a)), box(Categorical(b))])
+        expected = Categorical(combined)
+        tm.assert_categorical_equal(result, expected)
 
+    def test_union_categorical_ordered_appearance(self):
         # new categories ordered by appearance
         s = Categorical(["x", "y", "z"])
         s2 = Categorical(["a", "b", "c"])
@@ -52,12 +58,14 @@ class TestUnionCategoricals:
         )
         tm.assert_categorical_equal(result, expected)
 
+    def test_union_categorical_ordered_true(self):
         s = Categorical([0, 1.2, 2], ordered=True)
         s2 = Categorical([0, 1.2, 2], ordered=True)
         result = union_categoricals([s, s2])
         expected = Categorical([0, 1.2, 2, 0, 1.2, 2], ordered=True)
         tm.assert_categorical_equal(result, expected)
 
+    def test_union_categorical_match_types(self):
         # must exactly match types
         s = Categorical([0, 1.2, 2])
         s2 = Categorical([2, 3, 4])
@@ -65,6 +73,7 @@ class TestUnionCategoricals:
         with pytest.raises(TypeError, match=msg):
             union_categoricals([s, s2])
 
+    def test_union_categorical_empty(self):
         msg = "No Categoricals to union"
         with pytest.raises(ValueError, match=msg):
             union_categoricals([])
@@ -72,13 +81,13 @@ class TestUnionCategoricals:
     def test_union_categoricals_nan(self):
         # GH 13759
         res = union_categoricals(
-            [pd.Categorical([1, 2, np.nan]), pd.Categorical([3, 2, np.nan])]
+            [Categorical([1, 2, np.nan]), Categorical([3, 2, np.nan])]
         )
         exp = Categorical([1, 2, np.nan, 3, 2, np.nan])
         tm.assert_categorical_equal(res, exp)
 
         res = union_categoricals(
-            [pd.Categorical(["A", "B"]), pd.Categorical(["B", "B", np.nan])]
+            [Categorical(["A", "B"]), Categorical(["B", "B", np.nan])]
         )
         exp = Categorical(["A", "B", "B", "B", np.nan])
         tm.assert_categorical_equal(res, exp)
@@ -86,7 +95,7 @@ class TestUnionCategoricals:
         val1 = [pd.Timestamp("2011-01-01"), pd.Timestamp("2011-03-01"), pd.NaT]
         val2 = [pd.NaT, pd.Timestamp("2011-01-01"), pd.Timestamp("2011-02-01")]
 
-        res = union_categoricals([pd.Categorical(val1), pd.Categorical(val2)])
+        res = union_categoricals([Categorical(val1), Categorical(val2)])
         exp = Categorical(
             val1 + val2,
             categories=[
@@ -100,27 +109,30 @@ class TestUnionCategoricals:
         # all NaN
         res = union_categoricals(
             [
-                pd.Categorical(np.array([np.nan, np.nan], dtype=object)),
-                pd.Categorical(["X"]),
+                Categorical(np.array([np.nan, np.nan], dtype=object)),
+                Categorical(["X"], categories=pd.Index(["X"], dtype=object)),
             ]
         )
         exp = Categorical([np.nan, np.nan, "X"])
         tm.assert_categorical_equal(res, exp)
 
         res = union_categoricals(
-            [pd.Categorical([np.nan, np.nan]), pd.Categorical([np.nan, np.nan])]
+            [Categorical([np.nan, np.nan]), Categorical([np.nan, np.nan])]
         )
         exp = Categorical([np.nan, np.nan, np.nan, np.nan])
         tm.assert_categorical_equal(res, exp)
 
-    def test_union_categoricals_empty(self):
+    @pytest.mark.parametrize("val", [[], ["1"]])
+    def test_union_categoricals_empty(self, val, request, using_infer_string):
         # GH 13759
-        res = union_categoricals([pd.Categorical([]), pd.Categorical([])])
-        exp = Categorical([])
-        tm.assert_categorical_equal(res, exp)
-
-        res = union_categoricals([Categorical([]), Categorical(["1"])])
-        exp = Categorical(["1"])
+        if using_infer_string and val == ["1"]:
+            request.applymarker(
+                pytest.mark.xfail(
+                    reason="TDOD(infer_string) object and strings dont match"
+                )
+            )
+        res = union_categoricals([Categorical([]), Categorical(val)])
+        exp = Categorical(val)
         tm.assert_categorical_equal(res, exp)
 
     def test_union_categorical_same_category(self):
@@ -131,6 +143,7 @@ class TestUnionCategoricals:
         exp = Categorical([1, 2, 3, 4, 3, 2, 1, np.nan], categories=[1, 2, 3, 4])
         tm.assert_categorical_equal(res, exp)
 
+    def test_union_categorical_same_category_str(self):
         c1 = Categorical(["z", "z", "z"], categories=["x", "y", "z"])
         c2 = Categorical(["x", "x", "x"], categories=["x", "y", "z"])
         res = union_categoricals([c1, c2])
@@ -275,7 +288,8 @@ class TestUnionCategoricals:
 
         c1 = Categorical(["b", "a"], categories=["b", "a", "c"], ordered=True)
         c2 = Categorical(["a", "c"], categories=["b", "a", "c"], ordered=True)
-        with pytest.raises(TypeError):
+        msg = "Cannot use sort_categories=True with ordered Categoricals"
+        with pytest.raises(TypeError, match=msg):
             union_categoricals([c1, c2], sort_categories=True)
 
     def test_union_categoricals_sort_false(self):
@@ -288,6 +302,7 @@ class TestUnionCategoricals:
         )
         tm.assert_categorical_equal(result, expected)
 
+    def test_union_categoricals_sort_false_fastpath(self):
         # fastpath
         c1 = Categorical(["a", "b"], categories=["b", "a", "c"])
         c2 = Categorical(["b", "c"], categories=["b", "a", "c"])
@@ -295,6 +310,7 @@ class TestUnionCategoricals:
         expected = Categorical(["a", "b", "b", "c"], categories=["b", "a", "c"])
         tm.assert_categorical_equal(result, expected)
 
+    def test_union_categoricals_sort_false_skipresort(self):
         # fastpath - skip resort
         c1 = Categorical(["a", "b"], categories=["a", "b", "c"])
         c2 = Categorical(["b", "c"], categories=["a", "b", "c"])
@@ -302,24 +318,28 @@ class TestUnionCategoricals:
         expected = Categorical(["a", "b", "b", "c"], categories=["a", "b", "c"])
         tm.assert_categorical_equal(result, expected)
 
+    def test_union_categoricals_sort_false_one_nan(self):
         c1 = Categorical(["x", np.nan])
         c2 = Categorical([np.nan, "b"])
         result = union_categoricals([c1, c2], sort_categories=False)
         expected = Categorical(["x", np.nan, np.nan, "b"], categories=["x", "b"])
         tm.assert_categorical_equal(result, expected)
 
+    def test_union_categoricals_sort_false_only_nan(self):
         c1 = Categorical([np.nan])
         c2 = Categorical([np.nan])
         result = union_categoricals([c1, c2], sort_categories=False)
         expected = Categorical([np.nan, np.nan])
         tm.assert_categorical_equal(result, expected)
 
+    def test_union_categoricals_sort_false_empty(self):
         c1 = Categorical([])
         c2 = Categorical([])
         result = union_categoricals([c1, c2], sort_categories=False)
         expected = Categorical([])
         tm.assert_categorical_equal(result, expected)
 
+    def test_union_categoricals_sort_false_ordered_true(self):
         c1 = Categorical(["b", "a"], categories=["b", "a", "c"], ordered=True)
         c2 = Categorical(["a", "c"], categories=["b", "a", "c"], ordered=True)
         result = union_categoricals([c1, c2], sort_categories=False)
@@ -331,7 +351,7 @@ class TestUnionCategoricals:
     def test_union_categorical_unwrap(self):
         # GH 14173
         c1 = Categorical(["a", "b"])
-        c2 = pd.Series(["b", "c"], dtype="category")
+        c2 = Series(["b", "c"], dtype="category")
         result = union_categoricals([c1, c2])
         expected = Categorical(["a", "b", "b", "c"])
         tm.assert_categorical_equal(result, expected)
@@ -344,5 +364,6 @@ class TestUnionCategoricals:
         result = union_categoricals([c1, c2])
         tm.assert_categorical_equal(result, expected)
 
-        with pytest.raises(TypeError):
+        msg = "all components to combine must be Categorical"
+        with pytest.raises(TypeError, match=msg):
             union_categoricals([c1, ["a", "b", "c"]])
