@@ -1,5 +1,7 @@
-""" test the scalar Timedelta """
+"""test the scalar Timedelta"""
+
 from datetime import timedelta
+import sys
 
 from hypothesis import (
     given,
@@ -9,96 +11,24 @@ import numpy as np
 import pytest
 
 from pandas._libs import lib
+from pandas._libs.missing import NA
 from pandas._libs.tslibs import (
     NaT,
     iNaT,
 )
 from pandas._libs.tslibs.dtypes import NpyDatetimeUnit
-from pandas.errors import OutOfBoundsTimedelta
+from pandas.errors import (
+    OutOfBoundsTimedelta,
+    Pandas4Warning,
+)
 
-import pandas as pd
 from pandas import (
     Timedelta,
-    TimedeltaIndex,
     to_timedelta,
 )
 import pandas._testing as tm
 
 
-class TestAsUnit:
-    def test_as_unit(self):
-        td = Timedelta(days=1)
-
-        assert td.as_unit("ns") is td
-
-        res = td.as_unit("us")
-        assert res._value == td._value // 1000
-        assert res._creso == NpyDatetimeUnit.NPY_FR_us.value
-
-        rt = res.as_unit("ns")
-        assert rt._value == td._value
-        assert rt._creso == td._creso
-
-        res = td.as_unit("ms")
-        assert res._value == td._value // 1_000_000
-        assert res._creso == NpyDatetimeUnit.NPY_FR_ms.value
-
-        rt = res.as_unit("ns")
-        assert rt._value == td._value
-        assert rt._creso == td._creso
-
-        res = td.as_unit("s")
-        assert res._value == td._value // 1_000_000_000
-        assert res._creso == NpyDatetimeUnit.NPY_FR_s.value
-
-        rt = res.as_unit("ns")
-        assert rt._value == td._value
-        assert rt._creso == td._creso
-
-    def test_as_unit_overflows(self):
-        # microsecond that would be just out of bounds for nano
-        us = 9223372800000000
-        td = Timedelta._from_value_and_reso(us, NpyDatetimeUnit.NPY_FR_us.value)
-
-        msg = "Cannot cast 106752 days 00:00:00 to unit='ns' without overflow"
-        with pytest.raises(OutOfBoundsTimedelta, match=msg):
-            td.as_unit("ns")
-
-        res = td.as_unit("ms")
-        assert res._value == us // 1000
-        assert res._creso == NpyDatetimeUnit.NPY_FR_ms.value
-
-    def test_as_unit_rounding(self):
-        td = Timedelta(microseconds=1500)
-        res = td.as_unit("ms")
-
-        expected = Timedelta(milliseconds=1)
-        assert res == expected
-
-        assert res._creso == NpyDatetimeUnit.NPY_FR_ms.value
-        assert res._value == 1
-
-        with pytest.raises(ValueError, match="Cannot losslessly convert units"):
-            td.as_unit("ms", round_ok=False)
-
-    def test_as_unit_non_nano(self):
-        # case where we are going neither to nor from nano
-        td = Timedelta(days=1).as_unit("ms")
-        assert td.days == 1
-        assert td._value == 86_400_000
-        assert td.components.days == 1
-        assert td._d == 1
-        assert td.total_seconds() == 86400
-
-        res = td.as_unit("us")
-        assert res._value == 86_400_000_000
-        assert res.components.days == 1
-        assert res.components.hours == 0
-        assert res._d == 1
-        assert res._h == 0
-        assert res.total_seconds() == 86400
-
-
 class TestNonNano:
     @pytest.fixture(params=["s", "ms", "us"])
     def unit_str(self, request):
@@ -153,7 +83,7 @@ class TestNonNano:
 
     def test_cmp_cross_reso(self, td):
         # numpy gets this wrong because of silent overflow
-        other = Timedelta(days=106751, unit="ns")
+        other = Timedelta(days=106751)
         assert other < td
         assert td > other
         assert not other == td
@@ -212,6 +142,19 @@ class TestNonNano:
         assert res._value == td._value / 2
         assert res._creso == td._creso
 
+    def test_truediv_na_type_not_supported(self, td):
+        msg_td_floordiv_na = (
+            r"unsupported operand type\(s\) for /: 'Timedelta' and 'NAType'"
+        )
+        with pytest.raises(TypeError, match=msg_td_floordiv_na):
+            td / NA
+
+        msg_na_floordiv_td = (
+            r"unsupported operand type\(s\) for /: 'NAType' and 'Timedelta'"
+        )
+        with pytest.raises(TypeError, match=msg_na_floordiv_td):
+            NA / td
+
     def test_floordiv_timedeltalike(self, td):
         assert td // td == 1
         assert (2.5 * td) // td == 2
@@ -256,6 +199,19 @@ class TestNonNano:
         assert res._value == td._value // 2
         assert res._creso == td._creso
 
+    def test_floordiv_na_type_not_supported(self, td):
+        msg_td_floordiv_na = (
+            r"unsupported operand type\(s\) for //: 'Timedelta' and 'NAType'"
+        )
+        with pytest.raises(TypeError, match=msg_td_floordiv_na):
+            td // NA
+
+        msg_na_floordiv_td = (
+            r"unsupported operand type\(s\) for //: 'NAType' and 'Timedelta'"
+        )
+        with pytest.raises(TypeError, match=msg_na_floordiv_td):
+            NA // td
+
     def test_addsub_mismatched_reso(self, td):
         # need to cast to since td is out of bounds for ns, so
         #  so we would raise OverflowError without casting
@@ -305,6 +261,12 @@ class TestNonNano:
         assert result == expected
         assert result._creso == expected._creso
 
+    def test_hash(self) -> None:
+        # GH#54037
+        second_resolution_max = Timedelta(0).as_unit("s").max
+
+        assert hash(second_resolution_max)
+
 
 def test_timedelta_class_min_max_resolution():
     # when accessed on the class (as opposed to an instance), we default
@@ -321,7 +283,7 @@ def test_timedelta_class_min_max_resolution():
 
 class TestTimedeltaUnaryOps:
     def test_invert(self):
-        td = Timedelta(10, unit="d")
+        td = Timedelta(10, unit="D")
 
         msg = "bad operand type for unary ~"
         with pytest.raises(TypeError, match=msg):
@@ -336,17 +298,17 @@ class TestTimedeltaUnaryOps:
             ~(td.to_timedelta64())
 
     def test_unary_ops(self):
-        td = Timedelta(10, unit="d")
+        td = Timedelta(10, unit="D")
 
         # __neg__, __pos__
-        assert -td == Timedelta(-10, unit="d")
-        assert -td == Timedelta("-10d")
-        assert +td == Timedelta(10, unit="d")
+        assert -td == Timedelta(-10, unit="D")
+        assert -td == Timedelta("-10D")
+        assert +td == Timedelta(10, unit="D")
 
         # __abs__, __abs__(__neg__)
         assert abs(td) == td
         assert abs(-td) == td
-        assert abs(-td) == Timedelta("10d")
+        assert abs(-td) == Timedelta("10D")
 
 
 class TestTimedeltas:
@@ -375,7 +337,8 @@ class TestTimedeltas:
         assert np.isnan(rng.total_seconds())
 
     def test_conversion(self):
-        for td in [Timedelta(10, unit="d"), Timedelta("1 days, 10:11:12.012345")]:
+        for td in [Timedelta(10, unit="D"), Timedelta("1 days, 10:11:12.012345")]:
+            td = td.as_unit("ns")
             pydt = td.to_pytimedelta()
             assert td == Timedelta(pydt)
             assert td == pydt
@@ -423,8 +386,8 @@ class TestTimedeltas:
         assert abs(td) == Timedelta("13:48:48")
         assert str(td) == "-1 days +10:11:12"
         assert -td == Timedelta("0 days 13:48:48")
-        assert -Timedelta("-1 days, 10:11:12")._value == 49728000000000
-        assert Timedelta("-1 days, 10:11:12")._value == -49728000000000
+        assert -Timedelta("-1 days, 10:11:12")._value == 49728000000
+        assert Timedelta("-1 days, 10:11:12")._value == -49728000000
 
         rng = to_timedelta("-1 days, 10:11:12.100123456")
         assert rng.days == -1
@@ -467,11 +430,13 @@ class TestTimedeltas:
         assert tup.microseconds == 999
         assert tup.nanoseconds == 0
 
+    # TODO: this is a test of to_timedelta string parsing
     def test_iso_conversion(self):
         # GH #21877
         expected = Timedelta(1, unit="s")
         assert to_timedelta("P0DT0H0M1S") == expected
 
+    # TODO: this is a test of to_timedelta returning NaT
     def test_nat_converters(self):
         result = to_timedelta("nat").to_numpy()
         assert result.dtype.kind == "M"
@@ -481,133 +446,6 @@ class TestTimedeltas:
         assert result.dtype.kind == "M"
         assert result.astype("int64") == iNaT
 
-    @pytest.mark.parametrize(
-        "unit, np_unit",
-        [(value, "W") for value in ["W", "w"]]
-        + [(value, "D") for value in ["D", "d", "days", "day", "Days", "Day"]]
-        + [
-            (value, "m")
-            for value in [
-                "m",
-                "minute",
-                "min",
-                "minutes",
-                "t",
-                "Minute",
-                "Min",
-                "Minutes",
-                "T",
-            ]
-        ]
-        + [
-            (value, "s")
-            for value in [
-                "s",
-                "seconds",
-                "sec",
-                "second",
-                "S",
-                "Seconds",
-                "Sec",
-                "Second",
-            ]
-        ]
-        + [
-            (value, "ms")
-            for value in [
-                "ms",
-                "milliseconds",
-                "millisecond",
-                "milli",
-                "millis",
-                "l",
-                "MS",
-                "Milliseconds",
-                "Millisecond",
-                "Milli",
-                "Millis",
-                "L",
-            ]
-        ]
-        + [
-            (value, "us")
-            for value in [
-                "us",
-                "microseconds",
-                "microsecond",
-                "micro",
-                "micros",
-                "u",
-                "US",
-                "Microseconds",
-                "Microsecond",
-                "Micro",
-                "Micros",
-                "U",
-            ]
-        ]
-        + [
-            (value, "ns")
-            for value in [
-                "ns",
-                "nanoseconds",
-                "nanosecond",
-                "nano",
-                "nanos",
-                "n",
-                "NS",
-                "Nanoseconds",
-                "Nanosecond",
-                "Nano",
-                "Nanos",
-                "N",
-            ]
-        ],
-    )
-    @pytest.mark.parametrize("wrapper", [np.array, list, pd.Index])
-    def test_unit_parser(self, unit, np_unit, wrapper):
-        # validate all units, GH 6855, GH 21762
-        # array-likes
-        expected = TimedeltaIndex(
-            [np.timedelta64(i, np_unit) for i in np.arange(5).tolist()],
-            dtype="m8[ns]",
-        )
-        # TODO(2.0): the desired output dtype may have non-nano resolution
-        result = to_timedelta(wrapper(range(5)), unit=unit)
-        tm.assert_index_equal(result, expected)
-        result = TimedeltaIndex(wrapper(range(5)), unit=unit)
-        tm.assert_index_equal(result, expected)
-
-        str_repr = [f"{x}{unit}" for x in np.arange(5)]
-        result = to_timedelta(wrapper(str_repr))
-        tm.assert_index_equal(result, expected)
-        result = to_timedelta(wrapper(str_repr))
-        tm.assert_index_equal(result, expected)
-
-        # scalar
-        expected = Timedelta(np.timedelta64(2, np_unit).astype("timedelta64[ns]"))
-        result = to_timedelta(2, unit=unit)
-        assert result == expected
-        result = Timedelta(2, unit=unit)
-        assert result == expected
-
-        result = to_timedelta(f"2{unit}")
-        assert result == expected
-        result = Timedelta(f"2{unit}")
-        assert result == expected
-
-    @pytest.mark.parametrize("unit", ["Y", "y", "M"])
-    def test_unit_m_y_raises(self, unit):
-        msg = "Units 'M', 'Y', and 'y' are no longer supported"
-        with pytest.raises(ValueError, match=msg):
-            Timedelta(10, unit)
-
-        with pytest.raises(ValueError, match=msg):
-            to_timedelta(10, unit)
-
-        with pytest.raises(ValueError, match=msg):
-            to_timedelta([1, 2], unit)
-
     def test_numeric_conversions(self):
         assert Timedelta(0) == np.timedelta64(0, "ns")
         assert Timedelta(10) == np.timedelta64(10, "ns")
@@ -616,7 +454,7 @@ class TestTimedeltas:
         assert Timedelta(10, unit="us") == np.timedelta64(10, "us")
         assert Timedelta(10, unit="ms") == np.timedelta64(10, "ms")
         assert Timedelta(10, unit="s") == np.timedelta64(10, "s")
-        assert Timedelta(10, unit="d") == np.timedelta64(10, "D")
+        assert Timedelta(10, unit="D") == np.timedelta64(10, "D")
 
     def test_timedelta_conversions(self):
         assert Timedelta(timedelta(seconds=1)) == np.timedelta64(1, "s").astype(
@@ -639,134 +477,8 @@ class TestTimedeltas:
         with pytest.raises(ValueError, match=msg):
             td.to_numpy(copy=True)
 
-    @pytest.mark.parametrize(
-        "freq,s1,s2",
-        [
-            # This first case has s1, s2 being the same as t1,t2 below
-            (
-                "N",
-                Timedelta("1 days 02:34:56.789123456"),
-                Timedelta("-1 days 02:34:56.789123456"),
-            ),
-            (
-                "U",
-                Timedelta("1 days 02:34:56.789123000"),
-                Timedelta("-1 days 02:34:56.789123000"),
-            ),
-            (
-                "L",
-                Timedelta("1 days 02:34:56.789000000"),
-                Timedelta("-1 days 02:34:56.789000000"),
-            ),
-            ("S", Timedelta("1 days 02:34:57"), Timedelta("-1 days 02:34:57")),
-            ("2S", Timedelta("1 days 02:34:56"), Timedelta("-1 days 02:34:56")),
-            ("5S", Timedelta("1 days 02:34:55"), Timedelta("-1 days 02:34:55")),
-            ("T", Timedelta("1 days 02:35:00"), Timedelta("-1 days 02:35:00")),
-            ("12T", Timedelta("1 days 02:36:00"), Timedelta("-1 days 02:36:00")),
-            ("H", Timedelta("1 days 03:00:00"), Timedelta("-1 days 03:00:00")),
-            ("d", Timedelta("1 days"), Timedelta("-1 days")),
-        ],
-    )
-    def test_round(self, freq, s1, s2):
-        t1 = Timedelta("1 days 02:34:56.789123456")
-        t2 = Timedelta("-1 days 02:34:56.789123456")
-
-        r1 = t1.round(freq)
-        assert r1 == s1
-        r2 = t2.round(freq)
-        assert r2 == s2
-
-    def test_round_invalid(self):
-        t1 = Timedelta("1 days 02:34:56.789123456")
-
-        for freq, msg in [
-            ("Y", "<YearEnd: month=12> is a non-fixed frequency"),
-            ("M", "<MonthEnd> is a non-fixed frequency"),
-            ("foobar", "Invalid frequency: foobar"),
-        ]:
-            with pytest.raises(ValueError, match=msg):
-                t1.round(freq)
-
-    def test_round_implementation_bounds(self):
-        # See also: analogous test for Timestamp
-        # GH#38964
-        result = Timedelta.min.ceil("s")
-        expected = Timedelta.min + Timedelta(seconds=1) - Timedelta(145224193)
-        assert result == expected
-
-        result = Timedelta.max.floor("s")
-        expected = Timedelta.max - Timedelta(854775807)
-        assert result == expected
-
-        with pytest.raises(OverflowError, match="value too large"):
-            Timedelta.min.floor("s")
-
-        # the second message here shows up in windows builds
-        msg = "|".join(
-            ["Python int too large to convert to C long", "int too big to convert"]
-        )
-        with pytest.raises(OverflowError, match=msg):
-            Timedelta.max.ceil("s")
-
-    @pytest.mark.xfail(reason="Failing on builds", strict=False)
-    @given(val=st.integers(min_value=iNaT + 1, max_value=lib.i8max))
-    @pytest.mark.parametrize(
-        "method", [Timedelta.round, Timedelta.floor, Timedelta.ceil]
-    )
-    def test_round_sanity(self, val, method):
-        val = np.int64(val)
-        td = Timedelta(val)
-
-        assert method(td, "ns") == td
-
-        res = method(td, "us")
-        nanos = 1000
-        assert np.abs((res - td).value) < nanos
-        assert res.value % nanos == 0
-
-        res = method(td, "ms")
-        nanos = 1_000_000
-        assert np.abs((res - td).value) < nanos
-        assert res.value % nanos == 0
-
-        res = method(td, "s")
-        nanos = 1_000_000_000
-        assert np.abs((res - td).value) < nanos
-        assert res.value % nanos == 0
-
-        res = method(td, "min")
-        nanos = 60 * 1_000_000_000
-        assert np.abs((res - td).value) < nanos
-        assert res.value % nanos == 0
-
-        res = method(td, "h")
-        nanos = 60 * 60 * 1_000_000_000
-        assert np.abs((res - td).value) < nanos
-        assert res.value % nanos == 0
-
-        res = method(td, "D")
-        nanos = 24 * 60 * 60 * 1_000_000_000
-        assert np.abs((res - td).value) < nanos
-        assert res.value % nanos == 0
-
-    @pytest.mark.parametrize("unit", ["ns", "us", "ms", "s"])
-    def test_round_non_nano(self, unit):
-        td = Timedelta("1 days 02:34:57").as_unit(unit)
-
-        res = td.round("min")
-        assert res == Timedelta("1 days 02:35:00")
-        assert res._creso == td._creso
-
-        res = td.floor("min")
-        assert res == Timedelta("1 days 02:34:00")
-        assert res._creso == td._creso
-
-        res = td.ceil("min")
-        assert res == Timedelta("1 days 02:35:00")
-        assert res._creso == td._creso
-
     def test_identity(self):
-        td = Timedelta(10, unit="d")
+        td = Timedelta(10, unit="D")
         assert isinstance(td, Timedelta)
         assert isinstance(td, timedelta)
 
@@ -781,7 +493,10 @@ class TestTimedeltas:
 
         assert Timedelta("1000") == np.timedelta64(1000, "ns")
         assert Timedelta("1000ns") == np.timedelta64(1000, "ns")
-        assert Timedelta("1000NS") == np.timedelta64(1000, "ns")
+
+        msg = "'NS' is deprecated and will be removed in a future version."
+        with tm.assert_produces_warning(Pandas4Warning, match=msg):
+            assert Timedelta("1000NS") == np.timedelta64(1000, "ns")
 
         assert Timedelta("10us") == np.timedelta64(10000, "ns")
         assert Timedelta("100us") == np.timedelta64(100000, "ns")
@@ -800,8 +515,10 @@ class TestTimedeltas:
         assert Timedelta("100s") == np.timedelta64(100000000000, "ns")
         assert Timedelta("1000s") == np.timedelta64(1000000000000, "ns")
 
-        assert Timedelta("1d") == conv(np.timedelta64(1, "D"))
-        assert Timedelta("-1d") == -conv(np.timedelta64(1, "D"))
+        msg = "'d' is deprecated and will be removed in a future version."
+        with tm.assert_produces_warning(Pandas4Warning, match=msg):
+            assert Timedelta("1d") == conv(np.timedelta64(1, "D"))
+        assert Timedelta("-1D") == -conv(np.timedelta64(1, "D"))
         assert Timedelta("1D") == conv(np.timedelta64(1, "D"))
         assert Timedelta("10D") == conv(np.timedelta64(10, "D"))
         assert Timedelta("100D") == conv(np.timedelta64(100, "D"))
@@ -850,9 +567,9 @@ class TestTimedeltas:
         with pytest.raises(ValueError, match=msg):
             Timedelta("- 1days, 00")
 
-    def test_pickle(self):
+    def test_pickle(self, temp_file):
         v = Timedelta("1 days 10:11:12.0123456")
-        v_p = tm.round_trip_pickle(v)
+        v_p = tm.round_trip_pickle(v, temp_file)
         assert v == v_p
 
     def test_timedelta_hash_equality(self):
@@ -871,6 +588,30 @@ class TestTimedeltas:
         ns_td = Timedelta(1, "ns")
         assert hash(ns_td) != hash(ns_td.to_pytimedelta())
 
+    @pytest.mark.slow
+    @pytest.mark.xfail(
+        reason="pd.Timedelta violates the Python hash invariant (GH#44504).",
+    )
+    @given(
+        st.integers(
+            min_value=(-sys.maxsize - 1) // 500,
+            max_value=sys.maxsize // 500,
+        )
+    )
+    def test_hash_equality_invariance(self, half_microseconds: int) -> None:
+        # GH#44504
+
+        nanoseconds = half_microseconds * 500
+
+        pandas_timedelta = Timedelta(nanoseconds)
+        numpy_timedelta = np.timedelta64(nanoseconds)
+
+        # See: https://docs.python.org/3/glossary.html#term-hashable
+        # Hashable objects which compare equal must have the same hash value.
+        assert pandas_timedelta != numpy_timedelta or hash(pandas_timedelta) == hash(
+            numpy_timedelta
+        )
+
     def test_implementation_limits(self):
         min_td = Timedelta(Timedelta.min)
         max_td = Timedelta(Timedelta.max)
@@ -904,39 +645,59 @@ class TestTimedeltas:
 
     def test_total_seconds_precision(self):
         # GH 19458
-        assert Timedelta("30S").total_seconds() == 30.0
+        assert Timedelta("30s").total_seconds() == 30.0
         assert Timedelta("0").total_seconds() == 0.0
-        assert Timedelta("-2S").total_seconds() == -2.0
-        assert Timedelta("5.324S").total_seconds() == 5.324
-        assert (Timedelta("30S").total_seconds() - 30.0) < 1e-20
-        assert (30.0 - Timedelta("30S").total_seconds()) < 1e-20
+        assert Timedelta("-2s").total_seconds() == -2.0
+        assert Timedelta("5.324s").total_seconds() == 5.324
+        assert (Timedelta("30s").total_seconds() - 30.0) < 1e-20
+        assert (30.0 - Timedelta("30s").total_seconds()) < 1e-20
 
     def test_resolution_string(self):
         assert Timedelta(days=1).resolution_string == "D"
-        assert Timedelta(days=1, hours=6).resolution_string == "H"
-        assert Timedelta(days=1, minutes=6).resolution_string == "T"
-        assert Timedelta(days=1, seconds=6).resolution_string == "S"
-        assert Timedelta(days=1, milliseconds=6).resolution_string == "L"
-        assert Timedelta(days=1, microseconds=6).resolution_string == "U"
-        assert Timedelta(days=1, nanoseconds=6).resolution_string == "N"
+        assert Timedelta(days=1, hours=6).resolution_string == "h"
+        assert Timedelta(days=1, minutes=6).resolution_string == "min"
+        assert Timedelta(days=1, seconds=6).resolution_string == "s"
+        assert Timedelta(days=1, milliseconds=6).resolution_string == "ms"
+        assert Timedelta(days=1, microseconds=6).resolution_string == "us"
+        assert Timedelta(days=1, nanoseconds=6).resolution_string == "ns"
 
     def test_resolution_deprecated(self):
         # GH#21344
         td = Timedelta(days=4, hours=3)
         result = td.resolution
-        assert result == Timedelta(nanoseconds=1)
+        assert result == Timedelta(microseconds=1)
 
         # Check that the attribute is available on the class, mirroring
         #  the stdlib timedelta behavior
         result = Timedelta.resolution
         assert result == Timedelta(nanoseconds=1)
 
+    @pytest.mark.parametrize(
+        "unit,unit_depr",
+        [
+            ("W", "w"),
+            ("D", "d"),
+            ("min", "MIN"),
+            ("s", "S"),
+            ("h", "H"),
+            ("ms", "MS"),
+            ("us", "US"),
+        ],
+    )
+    def test_unit_deprecated(self, unit, unit_depr):
+        # GH#59051
+        msg = f"'{unit_depr}' is deprecated and will be removed in a future version."
+
+        with tm.assert_produces_warning(Pandas4Warning, match=msg):
+            result = Timedelta(1, unit_depr)
+        assert result == Timedelta(1, unit)
+
 
 @pytest.mark.parametrize(
     "value, expected",
     [
-        (Timedelta("10S"), True),
-        (Timedelta("-10S"), True),
+        (Timedelta("10s"), True),
+        (Timedelta("-10s"), True),
         (Timedelta(10, unit="ns"), True),
         (Timedelta(0, unit="ns"), False),
         (Timedelta(-10, unit="ns"), True),
@@ -960,3 +721,20 @@ def test_timedelta_attribute_precision():
     result += td.nanoseconds
     expected = td._value
     assert result == expected
+
+
+def test_to_pytimedelta_large_values():
+    td = Timedelta(1152921504609987375, unit="ns")
+    result = td.to_pytimedelta()
+    expected = timedelta(days=13343, seconds=86304, microseconds=609987)
+    assert result == expected
+
+
+def test_timedelta_week_suffix():
+    # GH#12691 ensure 'W' suffix works as a string passed to Timedelta
+    expected = Timedelta("7 days")
+    result = Timedelta(1, unit="W")
+    assert result == expected
+
+    result = Timedelta("1W")
+    assert result == expected
