@@ -5,15 +5,23 @@ import string
 import numpy as np
 import pytest
 
-import pandas.util._test_decorators as td
-
 import pandas as pd
 import pandas._testing as tm
 from pandas.arrays import SparseArray
 
-BINARY_UFUNCS = [np.add, np.logaddexp]  # dunder op
-SPARSE = [True, False]
-SPARSE_IDS = ["sparse", "dense"]
+
+@pytest.fixture(params=[np.add, np.logaddexp])
+def ufunc(request):
+    # dunder op
+    return request.param
+
+
+@pytest.fixture(
+    params=[pytest.param(True, marks=pytest.mark.fails_arm_wheels), False],
+    ids=["sparse", "dense"],
+)
+def sparse(request):
+    return request.param
 
 
 @pytest.fixture
@@ -21,18 +29,17 @@ def arrays_for_binary_ufunc():
     """
     A pair of random, length-100 integer-dtype arrays, that are mostly 0.
     """
-    a1 = np.random.randint(0, 10, 100, dtype="int64")
-    a2 = np.random.randint(0, 10, 100, dtype="int64")
+    a1 = np.random.default_rng(2).integers(0, 10, 100, dtype="int64")
+    a2 = np.random.default_rng(2).integers(0, 10, 100, dtype="int64")
     a1[::3] = 0
     a2[::4] = 0
     return a1, a2
 
 
 @pytest.mark.parametrize("ufunc", [np.positive, np.floor, np.exp])
-@pytest.mark.parametrize("sparse", SPARSE, ids=SPARSE_IDS)
 def test_unary_ufunc(ufunc, sparse):
     # Test that ufunc(pd.Series) == pd.Series(ufunc)
-    arr = np.random.randint(0, 10, 10, dtype="int64")
+    arr = np.random.default_rng(2).integers(0, 10, 10, dtype="int64")
     arr[::2] = 0
     if sparse:
         arr = SparseArray(arr, dtype=pd.SparseDtype("int64", 0))
@@ -46,8 +53,6 @@ def test_unary_ufunc(ufunc, sparse):
     tm.assert_series_equal(result, expected)
 
 
-@pytest.mark.parametrize("ufunc", BINARY_UFUNCS)
-@pytest.mark.parametrize("sparse", SPARSE, ids=SPARSE_IDS)
 @pytest.mark.parametrize("flip", [True, False], ids=["flipped", "straight"])
 def test_binary_ufunc_with_array(flip, sparse, ufunc, arrays_for_binary_ufunc):
     # Test that ufunc(pd.Series(a), array) == pd.Series(ufunc(a, b))
@@ -72,8 +77,6 @@ def test_binary_ufunc_with_array(flip, sparse, ufunc, arrays_for_binary_ufunc):
     tm.assert_series_equal(result, expected)
 
 
-@pytest.mark.parametrize("ufunc", BINARY_UFUNCS)
-@pytest.mark.parametrize("sparse", SPARSE, ids=SPARSE_IDS)
 @pytest.mark.parametrize("flip", [True, False], ids=["flipped", "straight"])
 def test_binary_ufunc_with_index(flip, sparse, ufunc, arrays_for_binary_ufunc):
     # Test that
@@ -101,8 +104,6 @@ def test_binary_ufunc_with_index(flip, sparse, ufunc, arrays_for_binary_ufunc):
     tm.assert_series_equal(result, expected)
 
 
-@pytest.mark.parametrize("ufunc", BINARY_UFUNCS)
-@pytest.mark.parametrize("sparse", SPARSE, ids=SPARSE_IDS)
 @pytest.mark.parametrize("shuffle", [True, False], ids=["unaligned", "aligned"])
 @pytest.mark.parametrize("flip", [True, False], ids=["flipped", "straight"])
 def test_binary_ufunc_with_series(
@@ -120,7 +121,7 @@ def test_binary_ufunc_with_series(
     series = pd.Series(a1, name=name)
     other = pd.Series(a2, name=name)
 
-    idx = np.random.permutation(len(a1))
+    idx = np.random.default_rng(2).permutation(len(a1))
 
     if shuffle:
         other = other.take(idx)
@@ -143,8 +144,6 @@ def test_binary_ufunc_with_series(
     tm.assert_series_equal(result, expected)
 
 
-@pytest.mark.parametrize("ufunc", BINARY_UFUNCS)
-@pytest.mark.parametrize("sparse", SPARSE, ids=SPARSE_IDS)
 @pytest.mark.parametrize("flip", [True, False])
 def test_binary_ufunc_scalar(ufunc, sparse, flip, arrays_for_binary_ufunc):
     # Test that
@@ -170,7 +169,6 @@ def test_binary_ufunc_scalar(ufunc, sparse, flip, arrays_for_binary_ufunc):
 
 
 @pytest.mark.parametrize("ufunc", [np.divmod])  # TODO: np.modf, np.frexp
-@pytest.mark.parametrize("sparse", SPARSE, ids=SPARSE_IDS)
 @pytest.mark.parametrize("shuffle", [True, False])
 @pytest.mark.filterwarnings("ignore:divide by zero:RuntimeWarning")
 def test_multiple_output_binary_ufuncs(ufunc, sparse, shuffle, arrays_for_binary_ufunc):
@@ -203,7 +201,6 @@ def test_multiple_output_binary_ufuncs(ufunc, sparse, shuffle, arrays_for_binary
     tm.assert_series_equal(result[1], pd.Series(expected[1]))
 
 
-@pytest.mark.parametrize("sparse", SPARSE, ids=SPARSE_IDS)
 def test_multiple_output_ufunc(sparse, arrays_for_binary_ufunc):
     # Test that the same conditions from unary input apply to multi-output
     # ufuncs
@@ -223,8 +220,6 @@ def test_multiple_output_ufunc(sparse, arrays_for_binary_ufunc):
     tm.assert_series_equal(result[1], pd.Series(expected[1], name="name"))
 
 
-@pytest.mark.parametrize("sparse", SPARSE, ids=SPARSE_IDS)
-@pytest.mark.parametrize("ufunc", BINARY_UFUNCS)
 def test_binary_ufunc_drops_series_name(ufunc, sparse, arrays_for_binary_ufunc):
     # Drop the names when they differ.
     a1, a2 = arrays_for_binary_ufunc
@@ -280,7 +275,7 @@ class TestNumpyReductions:
 
         if isinstance(values, pd.core.arrays.SparseArray):
             mark = pytest.mark.xfail(reason="SparseArray has no 'prod'")
-            request.node.add_marker(mark)
+            request.applymarker(mark)
 
         if values.dtype.kind in "iuf":
             result = np.multiply.reduce(obj)
@@ -297,7 +292,7 @@ class TestNumpyReductions:
         else:
             msg = "|".join(
                 [
-                    "does not support reduction",
+                    "does not support operation",
                     "unsupported operand type",
                     "ufunc 'multiply' cannot use operands",
                 ]
@@ -327,7 +322,7 @@ class TestNumpyReductions:
         else:
             msg = "|".join(
                 [
-                    "does not support reduction",
+                    "does not support operation",
                     "unsupported operand type",
                     "ufunc 'add' cannot use operands",
                 ]
@@ -335,12 +330,12 @@ class TestNumpyReductions:
             with pytest.raises(TypeError, match=msg):
                 np.add.reduce(obj)
 
-    def test_max(self, values_for_np_reduce, box_with_array):
+    def test_max(self, values_for_np_reduce, box_with_array, using_python_scalars):
         box = box_with_array
         values = values_for_np_reduce
 
         same_type = True
-        if box is pd.Index and values.dtype.kind in ["i", "f"]:
+        if box is pd.Index and values.dtype.kind in "if":
             # ATM Index casts to object, so we get python ints/floats
             same_type = False
 
@@ -354,17 +349,19 @@ class TestNumpyReductions:
             tm.assert_series_equal(result, expected)
         else:
             expected = values[1]
+            if using_python_scalars and values.dtype.kind in "if":
+                expected = expected.item()
             assert result == expected
             if same_type:
                 # check we have e.g. Timestamp instead of dt64
                 assert type(result) == type(expected)
 
-    def test_min(self, values_for_np_reduce, box_with_array):
+    def test_min(self, values_for_np_reduce, box_with_array, using_python_scalars):
         box = box_with_array
         values = values_for_np_reduce
 
         same_type = True
-        if box is pd.Index and values.dtype.kind in ["i", "f"]:
+        if box is pd.Index and values.dtype.kind in "if":
             # ATM Index casts to object, so we get python ints/floats
             same_type = False
 
@@ -377,6 +374,8 @@ class TestNumpyReductions:
             tm.assert_series_equal(result, expected)
         else:
             expected = values[0]
+            if using_python_scalars and values.dtype.kind in ["i", "f"]:
+                expected = expected.item()
             assert result == expected
             if same_type:
                 # check we have e.g. Timestamp instead of dt64
@@ -419,7 +418,7 @@ def test_outer():
     ser = pd.Series([1, 2, 3])
     obj = np.array([1, 2, 3])
 
-    with pytest.raises(NotImplementedError, match=tm.EMPTY_STRING_PATTERN):
+    with pytest.raises(NotImplementedError, match="^$"):
         np.subtract.outer(ser, obj)
 
 
@@ -433,6 +432,16 @@ def test_np_matmul():
     tm.assert_frame_equal(expected, result)
 
 
+@pytest.mark.parametrize("box", [pd.Index, pd.Series])
+def test_np_matmul_1D(box, using_python_scalars):
+    result = np.matmul(box([1, 2]), box([2, 3]))
+    assert result == 8
+    if using_python_scalars:
+        assert type(result) == int, type(result)
+    else:
+        assert type(result) == np.int64, type(result)
+
+
 def test_array_ufuncs_for_many_arguments():
     # GH39853
     def add3(x, y, z):
@@ -455,12 +464,12 @@ def test_array_ufuncs_for_many_arguments():
         ufunc(ser, ser, df)
 
 
-# TODO(CoW) see https://github.com/pandas-dev/pandas/pull/51082
-@td.skip_copy_on_write_not_yet_implemented
-def test_np_fix():
-    # np.fix is not a ufunc but is composed of several ufunc calls under the hood
-    # with `out` and `where` keywords
+def test_np_trunc():
+    # This used to test np.fix, which is not a ufunc but is composed of
+    # several ufunc calls under the hood with `out` and `where` keywords. But numpy
+    # is deprecating that (or at least discussing deprecating) in favor of np.trunc,
+    # which _is_ a ufunc without the out keyword usage.
     ser = pd.Series([-1.5, -0.5, 0.5, 1.5])
-    result = np.fix(ser)
+    result = np.trunc(ser)
     expected = pd.Series([-1.0, -0.0, 0.0, 1.0])
     tm.assert_series_equal(result, expected)
