@@ -3,12 +3,11 @@ from functools import partial
 import string
 import subprocess
 import sys
-import textwrap
 
 import numpy as np
 import pytest
 
-import pandas.util._test_decorators as td
+from pandas.compat import WASM
 
 import pandas as pd
 from pandas import Series
@@ -18,84 +17,91 @@ import pandas.core.common as com
 from pandas.util.version import Version
 
 
-def test_get_callable_name():
-    getname = com.get_callable_name
-
-    def fn(x):
+class TestGetCallableName:
+    def fn(self, x):
         return x
 
+    partial1 = partial(fn)
+    partial2 = partial(partial1)
     lambda_ = lambda x: x
-    part1 = partial(fn)
-    part2 = partial(part1)
 
-    class somecall:
+    class SomeCall:
         def __call__(self):
-            # This shouldn't actually get called below; somecall.__init__
+            # This shouldn't actually get called below; SomeCall.__init__
             #  should.
             raise NotImplementedError
 
-    assert getname(fn) == "fn"
-    assert getname(lambda_)
-    assert getname(part1) == "fn"
-    assert getname(part2) == "fn"
-    assert getname(somecall()) == "somecall"
-    assert getname(1) is None
-
-
-def test_any_none():
-    assert com.any_none(1, 2, 3, None)
-    assert not com.any_none(1, 2, 3, 4)
-
-
-def test_all_not_none():
-    assert com.all_not_none(1, 2, 3, 4)
-    assert not com.all_not_none(1, 2, 3, None)
-    assert not com.all_not_none(None, None, None, None)
-
-
-def test_random_state():
-    # Check with seed
-    state = com.random_state(5)
-    assert state.uniform() == np.random.RandomState(5).uniform()
-
-    # Check with random state object
-    state2 = np.random.RandomState(10)
-    assert com.random_state(state2).uniform() == np.random.RandomState(10).uniform()
-
-    # check with no arg random state
-    assert com.random_state() is np.random
-
-    # check array-like
-    # GH32503
-    state_arr_like = np.random.default_rng(None).integers(
-        0, 2**31, size=624, dtype="uint32"
-    )
-    assert (
-        com.random_state(state_arr_like).uniform()
-        == np.random.RandomState(state_arr_like).uniform()
+    @pytest.mark.parametrize(
+        "func, expected",
+        [
+            (fn, "fn"),
+            (partial1, "fn"),
+            (partial2, "fn"),
+            (lambda_, "<lambda>"),
+            (SomeCall(), "SomeCall"),
+            (1, None),
+        ],
     )
+    def test_get_callable_name(self, func, expected):
+        assert com.get_callable_name(func) == expected
 
-    # Check BitGenerators
-    # GH32503
-    assert (
-        com.random_state(np.random.MT19937(3)).uniform()
-        == np.random.RandomState(np.random.MT19937(3)).uniform()
-    )
-    assert (
-        com.random_state(np.random.PCG64(11)).uniform()
-        == np.random.RandomState(np.random.PCG64(11)).uniform()
-    )
 
-    # Error for floats or strings
-    msg = (
-        "random_state must be an integer, array-like, a BitGenerator, Generator, "
-        "a numpy RandomState, or None"
-    )
-    with pytest.raises(ValueError, match=msg):
-        com.random_state("test")
+class TestRandomState:
+    def test_seed(self):
+        seed = 5
+        assert com.random_state(seed).uniform() == np.random.RandomState(seed).uniform()
 
-    with pytest.raises(ValueError, match=msg):
-        com.random_state(5.5)
+    def test_object(self):
+        seed = 10
+        state_obj = np.random.RandomState(seed)
+        assert (
+            com.random_state(state_obj).uniform()
+            == np.random.RandomState(seed).uniform()
+        )
+
+    def test_default(self):
+        assert com.random_state() is np.random
+
+    def test_array_like(self):
+        state = np.random.default_rng(None).integers(0, 2**31, size=624, dtype="uint32")
+        assert (
+            com.random_state(state).uniform() == np.random.RandomState(state).uniform()
+        )
+
+    def test_bit_generators(self):
+        seed = 3
+        assert (
+            com.random_state(np.random.MT19937(seed)).uniform()
+            == np.random.RandomState(np.random.MT19937(seed)).uniform()
+        )
+
+        seed = 11
+        assert (
+            com.random_state(np.random.PCG64(seed)).uniform()
+            == np.random.RandomState(np.random.PCG64(seed)).uniform()
+        )
+
+    @pytest.mark.parametrize("state", ["test", 5.5])
+    def test_error(self, state):
+        msg = (
+            "random_state must be an integer, array-like, a BitGenerator, Generator, "
+            "a numpy RandomState, or None"
+        )
+        with pytest.raises(ValueError, match=msg):
+            com.random_state(state)
+
+
+@pytest.mark.parametrize("args, expected", [((1, 2, None), True), ((1, 2, 3), False)])
+def test_any_none(args, expected):
+    assert com.any_none(*args) is expected
+
+
+@pytest.mark.parametrize(
+    "args, expected",
+    [((1, 2, 3), True), ((1, 2, None), False), ((None, None, None), False)],
+)
+def test_all_not_none(args, expected):
+    assert com.all_not_none(*args) is expected
 
 
 @pytest.mark.parametrize(
@@ -137,21 +143,32 @@ def test_maybe_match_name(left, right, expected):
     assert res is expected or res == expected
 
 
+@pytest.mark.parametrize(
+    "into, msg",
+    [
+        (
+            # uninitialized defaultdict
+            collections.defaultdict,
+            r"to_dict\(\) only accepts initialized defaultdicts",
+        ),
+        (
+            # non-mapping subtypes,, instance
+            [],
+            "unsupported type: <class 'list'>",
+        ),
+        (
+            # non-mapping subtypes, class
+            list,
+            "unsupported type: <class 'list'>",
+        ),
+    ],
+)
+def test_standardize_mapping_type_error(into, msg):
+    with pytest.raises(TypeError, match=msg):
+        com.standardize_mapping(into)
+
+
 def test_standardize_mapping():
-    # No uninitialized defaultdicts
-    msg = r"to_dict\(\) only accepts initialized defaultdicts"
-    with pytest.raises(TypeError, match=msg):
-        com.standardize_mapping(collections.defaultdict)
-
-    # No non-mapping subtypes, instance
-    msg = "unsupported type: <class 'list'>"
-    with pytest.raises(TypeError, match=msg):
-        com.standardize_mapping([])
-
-    # No non-mapping subtypes, class
-    with pytest.raises(TypeError, match=msg):
-        com.standardize_mapping(list)
-
     fill = {"bad": "data"}
     assert com.standardize_mapping(fill) == dict
 
@@ -173,18 +190,16 @@ def test_version_tag():
     version = Version(pd.__version__)
     try:
         version > Version("0.0.1")
-    except TypeError:
+    except TypeError as err:
         raise ValueError(
             "No git tags exist, please sync tags between upstream and your repo"
-        )
+        ) from err
 
 
-@pytest.mark.parametrize(
-    "obj", [(obj,) for obj in pd.__dict__.values() if callable(obj)]
-)
-def test_serializable(obj):
+@pytest.mark.parametrize("obj", [obj for obj in pd.__dict__.values() if callable(obj)])
+def test_serializable(obj, temp_file):
     # GH 35611
-    unpickled = tm.round_trip_pickle(obj)
+    unpickled = tm.round_trip_pickle(obj, temp_file)
     assert type(obj) == type(unpickled)
 
 
@@ -219,6 +234,12 @@ class TestIsBoolIndexer:
         expected = df[[]]
         tm.assert_frame_equal(result, expected)
 
+    @pytest.mark.parametrize("scalar", [1, True])
+    def test_numpyextensionarray(self, scalar):
+        # GH 63391
+        arr = pd.arrays.NumpyExtensionArray(np.array([scalar]))
+        assert com.is_bool_indexer(arr) is isinstance(scalar, bool)
+
 
 @pytest.mark.parametrize("with_exception", [True, False])
 def test_temp_setattr(with_exception):
@@ -236,6 +257,7 @@ def test_temp_setattr(with_exception):
     assert ser.name == "first"
 
 
+@pytest.mark.skipif(WASM, reason="Can't start subprocesses in WASM")
 @pytest.mark.single_cpu
 def test_str_size():
     # GH#21758
@@ -249,41 +271,3 @@ def test_str_size():
     ]
     result = subprocess.check_output(call).decode()[-4:-1].strip("\n")
     assert int(result) == int(expected)
-
-
-@pytest.mark.single_cpu
-def test_bz2_missing_import():
-    # Check whether bz2 missing import is handled correctly (issue #53857)
-    code = """
-        import sys
-        sys.modules['bz2'] = None
-        import pytest
-        import pandas as pd
-        from pandas.compat import get_bz2_file
-        msg = 'bz2 module not available.'
-        with pytest.raises(RuntimeError, match=msg):
-            get_bz2_file()
-    """
-    code = textwrap.dedent(code)
-    call = [sys.executable, "-c", code]
-    subprocess.check_output(call)
-
-
-@td.skip_if_installed("pyarrow")
-@pytest.mark.parametrize("module", ["pandas", "pandas.arrays"])
-def test_pyarrow_missing_warn(module):
-    # GH56896
-    response = subprocess.run(
-        [sys.executable, "-c", f"import {module}"],
-        capture_output=True,
-        check=True,
-    )
-    msg = """
-Pyarrow will become a required dependency of pandas in the next major release of pandas (pandas 3.0),
-(to allow more performant data types, such as the Arrow string type, and better interoperability with other libraries)
-but was not found to be installed on your system.
-If this would cause problems for you,
-please provide us feedback at https://github.com/pandas-dev/pandas/issues/54466
-"""  # noqa: E501
-    stderr_msg = response.stderr.decode("utf-8")
-    assert msg in stderr_msg, stderr_msg
