@@ -1,6 +1,4 @@
 from functools import partial
-import operator
-import warnings
 
 import numpy as np
 import pytest
@@ -16,7 +14,6 @@ from pandas import (
 )
 import pandas._testing as tm
 from pandas.core import nanops
-from pandas.core.arrays import DatetimeArray
 
 use_bn = nanops._USE_BOTTLENECK
 
@@ -35,8 +32,7 @@ def arr_shape():
 
 @pytest.fixture
 def arr_float(arr_shape):
-    np.random.seed(11235)
-    return np.random.randn(*arr_shape)
+    return np.random.default_rng(2).standard_normal(arr_shape)
 
 
 @pytest.fixture
@@ -46,14 +42,12 @@ def arr_complex(arr_float):
 
 @pytest.fixture
 def arr_int(arr_shape):
-    np.random.seed(11235)
-    return np.random.randint(-10, 10, arr_shape)
+    return np.random.default_rng(2).integers(-10, 10, arr_shape)
 
 
 @pytest.fixture
 def arr_bool(arr_shape):
-    np.random.seed(11235)
-    return np.random.randint(0, 2, arr_shape) == 0
+    return np.random.default_rng(2).integers(0, 2, arr_shape) == 0
 
 
 @pytest.fixture
@@ -68,14 +62,12 @@ def arr_utf(arr_float):
 
 @pytest.fixture
 def arr_date(arr_shape):
-    np.random.seed(11235)
-    return np.random.randint(0, 20000, arr_shape).astype("M8[ns]")
+    return np.random.default_rng(2).integers(0, 20000, arr_shape).astype("M8[ns]")
 
 
 @pytest.fixture
 def arr_tdelta(arr_shape):
-    np.random.seed(11235)
-    return np.random.randint(0, 20000, arr_shape).astype("m8[ns]")
+    return np.random.default_rng(2).integers(0, 20000, arr_shape).astype("m8[ns]")
 
 
 @pytest.fixture
@@ -192,20 +184,23 @@ def arr_nan_float1_1d(arr_nan_float1):
 
 class TestnanopsDataFrame:
     def setup_method(self):
-        np.random.seed(11235)
         nanops._USE_BOTTLENECK = False
 
         arr_shape = (11, 7)
 
-        self.arr_float = np.random.randn(*arr_shape)
-        self.arr_float1 = np.random.randn(*arr_shape)
+        self.arr_float = np.random.default_rng(2).standard_normal(arr_shape)
+        self.arr_float1 = np.random.default_rng(2).standard_normal(arr_shape)
         self.arr_complex = self.arr_float + self.arr_float1 * 1j
-        self.arr_int = np.random.randint(-10, 10, arr_shape)
-        self.arr_bool = np.random.randint(0, 2, arr_shape) == 0
+        self.arr_int = np.random.default_rng(2).integers(-10, 10, arr_shape)
+        self.arr_bool = np.random.default_rng(2).integers(0, 2, arr_shape) == 0
         self.arr_str = np.abs(self.arr_float).astype("S")
         self.arr_utf = np.abs(self.arr_float).astype("U")
-        self.arr_date = np.random.randint(0, 20000, arr_shape).astype("M8[ns]")
-        self.arr_tdelta = np.random.randint(0, 20000, arr_shape).astype("m8[ns]")
+        self.arr_date = (
+            np.random.default_rng(2).integers(0, 20000, arr_shape).astype("M8[ns]")
+        )
+        self.arr_tdelta = (
+            np.random.default_rng(2).integers(0, 20000, arr_shape).astype("m8[ns]")
+        )
 
         self.arr_nan = np.tile(np.nan, arr_shape)
         self.arr_float_nan = np.vstack([self.arr_float, self.arr_nan])
@@ -301,6 +296,7 @@ class TestnanopsDataFrame:
         self,
         testfunc,
         targfunc,
+        testar,
         testarval,
         targarval,
         skipna,
@@ -308,7 +304,7 @@ class TestnanopsDataFrame:
         empty_targfunc=None,
         **kwargs,
     ):
-        for axis in list(range(targarval.ndim)) + [None]:
+        for axis in [*list(range(targarval.ndim)), None]:
             targartempval = targarval if skipna else testarval
             if skipna and empty_targfunc and isna(targartempval).all():
                 targ = empty_targfunc(targartempval, axis=axis, **kwargs)
@@ -324,7 +320,24 @@ class TestnanopsDataFrame:
                 else:
                     targ = bool(targ)
 
+            if testfunc.__name__ in ["nanargmax", "nanargmin"] and (
+                testar.startswith("arr_nan")
+                or (testar.endswith("nan") and (not skipna or axis == 1))
+            ):
+                with pytest.raises(ValueError, match="Encountered .* NA value"):
+                    testfunc(testarval, axis=axis, skipna=skipna, **kwargs)
+                return
             res = testfunc(testarval, axis=axis, skipna=skipna, **kwargs)
+
+            if (
+                isinstance(targ, np.complex128)
+                and isinstance(res, float)
+                and np.isnan(targ)
+                and np.isnan(res)
+            ):
+                # GH#18463
+                targ = res
+
             self.check_results(targ, res, axis, check_dtype=check_dtype)
             if skipna:
                 res = testfunc(testarval, axis=axis, **kwargs)
@@ -345,6 +358,7 @@ class TestnanopsDataFrame:
         self.check_fun_data(
             testfunc,
             targfunc,
+            testar,
             testarval2,
             targarval2,
             skipna=skipna,
@@ -365,6 +379,7 @@ class TestnanopsDataFrame:
         self.check_fun_data(
             testfunc,
             targfunc,
+            testar,
             testarval,
             targarval,
             skipna=skipna,
@@ -458,17 +473,16 @@ class TestnanopsDataFrame:
             nanops.nanmean, np.mean, skipna, allow_obj=False, allow_date=False
         )
 
+    @pytest.mark.filterwarnings("ignore::RuntimeWarning")
     def test_nanmedian(self, skipna):
-        with warnings.catch_warnings(record=True):
-            warnings.simplefilter("ignore", RuntimeWarning)
-            self.check_funs(
-                nanops.nanmedian,
-                np.median,
-                skipna,
-                allow_complex=False,
-                allow_date=False,
-                allow_obj="convert",
-            )
+        self.check_funs(
+            nanops.nanmedian,
+            np.median,
+            skipna,
+            allow_complex=False,
+            allow_date=False,
+            allow_obj="convert",
+        )
 
     @pytest.mark.parametrize("ddof", range(3))
     def test_nanvar(self, ddof, skipna):
@@ -494,15 +508,14 @@ class TestnanopsDataFrame:
             ddof=ddof,
         )
 
-    @td.skip_if_no_scipy
     @pytest.mark.parametrize("ddof", range(3))
     def test_nansem(self, ddof, skipna):
-        from scipy.stats import sem
+        sp_stats = pytest.importorskip("scipy.stats")
 
         with np.errstate(invalid="ignore"):
             self.check_funs(
                 nanops.nansem,
-                sem,
+                sp_stats.sem,
                 skipna,
                 allow_complex=False,
                 allow_date=False,
@@ -511,13 +524,12 @@ class TestnanopsDataFrame:
                 ddof=ddof,
             )
 
+    @pytest.mark.filterwarnings("ignore::RuntimeWarning")
     @pytest.mark.parametrize(
         "nan_op,np_op", [(nanops.nanmin, np.min), (nanops.nanmax, np.max)]
     )
     def test_nanops_with_warnings(self, nan_op, np_op, skipna):
-        with warnings.catch_warnings(record=True):
-            warnings.simplefilter("ignore", RuntimeWarning)
-            self.check_funs(nan_op, np_op, skipna, allow_obj=False)
+        self.check_funs(nan_op, np_op, skipna, allow_obj=False)
 
     def _argminmax_wrap(self, value, axis=None, func=None):
         res = func(value, axis)
@@ -525,26 +537,21 @@ class TestnanopsDataFrame:
         nullnan = isna(nans)
         if res.ndim:
             res[nullnan] = -1
-        elif (
-            hasattr(nullnan, "all")
-            and nullnan.all()
-            or not hasattr(nullnan, "all")
-            and nullnan
+        elif (hasattr(nullnan, "all") and nullnan.all()) or (
+            not hasattr(nullnan, "all") and nullnan
         ):
             res = -1
         return res
 
+    @pytest.mark.filterwarnings("ignore::RuntimeWarning")
     def test_nanargmax(self, skipna):
-        with warnings.catch_warnings(record=True):
-            warnings.simplefilter("ignore", RuntimeWarning)
-            func = partial(self._argminmax_wrap, func=np.argmax)
-            self.check_funs(nanops.nanargmax, func, skipna, allow_obj=False)
+        func = partial(self._argminmax_wrap, func=np.argmax)
+        self.check_funs(nanops.nanargmax, func, skipna, allow_obj=False)
 
+    @pytest.mark.filterwarnings("ignore::RuntimeWarning")
     def test_nanargmin(self, skipna):
-        with warnings.catch_warnings(record=True):
-            warnings.simplefilter("ignore", RuntimeWarning)
-            func = partial(self._argminmax_wrap, func=np.argmin)
-            self.check_funs(nanops.nanargmin, func, skipna, allow_obj=False)
+        func = partial(self._argminmax_wrap, func=np.argmin)
+        self.check_funs(nanops.nanargmin, func, skipna, allow_obj=False)
 
     def _skew_kurt_wrap(self, values, axis=None, func=None):
         if not isinstance(values.dtype.type, np.floating):
@@ -558,11 +565,10 @@ class TestnanopsDataFrame:
             return 0.0
         return result
 
-    @td.skip_if_no_scipy
     def test_nanskew(self, skipna):
-        from scipy.stats import skew
+        sp_stats = pytest.importorskip("scipy.stats")
 
-        func = partial(self._skew_kurt_wrap, func=skew)
+        func = partial(self._skew_kurt_wrap, func=sp_stats.skew)
         with np.errstate(invalid="ignore"):
             self.check_funs(
                 nanops.nanskew,
@@ -573,11 +579,10 @@ class TestnanopsDataFrame:
                 allow_tdelta=False,
             )
 
-    @td.skip_if_no_scipy
     def test_nankurt(self, skipna):
-        from scipy.stats import kurtosis
+        sp_stats = pytest.importorskip("scipy.stats")
 
-        func1 = partial(kurtosis, fisher=True)
+        func1 = partial(sp_stats.kurtosis, fisher=True)
         func = partial(self._skew_kurt_wrap, func=func1)
         with np.errstate(invalid="ignore"):
             self.check_funs(
@@ -705,30 +710,28 @@ class TestnanopsDataFrame:
         targ1 = np.corrcoef(self.arr_float_1d.flat, self.arr_float1_1d.flat)[0, 1]
         self.check_nancorr_nancov_1d(nanops.nancorr, targ0, targ1, method="pearson")
 
-    @td.skip_if_no_scipy
     def test_nancorr_kendall(self):
-        from scipy.stats import kendalltau
+        sp_stats = pytest.importorskip("scipy.stats")
 
-        targ0 = kendalltau(self.arr_float_2d, self.arr_float1_2d)[0]
-        targ1 = kendalltau(self.arr_float_2d.flat, self.arr_float1_2d.flat)[0]
+        targ0 = sp_stats.kendalltau(self.arr_float_2d, self.arr_float1_2d)[0]
+        targ1 = sp_stats.kendalltau(self.arr_float_2d.flat, self.arr_float1_2d.flat)[0]
         self.check_nancorr_nancov_2d(nanops.nancorr, targ0, targ1, method="kendall")
-        targ0 = kendalltau(self.arr_float_1d, self.arr_float1_1d)[0]
-        targ1 = kendalltau(self.arr_float_1d.flat, self.arr_float1_1d.flat)[0]
+        targ0 = sp_stats.kendalltau(self.arr_float_1d, self.arr_float1_1d)[0]
+        targ1 = sp_stats.kendalltau(self.arr_float_1d.flat, self.arr_float1_1d.flat)[0]
         self.check_nancorr_nancov_1d(nanops.nancorr, targ0, targ1, method="kendall")
 
-    @td.skip_if_no_scipy
     def test_nancorr_spearman(self):
-        from scipy.stats import spearmanr
+        sp_stats = pytest.importorskip("scipy.stats")
 
-        targ0 = spearmanr(self.arr_float_2d, self.arr_float1_2d)[0]
-        targ1 = spearmanr(self.arr_float_2d.flat, self.arr_float1_2d.flat)[0]
+        targ0 = sp_stats.spearmanr(self.arr_float_2d, self.arr_float1_2d)[0]
+        targ1 = sp_stats.spearmanr(self.arr_float_2d.flat, self.arr_float1_2d.flat)[0]
         self.check_nancorr_nancov_2d(nanops.nancorr, targ0, targ1, method="spearman")
-        targ0 = spearmanr(self.arr_float_1d, self.arr_float1_1d)[0]
-        targ1 = spearmanr(self.arr_float_1d.flat, self.arr_float1_1d.flat)[0]
+        targ0 = sp_stats.spearmanr(self.arr_float_1d, self.arr_float1_1d)[0]
+        targ1 = sp_stats.spearmanr(self.arr_float_1d.flat, self.arr_float1_1d.flat)[0]
         self.check_nancorr_nancov_1d(nanops.nancorr, targ0, targ1, method="spearman")
 
-    @td.skip_if_no_scipy
     def test_invalid_method(self):
+        pytest.importorskip("scipy")
         targ0 = np.corrcoef(self.arr_float_2d, self.arr_float1_2d)[0, 1]
         targ1 = np.corrcoef(self.arr_float_2d.flat, self.arr_float1_2d.flat)[0, 1]
         msg = "Unknown method 'foo', expected one of 'kendall', 'spearman'"
@@ -744,53 +747,6 @@ class TestnanopsDataFrame:
         self.check_nancorr_nancov_1d(nanops.nancov, targ0, targ1)
 
 
-@pytest.mark.parametrize(
-    "op,nanop",
-    [
-        (operator.eq, nanops.naneq),
-        (operator.ne, nanops.nanne),
-        (operator.gt, nanops.nangt),
-        (operator.ge, nanops.nange),
-        (operator.lt, nanops.nanlt),
-        (operator.le, nanops.nanle),
-    ],
-)
-def test_nan_comparison(request, op, nanop, disable_bottleneck):
-    arr_float = request.getfixturevalue("arr_float")
-    arr_float1 = request.getfixturevalue("arr_float")
-    targ0 = op(arr_float, arr_float1)
-    arr_nan = request.getfixturevalue("arr_nan")
-    arr_nan_nan = request.getfixturevalue("arr_nan_nan")
-    arr_float_nan = request.getfixturevalue("arr_float_nan")
-    arr_float1_nan = request.getfixturevalue("arr_float_nan")
-    arr_nan_float1 = request.getfixturevalue("arr_nan_float1")
-
-    while targ0.ndim:
-        res0 = nanop(arr_float, arr_float1)
-        tm.assert_almost_equal(targ0, res0)
-
-        if targ0.ndim > 1:
-            targ1 = np.vstack([targ0, arr_nan])
-        else:
-            targ1 = np.hstack([targ0, arr_nan])
-        res1 = nanop(arr_float_nan, arr_float1_nan)
-        tm.assert_numpy_array_equal(targ1, res1, check_dtype=False)
-
-        targ2 = arr_nan_nan
-        res2 = nanop(arr_float_nan, arr_nan_float1)
-        tm.assert_numpy_array_equal(targ2, res2, check_dtype=False)
-
-        # Lower dimension for next step in the loop
-        arr_float = np.take(arr_float, 0, axis=-1)
-        arr_float1 = np.take(arr_float1, 0, axis=-1)
-        arr_nan = np.take(arr_nan, 0, axis=-1)
-        arr_nan_nan = np.take(arr_nan_nan, 0, axis=-1)
-        arr_float_nan = np.take(arr_float_nan, 0, axis=-1)
-        arr_float1_nan = np.take(arr_float1_nan, 0, axis=-1)
-        arr_nan_float1 = np.take(arr_nan_float1, 0, axis=-1)
-        targ0 = np.take(targ0, 0, axis=-1)
-
-
 @pytest.mark.parametrize(
     "arr, correct",
     [
@@ -799,7 +755,6 @@ def test_nan_comparison(request, op, nanop, disable_bottleneck):
         ("arr_bool", False),
         ("arr_str", False),
         ("arr_utf", False),
-        ("arr_complex", False),
         ("arr_complex_nan", False),
         ("arr_nan_nanj", False),
         ("arr_nan_infj", True),
@@ -898,7 +853,9 @@ class TestEnsureNumeric:
 
         # Test convertible string ndarray
         s_values = np.array(["1", "2", "3"], dtype=object)
-        assert np.allclose(nanops._ensure_numeric(s_values), values)
+        msg = r"Could not convert \['1' '2' '3'\] to numeric"
+        with pytest.raises(TypeError, match=msg):
+            nanops._ensure_numeric(s_values)
 
         # Test non-convertible string ndarray
         s_values = np.array(["foo", "bar", "baz"], dtype=object)
@@ -907,12 +864,19 @@ class TestEnsureNumeric:
             nanops._ensure_numeric(s_values)
 
     def test_convertable_values(self):
-        assert np.allclose(nanops._ensure_numeric("1"), 1.0)
-        assert np.allclose(nanops._ensure_numeric("1.1"), 1.1)
-        assert np.allclose(nanops._ensure_numeric("1+1j"), 1 + 1j)
+        with pytest.raises(TypeError, match="Could not convert string '1' to numeric"):
+            nanops._ensure_numeric("1")
+        with pytest.raises(
+            TypeError, match="Could not convert string '1.1' to numeric"
+        ):
+            nanops._ensure_numeric("1.1")
+        with pytest.raises(
+            TypeError, match=r"Could not convert string '1\+1j' to numeric"
+        ):
+            nanops._ensure_numeric("1+1j")
 
     def test_non_convertable_values(self):
-        msg = "Could not convert foo to numeric"
+        msg = "Could not convert string 'foo' to numeric"
         with pytest.raises(TypeError, match=msg):
             nanops._ensure_numeric("foo")
 
@@ -1038,7 +1002,7 @@ class TestNanvarFixedValues:
 
     @property
     def prng(self):
-        return np.random.RandomState(1234)
+        return np.random.default_rng(2)
 
 
 class TestNanskewFixedValues:
@@ -1087,9 +1051,26 @@ class TestNanskewFixedValues:
         skew = nanops.nanskew(samples, skipna=True)
         tm.assert_almost_equal(skew, actual_skew)
 
+    @pytest.mark.parametrize(
+        "initial_data, nobs",
+        [
+            ([-2.05191341e-05, -4.10391103e-05], 27),
+            ([-2.05191341e-10, -4.10391103e-10], 27),
+            ([-2.05191341e-05, -4.10391103e-05], 10_000),
+            ([-2.05191341e-10, -4.10391103e-10], 10_000),
+        ],
+    )
+    def test_low_variance(self, initial_data, nobs):
+        st = pytest.importorskip("scipy.stats")
+        data = np.zeros((nobs,), dtype=np.float64)
+        data[: len(initial_data)] = initial_data
+        skew = nanops.nanskew(data)
+        expected = st.skew(data, bias=False)
+        tm.assert_almost_equal(skew, expected)
+
     @property
     def prng(self):
-        return np.random.RandomState(1234)
+        return np.random.default_rng(2)
 
 
 class TestNankurtFixedValues:
@@ -1108,16 +1089,16 @@ class TestNankurtFixedValues:
         # xref GH 11974
         data = val * np.ones(300)
         kurt = nanops.nankurt(data)
-        assert kurt == 0.0
+        tm.assert_equal(kurt, 0.0)
 
     def test_all_finite(self):
         alpha, beta = 0.3, 0.1
         left_tailed = self.prng.beta(alpha, beta, size=100)
-        assert nanops.nankurt(left_tailed) < 0
+        assert nanops.nankurt(left_tailed) < 2
 
         alpha, beta = 0.1, 0.3
         right_tailed = self.prng.beta(alpha, beta, size=100)
-        assert nanops.nankurt(right_tailed) > 0
+        assert nanops.nankurt(right_tailed) < 0
 
     def test_ground_truth(self, samples, actual_kurt):
         kurt = nanops.nankurt(samples)
@@ -1138,29 +1119,43 @@ class TestNankurtFixedValues:
         kurt = nanops.nankurt(samples, skipna=True)
         tm.assert_almost_equal(kurt, actual_kurt)
 
+    @pytest.mark.parametrize(
+        "initial_data, nobs",
+        [
+            ([-2.05191341e-05, -4.10391103e-05], 27),
+            ([-2.05191341e-10, -4.10391103e-10], 27),
+            ([-2.05191341e-05, -4.10391103e-05], 10_000),
+            ([-2.05191341e-10, -4.10391103e-10], 10_000),
+        ],
+    )
+    def test_low_variance(self, initial_data, nobs):
+        # GH#57972
+        st = pytest.importorskip("scipy.stats")
+        data = np.zeros((nobs,), dtype=np.float64)
+        data[: len(initial_data)] = initial_data
+        kurt = nanops.nankurt(data)
+        expected = st.kurtosis(data, bias=False)
+        tm.assert_almost_equal(kurt, expected)
+
     @property
     def prng(self):
-        return np.random.RandomState(1234)
+        return np.random.default_rng(2)
 
 
 class TestDatetime64NaNOps:
-    @pytest.fixture(params=["s", "ms", "us", "ns"])
-    def unit(self, request):
-        return request.param
-
     # Enabling mean changes the behavior of DataFrame.mean
     # See https://github.com/pandas-dev/pandas/issues/24752
     def test_nanmean(self, unit):
         dti = pd.date_range("2016-01-01", periods=3).as_unit(unit)
         expected = dti[1]
 
-        for obj in [dti, DatetimeArray(dti), Series(dti)]:
+        for obj in [dti, dti._data]:
             result = nanops.nanmean(obj)
             assert result == expected
 
         dti2 = dti.insert(1, pd.NaT)
 
-        for obj in [dti2, DatetimeArray(dti2), Series(dti2)]:
+        for obj in [dti2, dti2._data]:
             result = nanops.nanmean(obj)
             assert result == expected
 
@@ -1238,8 +1233,8 @@ def test_nanops_independent_of_mask_param(operation):
     # GH22764
     ser = Series([1, 2, np.nan, 3, np.nan, 4])
     mask = ser.isna()
-    median_expected = operation(ser)
-    median_result = operation(ser, mask=mask)
+    median_expected = operation(ser._values)
+    median_result = operation(ser._values, mask=mask)
     assert median_expected == median_result
 
 
@@ -1279,17 +1274,20 @@ def test_check_bottleneck_disallow(any_real_numpy_dtype, func):
 
 
 @pytest.mark.parametrize("val", [2**55, -(2**55), 20150515061816532])
-def test_nanmean_overflow(disable_bottleneck, val):
+def test_nanmean_overflow(disable_bottleneck, val, using_python_scalars):
     # GH 10155
     # In the previous implementation mean can overflow for int dtypes, it
     # is now consistent with numpy
 
     ser = Series(val, index=range(500), dtype=np.int64)
     result = ser.mean()
-    np_result = ser.values.mean()
     assert result == val
-    assert result == np_result
-    assert result.dtype == np.float64
+    if using_python_scalars:
+        assert type(result) == float
+    else:
+        np_result = ser.values.mean()
+        assert result == np_result
+        assert result.dtype == np.float64
 
 
 @pytest.mark.parametrize(
@@ -1304,13 +1302,18 @@ def test_nanmean_overflow(disable_bottleneck, val):
     ],
 )
 @pytest.mark.parametrize("method", ["mean", "std", "var", "skew", "kurt", "min", "max"])
-def test_returned_dtype(disable_bottleneck, dtype, method):
+def test_returned_dtype(disable_bottleneck, dtype, method, using_python_scalars):
     if dtype is None:
         pytest.skip("np.float128 not available")
 
     ser = Series(range(10), dtype=dtype)
     result = getattr(ser, method)()
-    if is_integer_dtype(dtype) and method not in ["min", "max"]:
+    if using_python_scalars:
+        if is_integer_dtype(dtype) and method in ["min", "max"]:
+            assert isinstance(result, int)
+        else:
+            assert type(result) == float
+    elif is_integer_dtype(dtype) and method not in ["min", "max"]:
         assert result.dtype == np.float64
     else:
         assert result.dtype == dtype
