@@ -5,12 +5,8 @@ from itertools import product
 import numpy as np
 import pytest
 
-from pandas.compat import (
-    is_ci_environment,
-    is_platform_windows,
-)
-
 from pandas import (
+    NA,
     DataFrame,
     MultiIndex,
     Series,
@@ -22,7 +18,7 @@ import pandas._testing as tm
 from pandas.core.algorithms import safe_sort
 import pandas.core.common as com
 from pandas.core.sorting import (
-    decons_group_index,
+    _decons_group_index,
     get_group_index,
     is_int64_overflow_possible,
     lexsort_indexer,
@@ -33,14 +29,14 @@ from pandas.core.sorting import (
 @pytest.fixture
 def left_right():
     low, high, n = -1 << 10, 1 << 10, 1 << 20
-    left = DataFrame(np.random.randint(low, high, (n, 7)), columns=list("ABCDEFG"))
+    left = DataFrame(
+        np.random.default_rng(2).integers(low, high, (n, 7)), columns=list("ABCDEFG")
+    )
     left["left"] = left.sum(axis=1)
-
-    # one-2-one match
-    i = np.random.permutation(len(left))
-    right = left.iloc[i].copy()
-    right.columns = right.columns[:-1].tolist() + ["right"]
-    right.index = np.arange(len(right))
+    right = left.sample(
+        frac=1, random_state=np.random.default_rng(2), ignore_index=True
+    )
+    right.columns = [*right.columns[:-1].tolist(), "right"]
     right["right"] *= -1
     return left, right
 
@@ -60,7 +56,7 @@ class TestSorting:
                 "F": B,
                 "G": A,
                 "H": B,
-                "values": np.random.randn(2500),
+                "values": np.random.default_rng(2).standard_normal(2500),
             }
         )
 
@@ -93,34 +89,34 @@ class TestSorting:
         grouped = data.groupby(["a", "b", "c", "d"])
         assert len(grouped) == len(values)
 
+    @pytest.mark.slow
     @pytest.mark.parametrize("agg", ["mean", "median"])
     def test_int64_overflow_groupby_large_df_shuffled(self, agg):
-        arr = np.random.randint(-1 << 12, 1 << 12, (1 << 15, 5))
-        i = np.random.choice(len(arr), len(arr) * 4)
+        rs = np.random.default_rng(2)
+        arr = rs.integers(-1 << 12, 1 << 12, (1 << 15, 5))
+        i = rs.choice(len(arr), len(arr) * 4)
         arr = np.vstack((arr, arr[i]))  # add some duplicate rows
 
-        i = np.random.permutation(len(arr))
+        i = rs.permutation(len(arr))
         arr = arr[i]  # shuffle rows
 
         df = DataFrame(arr, columns=list("abcde"))
-        df["jim"], df["joe"] = np.random.randn(2, len(df)) * 10
+        df["jim"], df["joe"] = np.zeros((2, len(df)))
         gr = df.groupby(list("abcde"))
 
         # verify this is testing what it is supposed to test!
-        assert is_int64_overflow_possible(gr.grouper.shape)
+        assert is_int64_overflow_possible(
+            tuple(ping.ngroups for ping in gr._grouper.groupings)
+        )
 
-        # manually compute groupings
-        jim, joe = defaultdict(list), defaultdict(list)
-        for key, a, b in zip(map(tuple, arr), df["jim"], df["joe"]):
-            jim[key].append(a)
-            joe[key].append(b)
+        mi = MultiIndex.from_arrays(
+            [ar.ravel() for ar in np.array_split(np.unique(arr, axis=0), 5, axis=1)],
+            names=list("abcde"),
+        )
 
-        assert len(gr) == len(jim)
-        mi = MultiIndex.from_tuples(jim.keys(), names=list("abcde"))
-
-        f = lambda a: np.fromiter(map(getattr(np, agg), a), dtype="f8")
-        arr = np.vstack((f(jim.values()), f(joe.values()))).T
-        res = DataFrame(arr, columns=["jim", "joe"], index=mi).sort_index()
+        res = DataFrame(
+            np.zeros((len(mi), 2)), columns=["jim", "joe"], index=mi
+        ).sort_index()
 
         tm.assert_frame_equal(getattr(gr, agg)(), res)
 
@@ -155,61 +151,33 @@ class TestSorting:
         tm.assert_numpy_array_equal(result, np.array(exp, dtype=np.intp))
 
     @pytest.mark.parametrize(
-        "ascending, na_position, exp, box",
+        "ascending, na_position, exp",
         [
             [
                 True,
                 "last",
                 list(range(5, 105)) + list(range(5)) + list(range(105, 110)),
-                list,
             ],
             [
                 True,
                 "first",
                 list(range(5)) + list(range(105, 110)) + list(range(5, 105)),
-                list,
             ],
             [
                 False,
                 "last",
                 list(range(104, 4, -1)) + list(range(5)) + list(range(105, 110)),
-                list,
             ],
             [
                 False,
                 "first",
                 list(range(5)) + list(range(105, 110)) + list(range(104, 4, -1)),
-                list,
-            ],
-            [
-                True,
-                "last",
-                list(range(5, 105)) + list(range(5)) + list(range(105, 110)),
-                lambda x: np.array(x, dtype="O"),
-            ],
-            [
-                True,
-                "first",
-                list(range(5)) + list(range(105, 110)) + list(range(5, 105)),
-                lambda x: np.array(x, dtype="O"),
-            ],
-            [
-                False,
-                "last",
-                list(range(104, 4, -1)) + list(range(5)) + list(range(105, 110)),
-                lambda x: np.array(x, dtype="O"),
-            ],
-            [
-                False,
-                "first",
-                list(range(5)) + list(range(105, 110)) + list(range(104, 4, -1)),
-                lambda x: np.array(x, dtype="O"),
             ],
         ],
     )
-    def test_nargsort(self, ascending, na_position, exp, box):
+    def test_nargsort(self, ascending, na_position, exp):
         # list places NaNs last, np.array(..., dtype="O") may not place NaNs first
-        items = box([np.nan] * 5 + list(range(100)) + [np.nan] * 5)
+        items = np.array([np.nan] * 5 + list(range(100)) + [np.nan] * 5, dtype="O")
 
         # mergesort is the most difficult to get right because we want it to be
         # stable.
@@ -228,8 +196,14 @@ class TestSorting:
 class TestMerge:
     def test_int64_overflow_outer_merge(self):
         # #2690, combinatorial explosion
-        df1 = DataFrame(np.random.randn(1000, 7), columns=list("ABCDEF") + ["G1"])
-        df2 = DataFrame(np.random.randn(1000, 7), columns=list("ABCDEF") + ["G2"])
+        df1 = DataFrame(
+            np.random.default_rng(2).standard_normal((1000, 7)),
+            columns=[*list("ABCDEF"), "G1"],
+        )
+        df2 = DataFrame(
+            np.random.default_rng(3).standard_normal((1000, 7)),
+            columns=[*list("ABCDEF"), "G2"],
+        )
         result = merge(df1, df2, how="outer")
         assert len(result) == 2000
 
@@ -245,14 +219,12 @@ class TestMerge:
         assert result.name is None
 
     @pytest.mark.slow
-    @pytest.mark.parametrize("how", ["left", "right", "outer", "inner"])
-    def test_int64_overflow_how_merge(self, left_right, how):
+    def test_int64_overflow_how_merge(self, left_right, join_type):
         left, right = left_right
 
         out = merge(left, right, how="outer")
         out.sort_values(out.columns.tolist(), inplace=True)
-        out.index = np.arange(len(out))
-        tm.assert_frame_equal(out, merge(left, right, how=how, sort=True))
+        tm.assert_frame_equal(out, merge(left, right, how=join_type, sort=True))
 
     @pytest.mark.slow
     def test_int64_overflow_sort_false_order(self, left_right):
@@ -266,13 +238,12 @@ class TestMerge:
         tm.assert_frame_equal(right, out[right.columns.tolist()])
 
     @pytest.mark.slow
-    @pytest.mark.parametrize("how", ["left", "right", "outer", "inner"])
-    @pytest.mark.parametrize("sort", [True, False])
-    def test_int64_overflow_one_to_many_none_match(self, how, sort):
+    def test_int64_overflow_one_to_many_none_match(self, join_type, sort):
         # one-2-many/none match
+        how = join_type
         low, high, n = -1 << 10, 1 << 10, 1 << 11
         left = DataFrame(
-            np.random.randint(low, high, (n, 7)).astype("int64"),
+            np.random.default_rng(2).integers(low, high, (n, 7)).astype("int64"),
             columns=list("ABCDEFG"),
         )
 
@@ -284,25 +255,24 @@ class TestMerge:
         left = concat([left, left], ignore_index=True)
 
         right = DataFrame(
-            np.random.randint(low, high, (n // 2, 7)).astype("int64"),
+            np.random.default_rng(3).integers(low, high, (n // 2, 7)).astype("int64"),
             columns=list("ABCDEFG"),
         )
 
         # add duplicates & overlap with left to the right frame
-        i = np.random.choice(len(left), n)
+        i = np.random.default_rng(4).choice(len(left), n)
         right = concat([right, right, left.iloc[i]], ignore_index=True)
 
-        left["left"] = np.random.randn(len(left))
-        right["right"] = np.random.randn(len(right))
+        left["left"] = np.random.default_rng(2).standard_normal(len(left))
+        right["right"] = np.random.default_rng(2).standard_normal(len(right))
 
         # shuffle left & right frames
-        i = np.random.permutation(len(left))
-        left = left.iloc[i].copy()
-        left.index = np.arange(len(left))
-
-        i = np.random.permutation(len(right))
-        right = right.iloc[i].copy()
-        right.index = np.arange(len(right))
+        left = left.sample(
+            frac=1, ignore_index=True, random_state=np.random.default_rng(5)
+        )
+        right = right.sample(
+            frac=1, ignore_index=True, random_state=np.random.default_rng(6)
+        )
 
         # manually compute outer merge
         ldict, rdict = defaultdict(list), defaultdict(list)
@@ -317,32 +287,14 @@ class TestMerge:
         for k, lval in ldict.items():
             rval = rdict.get(k, [np.nan])
             for lv, rv in product(lval, rval):
-                vals.append(
-                    k
-                    + (
-                        lv,
-                        rv,
-                    )
-                )
+                vals.append((*k, lv, rv))
 
         for k, rval in rdict.items():
             if k not in ldict:
-                for rv in rval:
-                    vals.append(
-                        k
-                        + (
-                            np.nan,
-                            rv,
-                        )
-                    )
+                vals.extend((*k, np.nan, rv) for rv in rval)
 
-        def align(df):
-            df = df.sort_values(df.columns.tolist())
-            df.index = np.arange(len(df))
-            return df
-
-        out = DataFrame(vals, columns=list("ABCDEFG") + ["left", "right"])
-        out = align(out)
+        out = DataFrame(vals, columns=[*list("ABCDEFG"), "left", "right"])
+        out = out.sort_values(out.columns.to_list(), ignore_index=True)
 
         jmask = {
             "left": out["left"].notna(),
@@ -352,19 +304,21 @@ class TestMerge:
         }
 
         mask = jmask[how]
-        frame = align(out[mask].copy())
+        frame = out[mask].sort_values(out.columns.to_list(), ignore_index=True)
         assert mask.all() ^ mask.any() or how == "outer"
 
         res = merge(left, right, how=how, sort=sort)
         if sort:
             kcols = list("ABCDEFG")
             tm.assert_frame_equal(
-                res[kcols].copy(), res[kcols].sort_values(kcols, kind="mergesort")
+                res[kcols], res[kcols].sort_values(kcols, kind="mergesort")
             )
 
         # as in GH9092 dtypes break with outer/right join
         # 2021-12-18: dtype does not break anymore
-        tm.assert_frame_equal(frame, align(res))
+        tm.assert_frame_equal(
+            frame, res.sort_values(res.columns.to_list(), ignore_index=True)
+        )
 
 
 @pytest.mark.parametrize(
@@ -389,9 +343,9 @@ class TestMerge:
 )
 def test_decons(codes_list, shape):
     group_index = get_group_index(codes_list, shape, sort=True, xnull=True)
-    codes_list2 = decons_group_index(group_index, shape)
+    codes_list2 = _decons_group_index(group_index, shape)
 
-    for a, b in zip(codes_list, codes_list2):
+    for a, b in zip(codes_list, codes_list2, strict=True):
         tm.assert_numpy_array_equal(a, b)
 
 
@@ -400,57 +354,57 @@ class TestSafeSort:
         "arg, exp",
         [
             [[3, 1, 2, 0, 4], [0, 1, 2, 3, 4]],
-            [list("baaacb"), np.array(list("aaabbc"), dtype=object)],
+            [
+                np.array(list("baaacb"), dtype=object),
+                np.array(list("aaabbc"), dtype=object),
+            ],
             [[], []],
         ],
     )
     def test_basic_sort(self, arg, exp):
-        result = safe_sort(arg)
+        result = safe_sort(np.array(arg))
         expected = np.array(exp)
         tm.assert_numpy_array_equal(result, expected)
 
     @pytest.mark.parametrize("verify", [True, False])
     @pytest.mark.parametrize(
-        "codes, exp_codes, na_sentinel",
+        "codes, exp_codes",
         [
-            [[0, 1, 1, 2, 3, 0, -1, 4], [3, 1, 1, 2, 0, 3, -1, 4], -1],
-            [[0, 1, 1, 2, 3, 0, 99, 4], [3, 1, 1, 2, 0, 3, 99, 4], 99],
-            [[], [], -1],
+            [[0, 1, 1, 2, 3, 0, -1, 4], [3, 1, 1, 2, 0, 3, -1, 4]],
+            [[], []],
         ],
     )
-    def test_codes(self, verify, codes, exp_codes, na_sentinel):
-        values = [3, 1, 2, 0, 4]
+    def test_codes(self, verify, codes, exp_codes):
+        values = np.array([3, 1, 2, 0, 4])
         expected = np.array([0, 1, 2, 3, 4])
 
         result, result_codes = safe_sort(
-            values, codes, na_sentinel=na_sentinel, verify=verify
+            values, codes, use_na_sentinel=True, verify=verify
         )
         expected_codes = np.array(exp_codes, dtype=np.intp)
         tm.assert_numpy_array_equal(result, expected)
         tm.assert_numpy_array_equal(result_codes, expected_codes)
 
-    @pytest.mark.skipif(
-        is_platform_windows() and is_ci_environment(),
-        reason="In CI environment can crash thread with: "
-        "Windows fatal exception: access violation",
-    )
-    @pytest.mark.parametrize("na_sentinel", [-1, 99])
-    def test_codes_out_of_bound(self, na_sentinel):
-        values = [3, 1, 2, 0, 4]
+    def test_codes_out_of_bound(self):
+        values = np.array([3, 1, 2, 0, 4])
         expected = np.array([0, 1, 2, 3, 4])
 
         # out of bound indices
         codes = [0, 101, 102, 2, 3, 0, 99, 4]
-        result, result_codes = safe_sort(values, codes, na_sentinel=na_sentinel)
-        expected_codes = np.array(
-            [3, na_sentinel, na_sentinel, 2, 0, 3, na_sentinel, 4], dtype=np.intp
-        )
+        result, result_codes = safe_sort(values, codes, use_na_sentinel=True)
+        expected_codes = np.array([3, -1, -1, 2, 0, 3, -1, 4], dtype=np.intp)
         tm.assert_numpy_array_equal(result, expected)
         tm.assert_numpy_array_equal(result_codes, expected_codes)
 
-    @pytest.mark.parametrize("box", [lambda x: np.array(x, dtype=object), list])
-    def test_mixed_integer(self, box):
-        values = box(["b", 1, 0, "a", 0, "b"])
+    @pytest.mark.parametrize("codes", [[-1, -1], [2, -1], [2, 2]])
+    def test_codes_empty_array_out_of_bound(self, codes):
+        empty_values = np.array([])
+        expected_codes = -np.ones_like(codes, dtype=np.intp)
+        _, result_codes = safe_sort(empty_values, codes)
+        tm.assert_numpy_array_equal(result_codes, expected_codes)
+
+    def test_mixed_integer(self):
+        values = np.array(["b", 1, 0, "a", 0, "b"], dtype=object)
         result = safe_sort(values)
         expected = np.array([0, 0, 1, "a", "b", "b"], dtype=object)
         tm.assert_numpy_array_equal(result, expected)
@@ -474,9 +428,9 @@ class TestSafeSort:
     @pytest.mark.parametrize(
         "arg, codes, err, msg",
         [
-            [1, None, TypeError, "Only list-like objects are allowed"],
-            [[0, 1, 2], 1, TypeError, "Only list-like objects or None"],
-            [[0, 1, 2, 1], [0, 1], ValueError, "values should be unique"],
+            [1, None, TypeError, "Only np.ndarray, ExtensionArray, and Index"],
+            [np.array([0, 1, 2]), 1, TypeError, "Only list-like objects or None"],
+            [np.array([0, 1, 2, 1]), [0, 1], ValueError, "values should be unique"],
         ],
     )
     def test_exceptions(self, arg, codes, err, msg):
@@ -484,7 +438,7 @@ class TestSafeSort:
             safe_sort(values=arg, codes=codes)
 
     @pytest.mark.parametrize(
-        "arg, exp", [[[1, 3, 2], [1, 2, 3]], [[1, 3, np.nan, 2], [1, 2, 3, np.nan]]]
+        "arg, exp", [[[1, 3, 2], [1, 2, 3]], [[1, 3, NA, 2], [1, 2, 3, NA]]]
     )
     def test_extension_array(self, arg, exp):
         a = array(arg, dtype="Int64")
@@ -493,20 +447,29 @@ class TestSafeSort:
         tm.assert_extension_array_equal(result, expected)
 
     @pytest.mark.parametrize("verify", [True, False])
-    @pytest.mark.parametrize("na_sentinel", [-1, 99])
-    def test_extension_array_codes(self, verify, na_sentinel):
+    def test_extension_array_codes(self, verify):
         a = array([1, 3, 2], dtype="Int64")
-        result, codes = safe_sort(
-            a, [0, 1, na_sentinel, 2], na_sentinel=na_sentinel, verify=verify
-        )
+        result, codes = safe_sort(a, [0, 1, -1, 2], use_na_sentinel=True, verify=verify)
         expected_values = array([1, 2, 3], dtype="Int64")
-        expected_codes = np.array([0, 2, na_sentinel, 1], dtype=np.intp)
+        expected_codes = np.array([0, 2, -1, 1], dtype=np.intp)
         tm.assert_extension_array_equal(result, expected_values)
         tm.assert_numpy_array_equal(codes, expected_codes)
 
 
-def test_mixed_str_nan():
-    values = np.array(["b", np.nan, "a", "b"], dtype=object)
+def test_mixed_str_null(nulls_fixture):
+    values = np.array(["b", nulls_fixture, "a", "b"], dtype=object)
     result = safe_sort(values)
-    expected = np.array([np.nan, "a", "b", "b"], dtype=object)
+    expected = np.array(["a", "b", "b", nulls_fixture], dtype=object)
     tm.assert_numpy_array_equal(result, expected)
+
+
+def test_safe_sort_multiindex():
+    # GH#48412
+    arr1 = Series([2, 1, NA, NA], dtype="Int64")
+    arr2 = [2, 1, 3, 3]
+    midx = MultiIndex.from_arrays([arr1, arr2])
+    result = safe_sort(midx)
+    expected = MultiIndex.from_arrays(
+        [Series([1, 2, NA, NA], dtype="Int64"), [1, 2, 3, 3]]
+    )
+    tm.assert_index_equal(result, expected)
