@@ -1,3 +1,5 @@
+import numpy as np
+
 import pandas.util._test_decorators as td
 
 import pandas as pd
@@ -20,11 +22,25 @@ def test_shares_memory_string():
     # GH#55823
     import pyarrow as pa
 
-    obj = pd.array(["a", "b"], dtype="string[pyarrow]")
+    obj = pd.array(["a", "b"], dtype=pd.StringDtype("pyarrow", na_value=pd.NA))
     assert tm.shares_memory(obj, obj)
 
-    obj = pd.array(["a", "b"], dtype="string[pyarrow_numpy]")
+    obj = pd.array(["a", "b"], dtype=pd.StringDtype("pyarrow", na_value=np.nan))
     assert tm.shares_memory(obj, obj)
 
     obj = pd.array(["a", "b"], dtype=pd.ArrowDtype(pa.string()))
     assert tm.shares_memory(obj, obj)
+
+
+def test_shares_memory_numpy():
+    arr = np.arange(10)
+    view = arr[:5]
+    assert tm.shares_memory(arr, view)
+    arr2 = np.arange(10)
+    assert not tm.shares_memory(arr, arr2)
+
+
+def test_shares_memory_rangeindex():
+    idx = pd.RangeIndex(10)
+    arr = np.arange(10)
+    assert not tm.shares_memory(idx, arr)
